//-------------------------------------------------------------------
//
//                         agd_demo.c
//
// This demo program shows, how to use layout algorithms of the
// AGD-Library with LEDA's GraphWin.
//
//    authors: David Alberts, Carsten Gutwenger, and some more ...
//    date:   july 1997
//-------------------------------------------------------------------

#include "GraphWinAGD.h"
#include <AGD/GraphWinInterface.h>
#include <AGD/VisibilityMapper.h>

#include <AGD/DfsRanking.h>
#include <AGD/TopSortRanking.h>
#include <AGD/HierarchicalRanking.h>
#include <AGD/BarycenterHeuristic.h>
#include <AGD/MedianHeuristic.h>
#include <AGD/GreedyInsertHeuristic.h>
#include <AGD/GreedySwitchHeuristic.h>
#include <AGD/SplitHeuristic.h>


int agd_demo_main()
{
	GraphWinAGD *GW = new GraphWinAGD;

	GW->open();

	delete GW;

	return 0;
}


GraphWinAGD::GraphWinAGD()
{
	// set some  GraphWin options to appropriate values
#if __LEDA__ >= 361
	     set_zoom_objects(true);
#else
	set_constant_width (false);
#endif
	set_node_shape (rectangle_node);

	// create menues for AGD
	layout_menu     = gw_add_menu (*this, "AGD Layout");
	int general_layout_menu         = gw_add_menu (*this, "General", layout_menu);
	int planar_poly_layout_menu     = gw_add_menu (*this, "Planar Polyline", layout_menu);
	int planar_straight_layout_menu = gw_add_menu (*this, "Planar Straight-Line", layout_menu);
	animation_menu  = gw_add_menu (*this, "Animation");
	generators_menu = gw_add_menu (*this, "AGD Generators");
	options_menu    = gw_add_menu (*this, "AGD Options");
	graph_menu      = gw_add_menu (*this, "AGD Graph");
	augment_menu    = gw_add_menu (*this, "Augmentation", graph_menu);
	subgraph_menu   = gw_add_menu (*this, "Subgraph", graph_menu);

	// create menu entries for layout algorithms
	gw_add_simple_call (*this, callPlanarizationLayout, planarization.name(), general_layout_menu);
	gw_add_simple_call (*this, callSugiyamaLayout, sugiyama.name(), general_layout_menu);
	gw_add_simple_call (*this, callSpringLayout, spring.name(), general_layout_menu);
	gw_add_simple_call (*this, callTutteLayout, tutte.name(), general_layout_menu);

	gw_add_simple_call (*this, callPlanarStraightLayout, straight.name(),
		planar_straight_layout_menu);
	gw_add_simple_call (*this, callPlanarDrawLayout, planar_draw.name(),
		planar_straight_layout_menu);
	gw_add_simple_call (*this, callConvexLayout, convex.name(), planar_straight_layout_menu);
	gw_add_simple_call (*this, callConvexDrawLayout, convex_draw.name(),
		planar_straight_layout_menu);
	gw_add_simple_call (*this, callFppLayout, fpp.name(), planar_straight_layout_menu);
	gw_add_simple_call (*this, callSchnyderLayout, schnyder.name(),
		planar_straight_layout_menu);
	gw_add_simple_call (*this, callTreeLayout, tree_layout.name(), planar_straight_layout_menu);
	gw_add_simple_call (*this, callVisibilityRepresentation, visib_rep.name(),
		planar_straight_layout_menu);

	gw_add_simple_call (*this, callMixedModelLayout, mixed.name(), planar_poly_layout_menu);
	gw_add_simple_call (*this, callQuodLayout, quod.name(), planar_poly_layout_menu);
	gw_add_simple_call (*this, callOrthoLayout, ortho.name(), planar_poly_layout_menu);

	// create menu entries for graph generators
	gw_add_simple_call (*this, callRandomPlanarTricGraph, "planar & triconnected",
		generators_menu);
	gw_add_simple_call (*this, callRandomPlanarBicGraph, "planar & biconnected",
		generators_menu);
	gw_add_simple_call (*this, callRandomGraph, "random", generators_menu);
	gw_add_simple_call (*this, callRandomGraph2, "random (2)", generators_menu);
	gw_add_simple_call (*this, callRandomFourGraph, "four-graph", generators_menu);
	gw_add_simple_call (*this, callRandomHierarchy, "hierarchical", generators_menu);

	// create menu entries for augmentation and subgraph algorithms
	gw_add_simple_call (*this, callMakeBic, "biconnected", augment_menu);
	gw_add_simple_call (*this, callPlanAug, "planar biconnected", augment_menu);
	gw_add_simple_call (*this, callPlanSub, "planar", subgraph_menu);

	gw_add_simple_call (*this, callOptionsConvex,     "Convex",         options_menu);
	gw_add_simple_call (*this, callOptionsConvexDraw, "ConvexDraw",     options_menu);
	gw_add_simple_call (*this, callOptionsStraight,   "PlanarStraight", options_menu);
	gw_add_simple_call (*this, callOptionsPlanarDraw, "PlanarDraw",     options_menu);
	gw_add_simple_call (*this, callOptionsMixedModel, "Mixed-Model",    options_menu);
	gw_add_simple_call (*this, callOptionsQuod,       "Quod",           options_menu);
	gw_add_simple_call (*this, callOptionsSpring,     "Spring Embedder",options_menu);
	gw_add_simple_call (*this, callOptionsTreeLayout, "Tree Layout",    options_menu);
	gw_add_simple_call (*this, callOptionsPlanarization,"Planarization",options_menu );
	gw_add_simple_call (*this, callOptionsSugiyama,   "Sugiyama",       options_menu );

	gw_add_simple_call (*this, animateSpring, "Spring-Embedder", animation_menu);

	// set an appropriate grid coordinate mapper for grid algorithms
	MaxAllMapper mapper (5,5,5);
	VisibilityMapper vis_mapper (5,5,5,5,5);

	fpp        .set_coordinate_mapper (mapper);
	convex     .set_coordinate_mapper (mapper);
	convex_draw.set_coordinate_mapper (mapper);
	straight   .set_coordinate_mapper (mapper);
	planar_draw.set_coordinate_mapper (mapper);
	mixed      .set_coordinate_mapper (mapper);
	ortho      .set_coordinate_mapper (mapper);
	schnyder   .set_coordinate_mapper (mapper);	
	quod       .set_coordinate_mapper (mapper);
	planarization.set_coordinate_mapper (mapper);
	visib_rep  .set_coordinate_mapper (vis_mapper);

	tree_layout.sib_separation(10.0);
	tree_layout.subtree_separation(10.0);
	tree_layout.level_separation(10.0);

	fhl.node_distance(10.0);
	fhl.layer_distance(10.0);

	planarization.set_planar_layouter(quod);
	planarization_layouter = 0;
	planarization_planarizer = 0;
	sug1_ranking = 0;
	sug1_cross_min = 0;
}

//
// call a layout algorithm L
//

bool GraphWinAGD::callLayout (GraphWinAGD& GW, LayoutModule& L) {
	string msg;
	GraphWinInterface A (GW); // an interface for our GraphWin

	float t;
	// can L draw the graph displayed in GW ?
	if (L.check(GW.get_graph(), msg)) {
		used_time(t);
		L.call (GW.get_graph(), A); // yes, we can call L
		t = used_time(t);
		cout << "calling " << L.name() << " required " << t << " seconds.\n";
		cout.flush();
		GW.fill_window();
		return true;
	} else {
		GW.acknowledge (msg); // no, so we display the error message
	}

	return false;
}

bool GraphWinAGD::callGridLayout (GraphWinAGD& GW, GridLayoutModule& L)
{
	string msg;
	if (callLayout(GW, L)) {
		IRect ir = L.get_grid_area();
		string msg("grid size: %d * %d", ir.width(), ir.height());
		GW.message (msg);

		return true;
	
	} else {
		GW.acknowledge (msg); // no, so we display the error message
		return false;
	}
}

bool GraphWinAGD::callAugment (GraphWinAGD& GW, AugmentModule& A) {
	string msg;

	// can A be called for the graph displayed in GW ?
	if (A.check(GW.get_graph(), msg)) {

		list<edge> e_list;
		edge e;

		A.call (GW.get_graph(), e_list); // yes, we can call A

		GW.update_graph();

		GW.deselect_all_edges();
		forall (e, e_list)
			GW.select(e);

		string msg("%d edge(s) added.", A.num_added_edges());
		GW.message (msg);

		return true;

	} else {
		GW.acknowledge (msg); // no, so we display the error message
	}

	return false;
}


bool GraphWinAGD::callSubgraph (GraphWinAGD& GW, SubgraphModule& S)
{
	graph &G = GW.get_graph();
	string msg;

	// can S be called for the graph displayed in GW ?
	if (S.check(G, msg)) {

		S.call (G); // yes, we can call S
		GW.update_graph();

		string msg("%d edge(s) deleted.", S.num_del_edges());
		GW.message (msg);

		return true;

	} else {
		GW.acknowledge (msg); // no, so we display the error message
	}

	return false;
}

void GraphWinAGD::optionsPlanarization()
{
	int layouter = planarization_layouter;

	panel P ("Options for algorithm Planarization");

	P.choice_item ("Layout:", layouter, "Orthogonal", "Mixed-Model");

	P.button ("Ok");
	int cancel = P.button ("Cancel");

	if (P.open (get_window()) != cancel) {
		planarization_layouter = layouter;
		switch (layouter) {
		case 0:
			planarization.set_planar_layouter(quod);
			break;
		case 1:
			planarization.set_planar_layouter(mixed);
			break;
		}
	}
}

void GraphWinAGD::optionsConvex()
{
	bool   size_opt   = convex.size_opt();
	double base_ratio = convex.base_ratio();

	panel P ("Options for algorithm Convex");

	P.bool_item ("size_opt:",   size_opt);
	P.real_item ("base_ratio:", base_ratio);

	P.button ("Ok");
	int cancel = P.button ("Cancel");

	if (P.open (get_window()) != cancel) {
		convex.size_opt   (size_opt);
		convex.base_ratio (base_ratio);
	}
}

void GraphWinAGD::optionsConvexDraw()
{
	bool   size_opt   = convex_draw.size_opt();
	bool   side_opt   = convex_draw.side_opt();
	double base_ratio = convex_draw.base_ratio();

	panel P ("Options for algorithm ConvexDraw");

	P.bool_item ("size_opt:",   size_opt);
	P.bool_item ("side_opt:",   side_opt);
	P.real_item ("base_ratio:", base_ratio);

	P.button ("Ok");
	int cancel = P.button ("Cancel");

	if (P.open (get_window()) != cancel) {
		convex_draw.size_opt   (size_opt);
		convex_draw.side_opt   (side_opt);
		convex_draw.base_ratio (base_ratio);
	}
}


void GraphWinAGD::optionsStraight()
{
	bool   size_opt   = straight.size_opt();
	bool   max_outer  = straight.max_outer();
	double base_ratio = straight.base_ratio();

	panel P ("Options for algorithm PlanarStraight");

	P.bool_item ("size_opt:",   size_opt);
	P.bool_item ("max_outer:",  max_outer);
	P.real_item ("base_ratio:", base_ratio);

	P.button ("Ok");
	int cancel = P.button ("Cancel");

	if (P.open (get_window()) != cancel) {
		straight.size_opt   (size_opt);
		straight.max_outer  (max_outer);
		straight.base_ratio (base_ratio);
	}
}

void GraphWinAGD::optionsPlanarDraw()
{
	bool   size_opt   = planar_draw.size_opt();
	bool   side_opt   = planar_draw.side_opt();
	bool   max_outer  = planar_draw.max_outer();
	double base_ratio = planar_draw.base_ratio();

	panel P ("Options for algorithm PlanarDraw");

	P.bool_item ("size_opt:",   size_opt);
	P.bool_item ("side_opt:",   side_opt);
	P.bool_item ("max_outer:",  max_outer);
	P.real_item ("base_ratio:", base_ratio);

	P.button ("Ok");
	int cancel = P.button ("Cancel");

	if (P.open (get_window()) != cancel) {
		planar_draw.size_opt   (size_opt);
		planar_draw.side_opt   (side_opt);
		planar_draw.max_outer  (max_outer);
		planar_draw.base_ratio (base_ratio);
	}
}

void GraphWinAGD::optionsMixedModel()
{
	int co_type            = (int)mixed.co_type();
	bool prefer_nodes      = mixed.prefer_nodes();
	double base_ratio      = mixed.base_ratio();
	bool do_postprocessing = mixed.do_postprocessing();

	panel P ("Options for algorithm Mixed-Model");

	P.choice_item ("co_type:", co_type, "triconnected", "biconnected");
	P.bool_item ("prefer_nodes:",  prefer_nodes);
	P.real_item ("base_ratio:", base_ratio);
	P.bool_item ("do_postprocessing:", do_postprocessing);

	P.button ("Ok");
	int cancel = P.button ("Cancel");

	if (P.open (get_window()) != cancel) {
		mixed.co_type           ((CanonicalOrderType)(co_type));
		mixed.prefer_nodes      (prefer_nodes);
		mixed.base_ratio        (base_ratio);
		mixed.do_postprocessing (do_postprocessing);
	}
}

void GraphWinAGD::optionsQuod()
{
	bool dissect = quod.dissect();
	bool verify  = quod.verify();

	panel P ("Options for algorithm Mixed-Model");

	P.bool_item ("dissect:",  dissect);
	P.bool_item ("verify:",  verify);

	P.button ("Ok");
	int cancel = P.button ("Cancel");

	if (P.open (get_window()) != cancel) {
		quod.dissect(dissect);
		quod.verify (verify);
	}
}

void GraphWinAGD::optionsSugiyama()
{
	int ranking = sug1_ranking;
	int cross_min = sug1_cross_min;
	int runs = sugiyama.runs();
	double node_dist = fhl.node_distance();
	double level_dist = fhl.layer_distance();
	bool fixed_level_dist = fhl.fixed_layer_distance();

	panel P ("Options for algorithm Sugiyama");

	P.choice_item ("ranking:", ranking, "Hierarchical", "DFS", "TopSort");
	P.choice_item ("heuristic:", cross_min, "Barycenter", "Median", "Greedy Insert", "Greedy Switch", "Split");
	P.int_item ("runs:", runs);
	P.real_item ("node distance:", node_dist);
	P.real_item ("level distance:", level_dist);
	P.bool_item ("fixed level distance:", fixed_level_dist);

	P.button ("Ok");
	int cancel = P.button ("Cancel");

	if (P.open (get_window()) != cancel) {
		sugiyama.runs(runs);
		fhl.node_distance(node_dist);
		fhl.layer_distance(level_dist);
		fhl.fixed_layer_distance(fixed_level_dist);

		sugiyama.set_comp_coord(fhl);

		if (ranking != sug1_ranking) {
			sug1_ranking = ranking;
			switch (ranking) {
			case 0:
				sugiyama.set_ranking(HierarchicalRanking());
				break;
			case 1:
				sugiyama.set_ranking(DfsRanking());
				break;
			case 2:
				sugiyama.set_ranking(TopSortRanking());
				break;
			}
		}
		if (cross_min != sug1_cross_min) {
			sug1_cross_min = cross_min;
			switch (cross_min) {
			case 0:
				sugiyama.set_cross_min(BarycenterHeuristic());
				break;
			case 1:
				sugiyama.set_cross_min(MedianHeuristic());
				break;
			case 2:
				sugiyama.set_cross_min(GreedyInsertHeuristic());
				break;
			case 3:
				sugiyama.set_cross_min(GreedySwitchHeuristic());
				break;
			case 4:
				sugiyama.set_cross_min(SplitHeuristic());
				break;
			}
		}
	}
}

void GraphWinAGD::optionsSpring()
{
	bool   noise = spring.noise();
	int    iter  = spring.iterations();

	panel P ("Options for algorithm Spring Embedder");

	P.bool_item ("noise:", noise);
	P.int_item  ("iterations:", iter);

	P.button ("Ok");
	int cancel = P.button ("Cancel");

	if (P.open (get_window()) != cancel) {
		spring.noise (noise);
		spring.iterations (iter);
	}
}


void GraphWinAGD::optionsTreeLayout()
{
	double sib_separation     = tree_layout.sib_separation();
	double level_separation   = tree_layout.level_separation();
	double subtree_separation = tree_layout.subtree_separation();

	panel P ("Options for algorithm TreeLayout");

	P.double_item ("sib_separation:", sib_separation);
	P.double_item ("level_separation:", level_separation);
	P.double_item ("subtree_separation:", subtree_separation);

	P.button ("Ok");
	int cancel = P.button ("Cancel");

	if (P.open (get_window()) != cancel) {
		tree_layout.sib_separation(sib_separation);
		tree_layout.level_separation(level_separation);
		tree_layout.subtree_separation(subtree_separation);
	}
}

/*
void GraphWinAGD::callSugiyamaLayout( GraphWin &GW )
{
	GraphWinAGD &gwAGD = (GraphWinAGD&)GW;
	SugiyamaLayout &sugi = gwAGD.sugiyama;

	if (callGridLayout (gwAGD, gwAGD.sugiyama) && gwAGD.sugiyama_infos) {
		panel results ("Results");

		results.text_item ("Used heuristic: ");
		results.text_item (sugi.name_of_heuristic (sugi.used_heuristic()));
		results.text_item ("");
		results.text_item ("Number of crossings: ");
		results.text_item (sugi_to_string(sugi.number_of_crossings()));
		results.text_item ("");
		results.text_item ("Number of runs: ");
		results.text_item (sugi_to_string(sugi.number_of_runs()));
		results.button ("OK");
		results.open (GW.get_window());
	}
}*/

void GraphWinAGD::random_bic_graph (graph& G, int num_nodes)
{
	random_tric_graph (G, num_nodes, 2*num_nodes);
	int i, n = num_nodes/2;

	for (i = 1; i <= n; i++) {
		edge e = G.choose_edge();
		G.del_edge(e);
	}

	Make_Biconnected (G);
}

int GraphWinAGD::anim_status;
bool GraphWinAGD::rh_planar, GraphWinAGD::rh_single;

void GraphWinAGD::wait (GraphWinAGD& GW, panel& P, float delay, int QUIT)
{
	float T1, T2;
	int val;
	double x, y;

	used_time (T1);
	do {
		T2 = T1;
		P.get_event(val,x,y);
	} while (used_time(T2) < delay && GW.anim_status != QUIT);
}

void GraphWinAGD::anim_action (int n)
{
	anim_status = n;
}
/*
void GraphWinAGD::show_results(const AGDModule& M, window& results)
{
	if (&M == &sugiyama && sugiyama_infos) {
		results.del_messages();
		results.message ("Used heuristic: " +
			sugiyama.name_of_heuristic(sugiyama.used_heuristic()));
		results.message ("Number of crossings: " +
			sugi_to_string(sugiyama.number_of_crossings()));
		results.message ("Number of runs: " +
			sugi_to_string(sugiyama.number_of_runs()));
		results.message( "Next fixed layer: " +
			sugi_to_string(sugiyama.fixed_layer()));
		results.message("Next changeable layer: " +
			sugi_to_string(sugiyama.changeable_layer()));
	}
}*/

void GraphWinAGD::animate(GraphWinAGD& GW, LayoutModule& L)
{
	//GraphWinAGD &gwAGD = (GraphWinAGD&)GW;
	//SugiyamaLayout &sugi = GW.sugiyama;
	graph& G = GW.get_graph();
	GraphWinInterface gwi(GW);

	string msg;
	if (L.check(G,msg) == false) {
		GW.acknowledge(msg);
		return;
	}

	// the graph may not get changed during animation!
	GW.disable_calls();

	// first call start_animation, if false is returned,
	// animation is not possible
	if (L.start_animation(G,gwi) == true) {

		// create a control panel for animation
		panel P("Animation Control Panel");

		P.buttons_per_line (5);
		const int
			SINGLE = P.button ("SINGLE", &anim_action),
			PLAY   = P.button ("PLAY"  , &anim_action),
			FAST   = P.button ("FAST"  , &anim_action),
			STOP   = P.button ("STOP"  , &anim_action),
			QUIT   = P.button ("QUIT"  , &anim_action);
		const float delay = 0.5f, delay_fast = 0.15f;

		P.display();

		// perform animation until the user presses the Quit button,
		// or no more steps are possible
		bool cont = true;
		anim_status = STOP;
		int val;
		double x, y;

		window results(200.0f, 100.0f, "Results");
/*		if (GW.sugiyama_infos && &L == &GW.sugiyama) {
			results.display(window::max, window::max);
			GW.show_results(L,results);
		}*/

		while (cont && anim_status != QUIT) {
			while (P.get_event(val,x,y) != no_event) ;

			if (anim_status == SINGLE || anim_status == PLAY ||
				anim_status == FAST)
			{
				cont = L.next_step();
				fit_in_window (GW, gwi);
				//GW.show_results(L,results);

				if (anim_status == SINGLE)
					anim_status = STOP;
				else {
					wait (GW, P, (anim_status == PLAY) ?
						delay : delay_fast, QUIT);
				}
			}
		}

		// call stop animation only if animation is still possible
		if (cont) {
		  L.stop_animation();
		  fit_in_window( GW, gwi ); // MiK
		}
		if (cont == false && anim_status != QUIT)
			GW.acknowledge ("Animation is finished.");
	}

	// now, we can enable calls again
	GW.enable_calls();
}

void GraphWinAGD::fit_in_window (GraphWinAGD& GW, LayoutInterface& A)
{
	const double eps = 0.000001;

	double xmin = GW.get_xmin();
	double xmax = GW.get_xmax();
	double ymin = GW.get_ymin();
	double ymax = GW.get_ymax();

	DRect bbox(A.area());

	//A.get_layout_size (bbox);

	if (bbox.left() >= xmin-eps)
		if (bbox.right() <= xmax+eps)
			if (-bbox.bottom() >= ymin-eps)
				if (-bbox.top() <= ymax+eps)
					return;

	double r = (xmax-xmin) / (ymax-ymin);

	double h = bbox.bottom() - bbox.top();

	double xm = Max (bbox.right(), h*r+bbox.left());

	GW.win_init (bbox.left(), xm, -bbox.bottom());
}

void GraphWinAGD::make_simple (graph& G)
{
	Delete_Loops (G);
	Make_Simple (G);

	edge_array<edge> rev(G);
	Is_Bidirected (G, rev);
	list<edge> del_edges;
	edge e;

	forall_edges (e, G) {
		if (rev[e] != nil) {
			del_edges.append(e);
			rev[rev[e]] = nil;
		}
	}

	forall (e, del_edges) G.del_edge(e);
}

void GraphWinAGD::random_graph_2 (graph& G, int num_nodes, int num_edges)
{
	random_graph (G, num_nodes, num_edges);

	make_simple(G);
}

void GraphWinAGD::random_graph_3 (graph& G, int num_nodes, int num_edges)
{
	random_planar_graph (G, num_nodes, num_edges);

	int i, m = num_edges/10;

	for (i = 1; i <= m; i++)
		G.new_edge(G.choose_node(),G.choose_node());

	make_simple(G);
}

void GraphWinAGD::set_N(node v, bool b, const graph& G, node_array<bool>& N)
{
	edge e;

	forall_inout_edges(e,v)
		N[G.opposite(v,e)] = b;
}

void GraphWinAGD::random_four_graph (graph& G, int n, int m)
{
	G.clear();

	array<node> T(n);
	node_array<bool> N(G,n,false);

	int i, j, k;
	random_source S;

	for (i = 0; i < n; i++)
		T[i] = G.new_node();

	for (k = 1; n >= 1 && k <= m; k++) {
		i = S(0,n-1);
		j = S(0,n-1);

		N[T[i]] = true;
		set_N(T[i],true,G,N);

		if (N[T[j]] == true) {
			int j1 = j;
			do {
				j = (j+1) % n;
			} while (N[T[j]] && j1!= j);
		}
		if (N[T[j]]) T[i] = T[--n];
		else {
			G.new_edge(T[i],T[j]);
			if (degree(T[i]) == 4) T[i] = T[--n];
			if (j >= n) j = i;
			if (degree(T[j]) == 4) T[j] = T[--n];
		}
		N[T[i]] = false;
		set_N(T[i],false,G,N);
	}
}

//////////////////////////////////////////////////////////////////////
// The following functions are copied from the GraphWin sources.
// Unfortunatly, there seems to be no simple concept to add new
// graph generators. Moreover, the functions used by GraphWin's
// generators are defined static and cannot be used by other modules!

void GraphWinAGD::embed_random(const graph& G, node_array<point>& dest) { 
	random_source ran;
	node v;
	double x,y;
	forall_nodes(v,G) {
		ran >> x >> y;
		dest[v]=point(x,y);
	}
}

void GraphWinAGD::stretch(const graph& G, 
				  node_array<double>& destx,
                  node_array<double>& desty,
                  double x0, double y0, double x1, double y1) {
	double xd=x1-x0;
	double yd=y1-y0;

	double f=20;

	x0+=xd/f;
	y0+=yd/f;

	  xd*=1.0-2.0/f;
	yd*=1.0-2.0/f;

	node v;
	forall_nodes(v,G) { 
		destx[v]=x0+destx[v]*xd;
		desty[v]=y0+desty[v]*yd;
	}
}

void GraphWinAGD::stretch(const graph& G, node_array<point>& dest,
                  double x0, double y0, double x1, double y1) {
 
	// stretches unit-square coordinates given in dest to window (x0,y0)(x1,y1)

	node_array<double> x(G),y(G);

	node v;
	forall_nodes(v,G) {
		x[v]=dest[v].xcoord();
		y[v]=dest[v].ycoord();
	}

	stretch(G,x,y,x0,y0,x1,y1);

	forall_nodes(v,G) dest[v]=point(x[v],y[v]);
}

void GraphWinAGD::callRandomHierarchy (GraphWin& gw)
{
	panel P;
	int num_nodes = gw.get_gen_nodes(),
		num_edges = gw.get_gen_edges();

	P.text_item ("Create Random Hierarchy");
	P.int_item ("nodes", num_nodes, 0, 200);
	P.int_item ("edges", num_edges, 0, 400);
	P.bool_item ("planar", rh_planar);
	P.bool_item ("single source", rh_single);
	P.button ("ok", 1);
	P.button ("cancel", 0);

	if (P.open(gw.get_window()) == 0) return;

	gw.set_gen_nodes (num_nodes);
	gw.set_gen_edges (num_edges);

	bool old_flush=gw.set_flush(false); 
	gw.clear_graph();
	graph& G=gw.get_graph();

	node_array<int> rank(G), pos(G);
	random_hierarchy(G,rank,pos,num_nodes,num_edges,rh_planar,rh_single);
	gw.update_graph();

	GraphWinAGD &gwAGD = (GraphWinAGD&)gw;
	string msg;
	GraphWinInterface A (gw);

	if (gwAGD.sugiyama.check(G, msg)) {
		gwAGD.sugiyama.ext_call (G, rank, A);
		gw.fill_window();
	}

	gw.set_flush(old_flush);
	gw.redraw();
}
/*
#include <fstream.h>
void GraphWinAGD::callRandomHierarchy (GraphWin& gw)
{
	int num_nodes,num_edges,_numberOfLayers;
	char  *filename = "MYCHECK";
	ifstream input(filename,ios::in);
	input >> num_nodes >> num_edges >> _numberOfLayers;

	bool old_flush=gw.set_flush(false); 
	gw.clear_graph();
	graph& G=gw.get_graph();

	// create new graph (by input file)
	// and compute grid coordinates

	node_map<int> x(G,0), y(G,0);  // holds grid layout
	edge_map<IPolyline> bends(G);

	int _source;
	int _sink, i;

	for(i=0;i < num_nodes;i++)
		G.new_node();

	array<node> nodes(num_nodes);
	array<int>  first(_numberOfLayers);


	//* Place nodes *

    node v;
    int  _vertex;
    int  _layer;
    for(i=0; i < _numberOfLayers;i++)
		first[i]=0;

    forall_nodes(v,G) {
		input >> _vertex >> _layer;
		nodes[_vertex] = v;
        x[nodes[_vertex]] =_layer;
        y[nodes[_vertex]] = first[_layer]++;
    }


	int _found = 0; 
	for (i = 1; i <= num_nodes; i++) {
		input >> _source;
		_found = 0;
		while (!_found) {
			input >> _sink;
			if (_sink != 0)
				G.new_edge(nodes[_source],nodes[_sink]);
			else 
				_found = 1;
		}
	}
	gw.update_graph();


	// assign layout

	GraphWinInterface A (gw);

	MaxAllMapper gcm(5,5,5);  // first arg is separation
	gcm.call(G,x,y,bends,IRect(),A);

	A.update();
}
*/
void GraphWinAGD::gen_random_graph(GraphWin& gw, void (*f)(graph&,int,int), string s) {
	panel P;
	int num_nodes = gw.get_gen_nodes(),
		num_edges = gw.get_gen_edges();

	P.text_item (s);
	P.int_item ("nodes", num_nodes, 0, 200);
	P.int_item ("edges", num_edges, 0, 400);
	P.button ("ok", 1);
	P.button ("cancel", 0);

	if (P.open(gw.get_window()) == 0) return;

	gw.set_gen_nodes (num_nodes);
	gw.set_gen_edges (num_edges);

	bool old_flush=gw.set_flush(false); 
	gw.clear_graph();
	graph& G=gw.get_graph();
	f (G, num_nodes, num_edges);
	gw.update_graph();
    node_array<point> dest(G);
	embed_random(G,dest);
	stretch(G,dest,gw.get_xmin(),gw.get_ymin(),gw.get_xmax(),gw.get_ymax());
	gw.set_position(dest);
	gw.embed_edges();
	gw.set_flush(old_flush);
	gw.redraw();
}

void GraphWinAGD::gen_random_graph(GraphWin& gw, void (*f)(graph&,int), string s) {
	panel P;
	int num_nodes = gw.get_gen_nodes();

	P.text_item (s);
	P.int_item ("nodes", num_nodes, 0, 200);
	P.button ("ok", 1);
	P.button ("cancel", 0);

	if (P.open(gw.get_window()) == 0) return;

	gw.set_gen_nodes (num_nodes);

	bool old_flush=gw.set_flush(false); 
	gw.clear_graph();
	graph& G=gw.get_graph();
	f (G, num_nodes);
	gw.update_graph();
    node_array<point> dest(G);
	embed_random(G,dest);
	stretch(G,dest,gw.get_xmin(),gw.get_ymin(),gw.get_xmax(),gw.get_ymax());
	gw.set_position(dest);
	gw.embed_edges();
	gw.set_flush(old_flush);
	gw.redraw();
}
