//-------------------------------------------------------------------
//
//                         AGDModule.h
//
//
//    author: Carsten Gutwenger
//    date:   july 1997
//-------------------------------------------------------------------


#ifndef _AGD_AGDMODULE_H
#define _AGD_AGDMODULE_H

#include <AGD/AGD.h>
#include <AGD/TaggedRule.h>
#include <AGD/KeySet.h>
#include <LEDA/array.h>


#ifdef _MSC_VER
istream& operator>> (istream& , leda_list<TaggedRule>& );
ostream& operator<< (ostream& , const leda_list<TaggedRule>& );
int compare (const leda_list<TaggedRule>&, const leda_list<TaggedRule>&);
#endif


/*{\Manpage {AGDModule} {} {AGD Modules} {M} }*/

/*{\Mbaseclasses

\begin{hierarchy}{6.5}{2.5}
\hdclass{0}{1.5}{5}{AGD}
\arhook{5}{2}{6}
\hdclass{1.5}{0}{5}{AGDModule}
\end{hierarchy}
}*/

class AGDModule : public AGD {

/*{\Mdefinition
The class |\Mname| is the base class for modules in the AGD--Library.
It provides the handling of pre- and postconditions, and defines
a uniform access to common informations about a module.

\headerline{The Type of a Module}
We can characterize the type of a module by defining the type of its
input $I_1,\ldots,I_n$, the type of its output $O_1,\ldots,O_m$, and
the associated functionality, e.g., augmentation or layout algorithm.
We express the different types of modules by abstract base classes
derived from |\Mname|, which define the input and output types,
and the functionality.

The input consists
of $n\geq0$ and the output of $m\geq0$ parameters of
an arbitrary abstract (not \CC) type. The call method of a particular
module uses appropriate classes to represent these types.
It is not necessary to represent each abstract type by exactly
one class, for example
\begin{itemize}
\item the type ``grid layout'' can be represented
by ``|node_map|\<|int|\>\& |x|, |node_map|\<|int|\>\& |y|,
|edge_map|\<|Ipolyline|\>\& |bends|'',
\item an input and output parameter ``graph'' can be represented
by a single |graph| reference.
\end{itemize}

\headerline{Precondition and Postcondition}

|\Mname| maintains a set |pre(i)| of properties for each input parameter
$I_i$, which must be satisfied by $I_i$ if the module is called. Properties
are expressed by key items (cf. class \ref{AGD Base}), and sets of
properties by the data type |KeySet|. We call
|pre(i)| the precondition of $I_i$, and $|pre|(1),\ldots,|pre|(n)$ the
precondition of the module.


%However, the input for |\Mvar| can satisfy more general preconditions |PRE(i)|,
%i.e., $|PRE(i)|\supseteq |pre(i)|$ holds. We call |PRE(i)| the guaranteed
%preconditions.

The postcondition of the output parameters are sets of properties that the output
will always satisfy. We allow that they depend
on the properties $P_1,\ldots,P_n$ that the input of the module satisfies. Clearly, these
properties must be more general than the required precondition, i.e., 
$P_i\supseteq |pre|(i)$ must hold for all $1\leq i\leq n$.
Therefore, we associate a function
$|post|(i):{\cal P}(|KEYS|)^n \to {\cal P}(|KEYS|)$ with each output parameter
$O_i$, where |KEYS| is the set of all properties.
We also reserve a special postcondition with index 0 for general properties
of the module, e.g., we declare that a layout algorithm supports step-by-step
animation by adding the key |animation| to this general postcondition.

The function |post(i)| is defined by a set of rules |post_rules(i)|, where each
rule is of the form
\[ (p_1,i_1) \wedge \ldots \wedge (p_k,i_k) \rightarrow p\mbox{ , }k\geq 0. \]
|\Mname| provides protected methods to insert rules into |post_rules(i)|, and
to remove rules from |post_rules(i)|. Rules in |post_rules(i)| can be accessed
via the item type |pr_item|, e.g., to remove a rule. The function defined by
|post_rules(i)| is given by
\begin{eqnarray*}
\lefteqn{|post(i)|(Q_1,\ldots,Q_n) =} \\
& & \{
p \mid \exists (p_1,i_1) \wedge \ldots \wedge (p_k,i_k) \to p \in |post_rules|(i)\mbox{ with }
p_j\in Q_{i_j}\forall 1\leq j\leq k\}
\end{eqnarray*}

Note, that the so defined functions are monotonous in the following way. If
$P_j\subseteq Q_j$ holds for all $1\leq j\leq n$, then
$|post(i)|(P_1,\ldots,P_n)\subseteq |post(i)|(Q_1,\ldots,Q_n)$ holds for each
$1\leq i\leq m$.

\headerline{Common Informations}
Every AGD module contains
\begin{itemize}
\item a name in a long (|long_name|) and a short (|name|) version.
\item an author (|author|).
\item the name of the author (|impl_author|), the version (|impl_version|),
and the date (|impl_date|) of the implementation.
\end{itemize}

The default implementations for |name()|, |author()|, |impl_author()|,
|impl_version()| and |impl_date()| return an empty string ``'' indicating
that this information is unknown. |long_name()| returns |name()| by default.

\bigskip
\headerline{Exchangeable Modules and Optional Parameters}
We recommend to provide a standard interface for exchangeable modules and
optional para\-meters. These recommendation is obeyed by the module
implementations in the library.

Exchangeable modules are implemented as module options (see~\ref{Module Options},
page~\pageref{Module Options}) with an additional method
set\_|name|(|type| |M|), which sets the module option
|name| to |M| if possible.

An optional parameter |name| of type |type| provides two methods
\begin{quote}\begin{tabular}{ll}
|void| & |name|(|type| |new_setting|) \\
|type| & |name|()
\end{tabular}\end{quote}
for setting and reading the current value of the option. The documentation
describes optional parameters in the definition section of its class in
the following way
\begin{itemize}
\item |type| {\bf name} = |initial_value|\\|description|
\end{itemize}
where |initial_value| is the default setting of the option.
}*/

public:

	static const int out_general;

	typedef list_item pr_item;

	/*{\Minitialization}*/

	AGDModule (int n, int m);
	/*{\Mcreate initializes a module with |n| input and |m| output parameters.
	Initializes |pre(i)| and |post(j)| with the empty set for all $1\leq i\leq n$,
	and $0\leq j\leq m$.}*/

	virtual ~AGDModule () { }

	/*{\Moperations 2 3.2}*/

	/*{\Mtext       
	\headerline{a) Queries on Preconditions and Postconditions}
	}*/

	// information about preconditions & properties of the algorithm

	int num_in_param () const {
		return _n;
	}
	/*{\Mop returns the number of input parameters $n$.}*/

	int num_out_param () const {
		return _m;
	}
	/*{\Mop returns the number of output parameters $m$.}*/

	const KeySet& precondition (int i) const {
#if !defined(LEDA_CHECKING_OFF)
		if (i < 1 || i > _n)
			error_handler(1, "AGDModule::preconditions(i): index out of range");
#endif
		return _preconditions (i);
	}
	/*{\Mop returns |pre(i)|. \precond $1\leq i\leq n$.}*/

	bool need_property (int i, const leda_string& p) const {
#if !defined(LEDA_CHECKING_OFF)
		if (i < 1 || i > _n)
			error_handler(1, "AGDModule::need precondition(i,p): index out of range");
#endif
		return _need_precondition (i, p);
	}
	/*{\Mop returns true iff $p\in |pre(i)|$. \precond $1\leq i\leq n$.}*/

	KeySet postcondition (int i) const {
		return postcondition (i, _pre);
	}
	/*{\Mop returns $|post|(i)(|pre|(1),\ldots,|pre|(n))$. \precond $0\leq i\leq m$.}*/

	KeySet postcondition (int i, const leda_array<KeySet>& PRE) const {
#if !defined(LEDA_CHECKING_OFF)
		if (i < 0 || i > _m)
			error_handler(1, "AGDModule::postcondition(i,PRE): index out of range");
		if (PRE.low() != 1 || PRE.high() != _n)
			error_handler(1, "AGDModule::postcondition(i,PRE): PRE has wrong bounds.");
#endif
		return _postconditions (i, PRE);
	}
	/*{\Mop returns $|post|(i)(|PRE|[1],\ldots,|PRE|[n])$. \precond $0\leq i\leq m$,
	|PRE|.low() = 1 and |PRE|.high() = n.}*/

	bool usable (const leda_array<KeySet>& PRE, const leda_array<KeySet>& POST) const {
#if !defined(LEDA_CHECKING_OFF)
		if (PRE.low() != 1 || PRE.high() != _n)
			error_handler(1, "AGDModule::usable(PRE,POST): PRE has wrong bounds.");
		if (POST.low() != 0 || POST.high() != _m)
			error_handler(1, "AGDModule::usable(PRE,POST): POST has wrong bounds.");
#endif
		return _usable (PRE, POST);
	}
	/*{\Mop returns |true| iff |\Mvar| can be used as module for guaranteed preconditions
	|PRE| and required postconditions |POST|, i.e., $|PRE|[i]\supseteq |pre|(i)\:\forall\,
	1\leq i\leq n$ and $|post|(j)(|PRE|)\subseteq |POST|[j]\:\forall\,0\leq j\leq m$.}*/

	virtual AGDModule *clone () const
	/*{\Moppure returns a pointer to a new instance with the same option
	settings as |\Mvar|. Must be overridden by module implementations.}*/
	= 0;

	/*{\Mtext
	\bigskip
	\headerline{b) Common Informations}
	}*/

	virtual leda_string name () const {
		return leda_string ("");
	}
	/*{\Mop returns the name of the algorithm.}*/

	virtual leda_string long_name () const {
		return name ();
	}
	/*{\Mop returns the name in an extended version.}*/

	virtual leda_string author () const {
		return leda_string ("");
	}
	/*{\Mop returns the author of the algorithm.}*/

	virtual leda_string impl_author () const {
		return leda_string ("");
	}
	/*{\Mop returns the author of the implementation.}*/

	virtual leda_string impl_version () const {
		return leda_string ("");
	}
	/*{\Mop returns the version of the implementation.}*/

	virtual leda_string impl_date () const {
		return leda_string ("");
	}
	/*{\Mop returns the date of the last change of the implementation.}*/


protected:
	/*{\Mtext       
	\bigskip
	\headerline{c) Protected Methods for Maintaining Preconditions} }*/

	agd_key add_precondition (const leda_string& p, int i = 1) {
#if !defined(LEDA_CHECKING_OFF)
		if (i < 1 || i > _n)
			error_handler(1, "AGDModule::add_precondition(p,i): index out of range");
#endif
		return _add_precondition (p,i);
	}
	/*{\Mop adds the key \<|p|\> to |pre(i)| and returns it.
	\precond $1\leq i\leq n$.}*/

	void add_precondition (agd_key k, int i = 1) {
#if !defined(LEDA_CHECKING_OFF)
		if (i < 1 || i > _n)
			error_handler(1, "AGDModule::add_precondition(k,i): index out of range");
#endif
		_add_precondition (k,i);
	}
	/*{\Mop adds the key |k| to |pre(i)|. \precond $1\leq i\leq n$.}*/

	void add_precondition (agd_key k, bool status, int i = 1) {
#if !defined(LEDA_CHECKING_OFF)
		if (i < 1 || i > _n)
			error_handler(1, "AGDModule::add_precondition(k,status,i): index out of range");
#endif
		_add_precondition (k,status,i);
	}
	/*{\Mop adds the key |k| to |pre(i)| if
	|status| = |true|, removes it otherwise. \precond $1\leq i\leq n$.}*/

	void add_preconditions (const leda_string& ps, char delim = ',', int i = 1) {
#if !defined(LEDA_CHECKING_OFF)
		if (i < 1 || i > _n)
			error_handler(1, "AGDModule::add_preconditions(ps,delim,i): index out of range");
#endif
		_add_preconditions (ps,delim,i);
	}
	/*{\Mop interprets |ps| as a list of strings seperated by |delim|, and adds
	the key for each string to |pre(i)|. \precond $1\leq i\leq n$.}*/

	void rem_precondition (agd_key k, int i = 1) {
#if !defined(LEDA_CHECKING_OFF)
		if (i < 1 || i > _n)
			error_handler(1, "AGDModule::rem_precondition(k,i): index out of range");
#endif
		_rem_precondition (k,i);
	}
	/*{\Mop removes the key |k| from |pre(i)|. \precond $1\leq i\leq n$.}*/


	/*{\Mtext       
	\bigskip
	\headerline{d) Protected Methods for Maintaining Postconditions} }*/

	pr_item add_post_rule (agd_key k, int i = 1) {
#if !defined(LEDA_CHECKING_OFF)
		if (i < 0 || i > _m)
			error_handler(1, "AGDModule::add_post_rule(k,i): index out of range");
#endif
		return _add_post_rule (k,i);
	}
	/*{\Mop adds a new rule item \<$\rightarrow k$\> to |post_rules(i)|
	and returns it. \precond $0\leq i\leq m$.}*/

	pr_item add_post_rule (agd_key p1, int i1, agd_key p, int i = 1) {
#if !defined(LEDA_CHECKING_OFF)
		if (i1 < 1 || i1 > _n || i < 0 || i > _m)
			error_handler(1, "AGDModule::add_post_rule(p1,i1,p,i): index out of range");
#endif
		return _add_post_rule (p1,i1,p,i);
	}
	/*{\Mop adds a new rule item \<$(p_1,i_1)\rightarrow p$\> to |post_rules(i)|
	and returns it. \precond $0\leq i\leq m$.}*/

	pr_item add_post_rule (const TaggedRule& r, int i = 1) {
#if !defined(LEDA_CHECKING_OFF)
		if (i < 0 || i > _m)
			error_handler(1, "AGDModule::add_post_rule(r,i): index out of range");
#endif
		return _add_post_rule (r,i);
	}
	/*{\Mop adds a new rule item \<$r$\> to |post_rules(i)| and returns it.
	\precond $0\leq i\leq m$.}*/

	void rem_post_rule (pr_item it, int i = 1) {
#if !defined(LEDA_CHECKING_OFF)
		if (i < 0 || i > _m)
			error_handler(1, "AGDModule::rem_post_rule(it,i): index out of range");
#endif
		_rem_post_rule (it,i);
	}
	/*{\Mop removes the rule item |it| from |post_rules(i)|.
	\precond $0\leq i\leq m$ and |it| is a rule item in |post_rules(i)|.}*/


protected:

	/*{\Mtext       
	\bigskip
	\headerline{Checking the Precondition of a Module} }*/

	/*{\Mtext
	Particular module types, for example |LayoutModule|, provide a method
	check that checks if a given input satisfies the precondition of the
	module. |\Mname| implements extendable default implementations to
	check a precondition for specific types, for example graphs. These
	default implementations can be used by the base class of a particular
	module type (e.g., |LayoutModule|) to implement its check method.

	The method check\_graph can test if a given input graph satisfies
	the following keys:
	|directed|, |undirected|, |planar|, |simple|, |no_self_loops|,
	|no_reversals|, |connected|, |biconnected|, |triconnected|,
	|bipartite|, |four_graph|, |acyclic|, |forest|, |tree|, |single_source|,
	|single_sink|. If check\_graph finds another key it calls
	check\_graph\_key.
	}*/

	bool check_graph (int i, const leda_graph& G, leda_string& message) const;
	/*{\Mop returns |true| if $G$ satisfies the preconditions |pre(i)|.
	Otherwise, |false| is returned, and |message| contains the
	error message.}*/

	virtual bool check_graph_key (agd_key p, const leda_graph& G,
		leda_string& message) const;
	/*{\Mop overridable to extend check\_graph. Must return |true| if |G|
	satisfies |p|, otherwise |false| must be returned and |message| must
	contain the error message.}*/


private:
	int _n, _m; // number of input and output parameters

	leda_array<KeySet> _pre;
	leda_array<leda_list<TaggedRule> > _post;

	KeySet _postconditions (int i, const leda_array<KeySet>& PRE) const;
	bool _need_precondition (int i, const leda_string & p) const;
	const KeySet& _preconditions (int i) const;
	bool _usable (const leda_array<KeySet>& PRE, const leda_array<KeySet>& POST) const;

	agd_key _add_precondition (const leda_string& p, int i = 1);
	void _add_precondition (agd_key k, int i = 1) ;
	void _add_precondition (agd_key k, bool status, int i = 1);
	void _add_preconditions (const leda_string& ps, char delim = ',', int i = 1);
	void _rem_precondition (agd_key k, int i = 1);

	pr_item _add_post_rule (agd_key k, int i = 1);
	pr_item _add_post_rule (agd_key p1, int i1, agd_key p, int i = 1);
	pr_item _add_post_rule (const TaggedRule& r, int i = 1);
	void _rem_post_rule (pr_item it, int i = 1);
};


inline bool AGDModule::check_graph_key (agd_key /*p*/,
	const leda_graph& /*G*/, leda_string& message) const
{
	message = "unknown key";
	return false;
}

#endif
