//-------------------------------------------------------------------
//
//                         AGDModuleOption.h
//
//    author: Carsten Gutwenger
//    date:   july 1997
//-------------------------------------------------------------------


#ifndef _AGD_AGD_MODULE_OPTION_H
#define _AGD_AGD_MODULE_OPTION_H


#include <LEDA/array.h>
#include <AGD/AGDModule.h>


class AGDModuleOptionImpl {

protected:
	leda_array<KeySet> _PRE, _POST;

public:
	AGDModuleOptionImpl() { }

	virtual ~AGDModuleOptionImpl() { }

	void _init (const leda_array<KeySet>& PRE, const leda_array<KeySet>& POST,
		const AGDModule& module);

	bool _set (const AGDModule& module);

	bool _check (const AGDModule& module);

};



/*{\Manpage {AGDModuleOption} {M} {Module Options} {opt} }*/

template <class M>
class AGDModuleOption : public AGDModuleOptionImpl {

/*{\Mdefinition
	The class |\Mname| is the base class for module options ...
}*/

public:

	/*{\Mcreation}*/

	AGDModuleOption() : AGDModuleOptionImpl() {
		_module = nil;
	}
	/*{\Mcreate creates an instance |\Mvar| of type |\Mname| ...}*/

	virtual ~AGDModuleOption() {
		if (_module != nil) delete _module;
	}


	/*{\Moperations}*/

	void init (const leda_array<KeySet>& PRE, const leda_array<KeySet>& POST, const M& module) {
#if !defined(LEDA_CHECKING_OFF)
		if (_module != nil)
			error_handler (1, "AGDModuleOption: reinitialization not possible!");
#endif
		_init (PRE, POST, module);
		_module = clone(module);
	}
	/*{\Mop initializes |\Mvar| with guaranteed preconditions |PRE|, required
	postconditions |POST|, and initial module |module|.}*/

	bool set (const M& module) {
#if !defined(LEDA_CHECKING_OFF)
		if (_module == nil)
			error_handler (1, "AGDModuleOption: module not initialized!");
#endif
		if (_check(module)) {
			delete _module;
			_module = clone(module);
			return true;
		} else
			return false;
	}
	/*{\Mop sets the module option to |module| and returns |true| if |module|
	is usable, returns |false| otherwise.}*/

	bool check (const M& module) {
		return _check (module);
	}
	/*{\Mop returns |true| if |module| is usable, |false| otherwise.}*/

	const M &get() const {
#if !defined(LEDA_CHECKING_OFF)
		if (_module == nil)
			error_handler (1, "AGDModuleOption::get: module not initialized!");
#endif
		return *_module;
	}

	M &get() {
#if !defined(LEDA_CHECKING_OFF)
		if (_module == nil)
			error_handler (1, "AGDModuleOption::get: module not initialized!");
#endif
		return *_module;
	}
	/*{\Mop returns a reference to the module contained in |\Mvar|,}*/

protected:

	virtual M *clone(const M& m) {
		return (M *) m.clone();
	}

	M *_module;

};


/*{\Mtext
\bigskip
\headerline{Declaring Module Options}
The macro {\bf AGD\_DECLARE\_MODULE}(|type|,|name|) should be used to declare
a module option of type |type| with name |name| in a class declaration. This
macro expands to

{\tt protected:\\
\hspace*{1cm}AGDModuleOption<{\it type}> {\it name};\\
public:\\
\hspace*{1cm}void set\_{\it name} (const {\it type} \&M) $\{$\\
\hspace*{2cm}{\it name}.set(M);\\
\hspace*{1cm}$\}$\\
}
}*/

#define AGD_DECLARE_MODULE(type,name) \
protected: \
	AGDModuleOption<type> name; \
public: \
	void set_##name (const type &M) { \
		name.set(M); \
	} \


/*{\Mexample
We create a sample layout algorithm with an optional augmentation
module. The augmentation module gets a simple, planar graph and
must augment it to a simple, planar, biconnected graph. The
call\_copy\_layout method shows how to use the module, and the
main function shows, how to set the module option to another
augmenter.

\small\tt
\begin{verbatim}
class SampleLayout : public LayoutModule
{
  AGD_DECLARE_MODULE(AugmentModule,augmenter);
    ...
public:
  SampleLayout();
  bool _call(GraphCopy& G, LayoutInterface& A);
};

SampleLayout::SampleLayout() : LayoutModule(true)
{
    ...
  array<KeySet> PRE(1,1), POST(0,1);
  PRE[1]  << key::planar << key::simple;
  POST[1] << key::planar << key::biconnected << key::simple;
  augmenter.init (PRE,POST,PlanAug());
    ...
}

SampleLayout::_call(GraphCopy& G, LayoutInterface& A)
{
    ...
  augmenter.get().call(G,L);  // make G biconnected
    ...
}

int main()
{
  GraphWin GW;
  GW.open();

  GraphWinInterface GWI(GW.get_graph());
  SampleLayout L;

  L.set_augmenter (PlanAugOpt());  // use an optimal augmenter
  L.call(GW,GWI);

    ...
  return 0;
}
\end{verbatim}
}*/


#endif
