//-------------------------------------------------------------------
//
//                        AugmentModule.h
//
//
//    author: Carsten Gutwenger
//    date:   july 1997
//-------------------------------------------------------------------


#ifndef _AGD_AUGMENTATION_MODULE_H
#define _AGD_AUGMENTATION_MODULE_H


#include <AGD/AGDModule.h>


/*{\Manpage {AugmentModule} {} {Augmentation Modules} {A} }*/

/*{\Mbaseclasses

\begin{hierarchy}{6.5}{2.5}
\hdclass{0}{1.5}{5.5}{AGDModule}
\arhook{5.5}{2}{6.5}
\hdclass{1.5}{0}{5.5}{AugmentModule}
\end{hierarchy}
}*/

class AugmentModule : public AGDModule {

/*{\Mdefinition
The class |\Mname| is the base class for augmentation modules. An
augmentation module transforms an input graph $G$ into an output
graph $G'$ by adding edges, such that $G'$ satisfies a certain
condition, e.g., biconnected. This condition is expressed by
the postcondition of the module.

\headerline{Input and Output Parameters}

\begin{tabular}{lcc}
input parameter: & graph & $|in_graph|=1$ \\
output parameter: & graph & $|out_graph|=1$
\end{tabular}

The integer constants |in_graph| and |out_graph| can be used to denote
the positions of the input and output parameters.

\headerline{Implementing Augmentation Algorithms}

An implementation of an augmentation module must override
the protected method call\_aug($G$,$L$), which gets as
input a graph reference $G$. The augmented edges are added
to $G$, and the list of added edges is returned in $L$.
}*/

public:

	// the input and output parameters of an augmentation module
	static const int in_graph;
	static const int out_graph;

	/*{\Minitialization A 5}*/

	AugmentModule () : AGDModule(1,1) { }

	/*{\Mcreate initializes an augmentation module.}*/

	virtual ~AugmentModule () { }


	/*{\Moperations}*/

	/*{\Mtext
	\headerline{Standard Interface} }*/

	bool check (const leda_graph& G, leda_string& message);
	/*{\Mop returns |true| if $G$ satisfies the precondition of |\Mvar|.
	Otherwise, |false| is returned, and |message| contains the
	error message.}*/

	void call (leda_graph& G);
	/*{\Mop calls the augmentation module for graph $G$.}*/

	void call (leda_graph& G, leda_list<leda_edge>& L);
	/*{\Mop calls the augmentation module for graph $G$. Returns the
	list of added edges in |L|.}*/
	
	int num_added_edges() const {
		return _num_added_edges;
	}
	/*{\Mop returns the number of added edges.}*/

protected:

	/*{\Mtext
	\bigskip
	\headerline{Protected Overridables to Implement Functionality} }*/

	virtual void _call (leda_graph& G, leda_list<leda_edge>& L)
	/*{\Moppure implements the augmentation algorithm for graph |G|.
	Returns the list of added edges in |L|.}*/
	= 0;

	/*{\Mtext
	\bigskip
	\headerline{Protected Auxiliary Methods} }*/

	static void hide_self_loops (leda_graph& G, leda_list<leda_edge>& L);
	/*{\Mstatic hides all self loops and appends them to |L|.}*/

	static void hide_multiple_edges (leda_graph& G, leda_list<leda_edge>& L);
	/*{\Mstatic hides all but one of each set of parallel edges and
	appends all hided edges to |L|.}*/

private:
	int _num_added_edges;
};


#endif
