//-------------------------------------------------------------------
//
//                       BarycenterHeuristic.h
//
//
//    author: Carsten Gutwenger
//            Michael Krger
//    date:   february 1998
//-------------------------------------------------------------------


#ifndef _AGD_BARYCENTER_HEURISTIC_H
#define _AGD_BARYCENTER_HEURISTIC_H


#include <AGD/TwoLayerCrossMin.h>


/*{\Manpage {BarycenterHeuristic} {} {Barycenter Heuristic} {M} }*/

/*{\Mbaseclasses

\begin{hierarchy}{8}{4}
\hdclass{0}{3}{5.5}{AGDModule}
\arhook{5.5}{3.5}{6.5}
\hdclass{1}{1.5}{6}{TwoLayerCrossMin}
\arhook{7}{2}{8}
\hdclass{2}{0}{6.5}{BarycenterHeuristic}
\end{hierarchy}
}*/

class BarycenterHeuristic : public TwoLayerCrossMin {

/*{\Mdefinition
The class |\Mname| implements the two layer crossing minimization heuristic
{\it Barycenter}. It reorders the nodes on the changeable layer according
to the barycenter weight~\cite{stt-mvuhs-81}
\[ Bary(v) = \left\{\begin{array}{cl}
\frac{\sum_{\D w\in adj(v)}\D pos(w)}{\D\mid adj(v)\mid} & \mbox{ if }adj(v)\neq\emptyset\\
0.0 & \mbox{ otherwise}
\end{array}\right. \]
where $v$ is a node on the changeable layer, $|adj|(v)$ is the set of
neighbors of $v$ on the fixed layer, and $|pos|(w)$ is the position
of $w$ on the fixed layer.
}*/

public:

	/*{\Mcreation M 5}*/

	BarycenterHeuristic() { }
	/*{\Mcreate  creates an instance |\Mvar| of type |\Mname|.}*/

	~BarycenterHeuristic() { }


	/*{\Moperations}*/

	void call (const leda_graph& G, Layer& L);
	/*{\Mop calls |\Mvar| for graph $G$ and layer $L$. Reorders the nodes
	on layer $L$.}*/

	void init (const leda_graph& G, const Hierarchy& /*H*/) {
		weight.init(G);
	}

	void cleanup () {
		weight.init();
	}

	AGDModule *clone() const {
		return new BarycenterHeuristic();
	}

private:

	leda_node_array<double> weight;
};


#endif
