//-------------------------------------------------------------------
//
//                       CanonicalOrder.h
//
//    author: Carsten Gutwenger
//    date:   february 1997
//-------------------------------------------------------------------


#ifndef __CanonicalOrder_h
#define __CanonicalOrder_h


#include <LEDA/array.h>
#include <LEDA/graph.h>



// represantation for the sets of the canonical order (V1,...,Vk)

/*{\Manpage {OrderSet} {} {Sets of a Canonical Ordering} {V} }*/

class OrderSet : public leda_array<leda_node> {

/*{\Mdefinition
An instance |\Mvar| of type |\Mname| represents one set of a canonical
ordering |C| of a graph $G=(V,E)$. It contains a set $\{v_,\ldots,v_p\}$ of
nodes of |G| ordered from left to right, a leftvertex $c_\ell$, a
rightvertex $c_r$, and additional information |has_left|
and |has_right|, where $|has_left| = |true| \iff (v_1,c_\ell) \in E$,
and $|has_right| = |true| \iff (v_p,c_r) \in E$.
}*/

public:

	/*{\Mcreation}*/

	OrderSet () : leda_array<leda_node>() {
		left (nil); right (nil);
		left_edge (nil); right_edge (nil);
	}
	/*{\Mcreate creates an instance |\Mvar| of type |\Mname| representing
	an empty set.}*/

	OrderSet (int l, leda_edge el = nil, leda_edge er = nil) : leda_array<leda_node> (1,l) {
		left  (el ? target(el) : nil);
		right (er ? target(er) : nil);
		left_edge (el); right_edge (er);
	}
	/*{\Mcreate creates an instance |\Mvar| of type |\Mname| representing
	a set of length |l| with left edge |el| and right edge |er|.}*/

	~OrderSet ()
		{ }


	/*{\Moperations}*/

	leda_node left() const {
		return leftvertex;
	}
	/*{\Mop returns the leftvertex $c_\ell$ of |\Mvar|.}*/

	leda_node right() const {
		return rightvertex;
	}
	/*{\Mop returns the rightvertex $c_r$ of |\Mvar|.}*/

	leda_edge left_edge() const {
		return _left_edge;
	}
	/*{\Mop returns the edge $(z_1,c_\ell)$ or |nil|.}*/

	leda_edge right_edge() const {
		return _right_edge;
	}
	/*{\Mop returns the edge $(z_p,c_r)$ or |nil|.}*/

	bool has_left() const {
		return _left_edge != nil;
	}
	/*{\Mop returns the information |has_left| of |\Mvar|.}*/

	bool has_right() const {
		return _right_edge != nil;
	}
	/*{\Mop returns the information |has_right| of |\Mvar|.}*/

	void left (leda_node cl) {
		leftvertex = cl;
	}
	/*{\Mop sets the leftvertex of |\Mvar| to |cl|.}*/

	void right (leda_node cr) {
		rightvertex = cr;
	}
	/*{\Mop sets the rightvertex of |\Mvar| to |cr|.}*/

	void left_edge (leda_edge el) {
		_left_edge = el;
	}
	/*{\Mop sets the edge $(z_1,c_\ell)$.}*/

	void right_edge (leda_edge er) {
		_right_edge = er;
	}
	/*{\Mop sets the edge $(z_p,c_r)$.}*/

	int len() const {
		return high();
	}
	/*{\Mop returns the length of |\Mvar|, i.e. the number of nodes in
	|\Mvar|.}*/

	leda_node operator[] (const int i) const {
		return leda_array<leda_node>::operator[](i);
	}

	leda_node& operator[] (const int i) {
		return leda_array<leda_node>::operator[](i);
	}
	/*{\Marrop returns the |i|-th node of |\Mvar| from left (the leftmost
	node has index 1).}*/

	void operator= (const OrderSet& S) {
		leda_array<leda_node>::operator= (S);
		left      (S.left());
		right     (S.right());
		left_edge  (S.left_edge());
		right_edge (S.right_edge());
	}

private:
	leda_node leftvertex, rightvertex;	
	leda_edge _left_edge, _right_edge;
};

/*{\Mimplementation
Order sets are implemented by Arrays. All operations take constant time.
}*/

ostream& operator<< (ostream& out, OrderSet);
istream& operator>> (istream& in, OrderSet);
inline int compare (const OrderSet&, const OrderSet&)
{ return 0; }


enum CanonicalOrderType { co_triconnected, co_biconnected };
//enum Base_Type { base_2_nodes, base_one_half, base_one_third };

/*{\Manpage {CanonicalOrder} {} {The Canonical Ordering of a Graph} {C} }*/

class CanonicalOrder {

/*{\Mdefinition
An instance |\Mvar| of type |\Mname| ia canonical ordering of a plane
graph |G=(V,E)|. It represents an ordered partition $V_1,\ldots,V_m$ of |V|.
We provide a canonical ordering for triconnected planar graphs (Goos Kant),
and another one for biconnected planar graphs:

{\bf Definition (canonical ordering for triconnected planar graphs):}
Let $G=(V,E)$ be a triconnected plane graph with $\Labs{V} \geq 3$,
outerface $F_{out}$, and $v_1,\ldots,v_q$ a path on $F_{out}$ with
$q < \Labs{F_{out}}$. Let $C = V_1,\ldots ,V_m$ be an
ordered partition of $V$, that is, $V_1\cup\ldots\cup V_m = V$ and $V_i\cap
V_j=\emptyset$ for $i\neq j$. Define $G_k$ to be the subgraph of
$G$ induced by $V_1\cup\ldots\cup V_k$, and denote by $C_k$ the
outerface of $G_k$. $C$ is a {\bf canonical ordering} (for triconnected
planar graphs)
of $G$ with base $v_1,\ldots,v_q$ if:
\begin{itemize}
\item $V_1 = \{v_1,\ldots,v_q\}$
\item $V_m = \{z_0\}$, where $z_0$ lies on the outerface of $G$.
\item Each $C_k$ $(k>1)$ is a cycle containing $v_1,\ldots,v_q$.
\item Each $G_k$ is biconnected and internally
triconnected, that is, removing two internal vertices of $G_k$
does not disconnect it.
\item For each $k$ in $\{2,\ldots,m-1\}$, one of the following conditions
holds:
\begin{enumerate} \renewcommand{\labelenumi}{(\alph{enumi})}
\item $V_k = \{z\}$, where $z$ belongs to $C_k$ and has at least one
neighbour in $G\setminus G_k$.
\item $V_k$ is a chain, $(z_1,\ldots,z_p)$, where each $z_i$
has at least one neighbour in $G\setminus G_k$, and where $z_1$ and
$z_p$ each have one neighbour on $C_{k-1}$, and these are the only
two neighbours of $V_k$ in $G_{k-1}$.
\end{enumerate}
\end{itemize}

This is the definition of the canonical ordering for triconnected
planar graphs by Goos Kant extended by the possibility to have
more than two nodes in $V_1$.

{\bf Definition (canonical ordering for biconnected planar graphs):}
Let $G = (V,E)$ be a biconnected plane graph with $\Labs{V} \geq 2$,
outerface $F_{out}$ and $v_1,\ldots,v_q$ a path on $F_{out}$ with
$(v_i,v_j) \notin E$ for all $j\neq i+1$. Let
$C = (V_1,\ldots,V_m)$ be an ordered partition of
$V$, that is, $V_1\cup\ldots\cup V_m = V$ and $V_i\cap
V_j=\emptyset$ for $i\neq j$.
Define $G_k$ to be the subgraph of $G$ induced by $V_1\cup V_2\ldots \cup V_k$,
and denote by $C_k$ the outerface of $G_k$.
$C$ is a {\bf canonical ordering} (for biconnected planar graphs) of $G$
with base $v_1,\ldots,v_q$ if:
\begin{itemize}
\item $V_1 = \{v_1,\ldots,v_q\}$
\item Each $C_k$ $(k>1)$ is a cycle containing $v_1,\ldots,v_q$.
\item Each $G_k$ is connected and internally biconnected,
that is, removing one internal vertex of $G_k$ does not
disconnect it.
\item For each $k$ in $\{2,\ldots,m-1\}$ holds:\\
Each $z\in V_k$ belongs to $C_k$, and one of the following conditions holds,
where $C_{k-1}=(v_1=c_1,\ldots,c_q=v_2)$:
\begin{enumerate} \renewcommand{\labelenumi}{(\arabic{enumi})}
\item $V_k=\{z\}$ and $z$ has at least two neighbours in $G_{k-1}$.
\item
  $V_k$ is a chain $(z_1,\ldots z_p)$ with $p\ge 1$ of vertices with
  degree $\le2$ in $G_k$, and there are two vertices
  $c_\ell$ and $c_r$ $(1\le \ell < r\le q)$ on $C_{k-1}$, and the following
  three conditions hold:
\begin{enumerate}\renewcommand{\labelenumii}{(\alph{enumii})}
\item For $\ell<i<r$ holds: $c_i$ has no neighbours in $G\setminus G_k$.
\item If $(c_\ell,z_1)\in E$, then $z_1$ immediatelly follows $c_{\ell+1}$
	in the counter-clockwise circular ordering around $c_\ell$ in $G$.
\item If $(z_p,c_r)\in E$, then $z_p$ immediatelly precedes $c_{r-1}$
	in the counter-clockwise circular ordering around $c_r$ in $G$.
\end{enumerate}
\end{enumerate}
\end{itemize}

The computation of the canonical ordering (call of the constructor)
allows the following options:

\begin{itemize}
\item {\bfseries co\_type=\{{\itshape co\_triconnected},
|co_biconnected| \}:}\\
selects the type of the canonical ordering.
\item {\bfseries max\_outer=\{{\itshape true}, |false| \}:}\\
determines which face of the graph is used as outerface. Selects the face
with maximal degree if |max_outer| is |true|, or the first
face in the list of faces otherwise.
\item {\bfseries prefer\_nodes=\{{\itshape false}, |true| \}:}\\
only used for the order type |co_triconnected|. If in a step of the
computation of the ordering, it is possible to select a vertex or a chain
as next order set, a vertex is selected if |prefer_nodes| is |true|,
a chain otherwise.
\item {\bfseries base\_ratio:}\\
a double value determining the length of the base, that is the number of
nodes in $V_1$.
If |base_ratio| is $r$ and the outerface is $F$, the length of
the base is $\min\left(2,\max\left(\ell, r\Labs{F}\right)\right)$,
where $\ell = \max\{k \mid v_1,\ldots,v_k$ is a path on $F_{out}$ with
$(v_i,v_j)\notin E$ for all $j\neq i+1 \}$.
\end{itemize}
}*/

public:

	/*{\Mcreation}*/

	CanonicalOrder (leda_graph &G,
		CanonicalOrderType co_type = co_triconnected,
		leda_face F = nil, bool prefer_nodes = false,
		double base_ratio = 0.0);
	/*{\Mcreate creates an instance |\Mvar| of type |\Mname| initialized
	to a canonical ordering for triconnected, planar graphs (|co_triconnected|),
	or for biconnected, planar graphs (|co_biconnected|). \precond |G|
	is a directed graph representing a planar map.}*/

	CanonicalOrder ()
		{ _g = nil; }

	virtual ~CanonicalOrder ()
		{ }


	/*{\Moperations}*/

	const leda_graph& get_graph() const {
		return *_g;
	}

	int length () const {
		return V.high();
	}
	/*{\Mop returns the number |m| of sets in the partition.}*/

	int len (int i) const {
		return V[i].len();
	}
	/*{\Mop returns the length of the |i|-th set $V_i$.}*/

	leda_node operator() (int i, int j) const {
		return (V[i])[j];
	}
	/*{\Mfunop returns the |j|-th node of the |i|-th set $V_i$.}*/

	const OrderSet& operator[](int i) const {
		return V[i];
	}
	/*{\Marrop returns a constant reference to the |i|-th set $V_i$.}*/

	leda_node left (int i) const {
		return V[i].left();
	}
	/*{\Mop returns the leftvertex of the |i|-th set $V_i$.}*/

	leda_node right (int i) const {
		return V[i].right();
	}
	/*{\Mop returns the rightvertex of the |i|-th set $V_i$.}*/

	int rank (leda_node v) const {
		return the_rank[v];
	}
	/*{\Mop returns the rank of node |v|, i.e. $|rank(v)| = i \iff
	v \in V_i$.}*/

	void push (int k, leda_node v, leda_node tgt);

	friend class CompOrder2;
	friend class CompOrder2_1;

protected:
	static void comp_tric_co (leda_graph& g, leda_list<OrderSet> &c_order,
		leda_face F = nil, bool prefer_nodes = false,
		double base_length = 0.0);
	static void comp_bic_co (leda_graph& g, leda_list<OrderSet> &c_order,
		leda_face F = nil, double base_ratio = 0.0);
	static void comp_bic_co2 (leda_graph &G, leda_list<OrderSet> &c_order,
		leda_face F_out, double base_ratio);

	static int get_base_chain (leda_graph& g, leda_face F, double base_length,
		leda_edge& e_left, leda_edge& e_right);
	static leda_edge find_max_base_chain (leda_graph& G, leda_face F, int& length);
	static leda_face max_face (leda_graph& G);
	void init_rank (leda_graph& g);

	const leda_graph *_g;
	leda_array<OrderSet> V;
	leda_node_map<int> the_rank;
};


/*{\Manpage {LmcOrder} {} {The Leftmost Canonical Order of a Graph} {Lmc} }*/

/*{\Mbaseclasses

\begin{hierarchy}{6.5}{2.5}
\hdclass{0}{1.5}{5}{CanonicalOrder}
\arhook{5}{2}{6}
\hdclass{1.5}{0}{5}{LmcOrder}
\end{hierarchy}
}*/

class LmcOrder : public CanonicalOrder {

/*{\Mdefinition
An instance |\Mvar| of type |\Mname| is a leftmost canonical ordering of a 
graph |G|. We define a leftmost canonical ordering as follows:

  {\bf Definition:} {\it A canonical ordering is a leftmost canonical
  (lmc-)ordering if we can add in any step $k$ a vertex $V_k$ with
  leftvertex $c_l$ or a vertex set $V_{k'}$ with leftvertex $c_{l'}$,
  and $l < l'$ holds, then $k < k'$.}
}*/

public:

	/*{\Mcreation}*/

	LmcOrder (leda_graph &g,
		CanonicalOrderType co_type = co_triconnected,
		leda_face F = nil, bool prefer_nodes = false,
		double base_ratio = 0.0);
	/*{\Mcreate creates an instance |\Mvar| of type |\Mname| initialized
	to a leftmost canonical order for |G|. \precond |G|
	is a directed graph representing a planar map.}*/

	LmcOrder () : CanonicalOrder ()
		{ }

};

/*{\Mimplementation
|\Mname| derives from |CanonicalOrder|. The transformation into a
leftmost ordering takes time $O(n)$, where $n$ is the number of nodes
of |G|.
}*/

#endif

