//-------------------------------------------------------------------
//
//                      ConvexDrawLayout.h
//
//    author: Carsten Gutwenger
//    date:   february 1997
//-------------------------------------------------------------------


#ifndef _AGD_CONVEX_DRAW_LAYOUT_H
#define _AGD_CONVEX_DRAW_LAYOUT_H


#include <AGD/GridLayoutModule.h>


class CanonicalOrder;


/*{\Manpage {ConvexDrawLayout} {} {The ConvexDraw Algorithm} {L} }*/

/*{\Mbaseclasses

\begin{hierarchy}{9.5}{5.5}
\hdclass{0}{4.5}{5}{AGDModule}
\arhook{5}{5}{6}
\hdclass{1.5}{3}{5}{LayoutModule}
\arhook{6.5}{3.5}{7.5}
\hdclass{2}{1.5}{6}{GridLayoutModule}
\arhook{8}{2}{9}
\hdclass{3.5}{0}{6}{ConvexDrawLayout}
\end{hierarchy}
\hfill\agdeps{convex_draw.eps}{5.5cm}
}*/

class ConvexDrawLayout : public GridLayoutModule {

/*{\Mdefinition
The class |\Mname| represents the layout algorithm ConvexDraw by
Marek Chrobak and Goos Kant~\cite{ck-cgd3c-93}.

This algorithm draws a planar, triconnected graph |G| straight-line
without crossings. |G| may not contain self-loops or mulitple edges.
All faces are drawn convex, the grid layout
size is at most $(n-1) \times (n-1)$ for a graph with $n$ nodes ($n \geq 3$).

\horizline
\commoninfo{ConvexDraw}{ConvexDraw (Kant)}{Marek Chrobak, Goos Kant}
{Carsten Gutwenger}{march 1997}{2.0}

\headerline{Pre- and Postcondition}
\begin{tabular}{lcl}
precondition & $=$ & $\{\, |planar|,|triconnected|,|simple|,|no self loops| \,\}$ \\
postcondition(|PRE|) & $=$ & $\{\, \mbox{|straight|-|line|},|no crossings|,|convex| \,\}$
\end{tabular}

\headerline{Optional Parameters}
Instances of |\Mname| provide the following optional parameters:
\begin{options}
\item |bool| {\bf size\_opt} = |true|\\
a boolean value determining if the grid size of the
layout is optimized compared to the original Convex algorithm. This
option reduces the grid size especially for graphs with few edges.
If the graph is (almost) triangulated, the grid size will be (almost)
the same.
\item |bool| {\bf side\_opt} = |false|\\
a boolean value determining if it is allowed to change
the shape of the outerface (from a triangle) in order to improve the
grid size of the layout.
\item |double| {\bf base\_ratio} $= 0.33$\\
a double value determining, how many nodes are drawn on the base line.
If |base_ratio| is $r$ and the outerface is $F$, the number of nodes
on the base line is $\min\left(2,\max\left(\ell, r\Labs{F}\right)\right)$,
where $\ell = \max\{k \mid v_1,\ldots,v_k$ is a path on $F_{out}$ with
$(v_i,v_j)\notin E$ for all $j\neq i+1 \}$.
\end{options}\horizline
}*/

public:

	/*{\Mcreation L 5}*/

	ConvexDrawLayout ();
	/*{\Mcreate creates an instance |\Mvar| of type |\Mname|.}*/

	virtual ~ConvexDrawLayout() { };


	/*{\Moperations 1.5 2.5}*/

	leda_string name () const {
		return "ConvexDraw";
	}

	leda_string long_name () const {
		return "ConvexDraw (Chrobak/Kant)";
	}

	leda_string author () const {
		return "Marek Chrobak, Goos Kant";
	}

	leda_string impl_author () const {
		return "Carsten Gutwenger";
	}

	leda_string impl_version () const {
		return "2.0";
	}

	leda_string impl_date () const {
		return "march 1997";
	}


	bool size_opt() const {
		return the_size_opt;
	}

	void size_opt (bool so) {
		the_size_opt = so;
	}

	bool side_opt() const {
		return the_side_opt;
	}

	void side_opt (bool so) {
		the_side_opt = so;
	}

	double base_ratio() const {
		return the_base_ratio;
	}
	
	void base_ratio (double br) {
		the_base_ratio = br;
	}

	AGDModule *clone () const;

	/*{\Moptions warnings=no}*/

	/*{\Mtext       
	\headerline{Generic Interface (Inherited Methods)}
	The detailed description of these methods can be found in the
	manual entries of the base classes (|LayoutModule|, |GridLayoutModule|).
	\setlength{\entrysep}{-1ex}}*/

	/* inherited
	bool check (const graph& G, string& message);
	*/
	/*{\Mop}*/

	/* inherited
	void call (const graph& G, LayoutInterface& A);
	*/
	/*{\Mop}*/

	/* inherited
	virtual void call (const graph& G, face f, LayoutInterface& A);
	*/
	/*{\Mop}*/

	/* inherited
	virtual void call (PlaneGraphCopy& PG, face f, LayoutInterface& A);
	*/
	/*{\Mop}*/

	/* inherited
	void call_grid (const graph& G, GridLayout &gl);
	*/
	/*{\Mop}*/

	/* inherited
	void call_grid (const graph& G, face f, GridLayout &gl);
	*/
	/*{\Mop}*/

	/* inherited
	void call_grid (PlaneGraphCopy& PG, face f, GridLayout &gl);
	*/
	/*{\Mop}*/

	/* inherited
	IRect get_grid_area ();
	*/
	/*{\Mop}*/

	/* inherited
	void set_coordinate_mapper (const GridCoordinateMapper& gcm);
	*/
	/*{\Mop}*/
	
	/*{\Mtext \setlength{\entrysep}{0.5ex}}*/


protected:
	void _call_grid (PlaneGraphCopy& PG, leda_face F, GridLayout &gl);

private:
	bool the_size_opt;
	bool the_side_opt;
	double the_base_ratio;

	void compute_coordinates (leda_graph &G, CanonicalOrder &c_o,
		leda_node_map<int>& x, leda_node_map<int>& y);
};

/*{\Mimplementation
The computation of
the layout takes time $O(n)$ where $n$ is the number of nodes
of the drawn graph.}*/


#endif
