//-------------------------------------------------------------------
//
//                         GraphCopy.h
//
//    author: Carsten Gutwenger
//    date:   may 1998
//-------------------------------------------------------------------


#ifndef _AGD_GRAPH_COPY_H
#define _AGD_GRAPH_COPY_H


#include <LEDA/graph.h>


/*{\Manpage {GraphCopy} {} {Copy of a Graph} {GC} }*/

/*{\Mbaseclasses

\begin{hierarchy}{6.5}{2.5}
\hdclass{0}{1.5}{5}{graph}
\arhook{5}{2}{6}
\hdclass{1.5}{0}{5}{GraphCopy}
\end{hierarchy}
}*/

class GraphCopy : public leda_graph {

/*{\Mdefinition
An instance |\Mvar| of type |\Mname| is a copy of an original graph |G|.
For each node |v'| of |\Mvar|, |\Mname| stores the corresponding node
|original(v')| in the original graph |G|, for each node $v$ in |G| the
newly created copy |copy(v)| in |\Mvar|, for each edge |e'| in |\Mvar|
the corresponding edge |original(e')| in |G|, and for each edge $e$ in
$G$ a list of dummy edges |chain(e)| in |\Mvar|.

For nodes or edges which are created afterwards with new\_node or new\_edge,
the corresponding node or edge is set to |nil|.}*/

public:

	/*{\Mcreation GC 5.5}*/

	GraphCopy(const GraphCopy& GC);
	GraphCopy() : leda_graph() {
		g_orig = nil;
	}
	/*{\Mcreate creates an instance |\Mvar| of type |\Mname| initialized
	to an empty graph without original graph.}*/

	GraphCopy(const leda_graph& G);
	/*{\Mcreate creates an instance |\Mvar| of type |\Mname| initialized
	to a copy of |G|.}*/

	virtual ~GraphCopy() { }


	/*{\Moperations}*/

	GraphCopy& operator=(const GraphCopy& GC);

	const leda_graph &original() const {
		return *g_orig;
	}
	/*{\Mop returns the original graph |G|.}*/

	leda_node original (leda_node v) const {
		return v_orig[v];
	}
	/*{\Mop returns the node of the original graph |G| that corresponds
	to |v|, |nil| if |v| was not in |G|, e.g. |v| was created afterwards.}*/

	leda_edge original (leda_edge e) const {
		return e_orig[e];
	}
	/*{\Mop returns the edge of the original graph |G| that corresponds
	to |e|, |nil| if |e| was not in |G|, e.g. |e| was created afterwards.}*/

	leda_node copy (leda_node v) const {
		return _copy[v];
	}
	/*{\Mop returns |copy(v)|. \precond $v\in G$.}*/

	const leda_list<leda_edge>& chain (leda_edge e) const {
		return _chain[e];
	}
	/*{\Mop returns the list of dummy edges for edge $e$.
	\precond $e$ is an edge in $G$.}*/

	bool is_dummy (leda_node v) const {
		return original(v) == nil;
	}
	/*{\Mop returns |true| iff $v$ has a corresponding node in the
	original graph.}*/
	
	bool is_dummy (leda_edge e) const {
		return original(e) == nil;
	}
	/*{\Mop returns |true| iff $e$ has a corresponding edge in the
	original graph.}*/

	bool is_reversed (leda_edge e) const {
		return source(e) != original(source(_chain[e].head()));
	}
	/*{\Mop returns |true| iff $e\in G$ has been reversed in |\Mvar|.}*/

	virtual leda_edge split (leda_edge e);
	/*{\Mop splits edge $e=(v,w)$ into edges $(v,u)$, $(u,w)$ by introducing
	a new noe $u$.Returns the edge $(u,w)$.}*/

	virtual bool is_map() const {
		return false;
	}
	/*{\Mop virtual method returning |false|.}*/

	// add a new node with predefined original node
	leda_node add_node (leda_node orig) {
		leda_node v = new_node();
		v_orig[_copy[orig] = v] = orig;
		return v;
	}

protected:

	void map_to_copy (leda_node_array<leda_node>& v_copy,
		leda_edge_array<leda_edge>& e_copy, const leda_graph& G);
	leda_edge _first_adjacent (leda_node v);
	leda_edge _succ_adjacent (leda_edge e, leda_node v);

	leda_graph const *g_orig;

	leda_node_array<leda_node> _copy;
	leda_edge_map<leda_edge> e_orig;
	leda_node_map<leda_node> v_orig;

	leda_edge_map<leda_list<leda_edge> > _chain;
	leda_edge_map<list_item> _in_chain;

	void _init_GC (const GraphCopy& GC);
};

/*{\Mimplementation
All operations for graphs are available for instances of
type |\Mname|, too. All additional operations take constant time.}*/


#endif
