//-------------------------------------------------------------------
//
//                      GraphWinInterface.h
//
//    author: Carsten Gutwenger, revised by David Alberts
//    date:   june 1997
//-------------------------------------------------------------------


#include <AGD/LayoutInterface.h>
#include <LEDA/graphwin.h>


#ifndef _AGD_GRAPH_WIN_INTERFACE_H
#define _AGD_GRAPH_WIN_INTERFACE_H


/*{\Manpage {GraphWinInterface} {} {Interface to GraphWin} {A} }*/

/*{\Mbaseclasses

\begin{hierarchy}{7}{2.5}
\hdclass{0}{1.5}{5.5}{LayoutInterface}
\arhook{5.5}{2}{6.5}
\hdclass{1}{0}{6}{GraphWinInterface}
\end{hierarchy}
}*/

class GraphWinInterface : public LayoutInterface {

/*{\Mdefinition
	An instance |\Mvar| of type |\Mname| is the interface to the
	graphical attributes of a |GraphWin| object.
}*/

public:

	/*{\Mcreation}*/

	GraphWinInterface (GraphWin& GW);
	/*{\Mcreate creates an instance |\Mvar| of type |\Mname| initialized
	to an interface to |GW|.}*/

	~GraphWinInterface();


	/*{\Moperations}*/

	double get_x (leda_node v) const {
		return gw_p->get_position(v).xcoord();
	}
	/*{\Mop returns the x-coordinate of the position of node |v|.}*/

	void set_x (leda_node v, double new_x) {
		gw_p->set_position(v,leda_point(new_x,-get_y(v)));
	}
	/*{\Mop sets the x-coordinate of the position of node |v|.}*/

	double get_y (leda_node v) const {
		return -gw_p->get_position(v).ycoord();
	}
	/*{\Mop returns the y-coordinate of the position of node |v|.}*/

	void set_y (leda_node v, double new_y) {
		gw_p->set_position(v,leda_point(get_x(v),-new_y));
	}
	/*{\Mop sets the y-coordinate of the position of node |v|.}*/

	DPoint get_position (leda_node v) const {
		leda_point p = gw_p->get_position(v);
		return DPoint(p.xcoord(),-p.ycoord());
	}
	/*{\Mop returns the position of node |v|.}*/

	void set_position (leda_node v, const DPoint& p) {
		gw_p->set_position(v,leda_point(p.xcoord(),-p.ycoord()));
	}
	/*{\Mop sets the position of node |v|.}*/

	DPolyline get_bend_points (leda_edge e) const {
		leda_list<leda_point> points = gw_p->get_bends(e);
		DPolyline l;
		leda_point p;
		forall(p,points) l.append(DPoint(p.xcoord(),-p.ycoord()));
		return l;
	}
	/*{\Mop returns the bend points of edge |e|.}*/

	void set_bend_points (leda_edge e, const DPolyline& l) {
		leda_list<leda_point> points;
		DPoint dp;
		forall(dp,l) points.append(leda_point(dp.xcoord(),-dp.ycoord()));
		gw_p->set_bends(e,points);
	}
	/*{\Mop sets the bend points of edge |e|.}*/

	double get_width (leda_node v) const {
		return gw_p->get_radius(v)*2;
	}
	/*{\Mop returns the width of the graphics of node |v|.}*/

	void set_width (leda_node v, double new_w) {
		gw_p->set_radius (v, new_w/2);
	}
	/*{\Mop sets the width of the graphics of node |v|.}*/

	double get_height (leda_node v) const {
		return gw_p->get_radius2(v)*2;
	}
	/*{\Mop returns the height of the graphics of node |v|.}*/

	void set_height (leda_node v, double new_h) {
		gw_p->set_radius2 (v, new_h/2);
	}
	/*{\Mop sets the height of the graphics of node |v|.}*/


	DPoint get_source_anchor (leda_edge e) const {
		leda_point p = gw_p->get_source_anchor(e);
		return DPoint(p.xcoord(),p.ycoord());
	}
	/*{\Mop returns the source anchor position of edge |e|.}*/

	void set_source_anchor (leda_edge e, const DPoint& pos) {
		gw_p->set_source_anchor (e, leda_point(pos.xcoord(),pos.ycoord()));
	}
	/*{\Mop sets the source anchor position of edge |e| to |pos|.}*/

	DPoint get_target_anchor (leda_edge e) const {
		leda_point p = gw_p->get_target_anchor(e);
		return DPoint(p.xcoord(),p.ycoord());
	}
	/*{\Mop returns the target anchor position of edge |e|.}*/

	void set_target_anchor (leda_edge e, const DPoint& pos) {
		gw_p->set_target_anchor (e, leda_point(pos.xcoord(),pos.ycoord()));
	}
	/*{\Mop sets the target anchor position of edge |e| to |pos|.}*/

	void init (GraphWin &GW);
	void update();
	void refresh() {
		gw_p->update_graph();
	}

private:

	GraphWin *gw_p;

	bool old_flush;
};



#endif
