//-------------------------------------------------------------------
//
//                           IPoint.h
//
//    author: Carsten Gutwenger
//    date:   june 1997
//-------------------------------------------------------------------


#ifndef _AGD_IPOINT_H
#define _AGD_IPOINT_H


#include <LEDA/basic.h>


class istream;
class ostream;


/*{\Manpage {IPoint} {} {Integer Points} {ip} }*/

class IPoint {

/*{\Mdefinition
An instance |\Mvar| of type |\Mname| is a point $(a,b)$, where $a$ and
$b$ are integers. We call $a$ the x-coordinate of |\Mvar| and $b$ the
y-coordinate of |\Mvar|.}*/

private:
	int the_x, the_y;

public:

	/*{\Mcreation}*/

	IPoint() { the_x = the_y = 0; }
	/*{\Mcreate  creates an instance |\Mvar| of type |\Mname| initialized to
	the point $(0,0)$.}*/

	IPoint (int x, int y) { the_x = x; the_y = y; }
	/*{\Mcreate  creates an instance |\Mvar| of type |\Mname| initialized to
	the point $(x,y)$.}*/

	IPoint (const IPoint& p) { the_x = p.the_x; the_y = p.the_y; }
	/*{\Mcreate  creates an instance |\Mvar| of type |\Mname| initialized to
	the point |p|.}*/


	/*{\Moperations}*/

	int xcoord() const { return the_x; }
	/*{\Mop returns the x-coordinate of |\Mvar|.}*/

	int ycoord() const { return the_y; }
	/*{\Mop returns the y-coordinate of |\Mvar|.}*/

	void xcoord (int x) { the_x = x; }
	/*{\Mop sets the x-coordinate of |\Mvar| to $x$.}*/

	void ycoord (int y) { the_y = y; }
	/*{\Mop sets the y-coordinate of |\Mvar| to $y$.}*/

	IPoint& inc_x (int dx) {
		the_x += dx;
		return *this;
	}
	/*{\Mop increments the x-coordinate of |\Mvar| by |dx|. Returns
	a reference to |\Mvar|.}*/

	IPoint& inc_y (int dy) {
		the_y += dy;
		return *this;
	}
	/*{\Mop increments the y-coordinate of |\Mvar| by |dy|. Returns
	a reference to |\Mvar|.}*/

	bool operator== (const IPoint& p) const {
		return (the_x == p.the_x && the_y == p.the_y);
	}
	/*{\Mbinop Test for equality.}*/

	bool operator!= (const IPoint& p) const {
		return (the_x != p.the_x || the_y != p.the_y);
	}
	/*{\Mbinop Test for inequality.}*/

	IPoint operator+ (const IPoint& ip1) const {
		return IPoint(the_x+ip1.the_x, the_y+ip1.the_y);
	}
	/*{\Mbinop returns the addition (of coordinates).}*/

	IPoint operator- (const IPoint& ip1) const {
		return IPoint(the_x-ip1.the_x, the_y-ip1.the_y);
	}
	/*{\Mbinop returns the subtraction (of coordinates).}*/

	IPoint& operator+= (const IPoint& ip1) {
		the_x += ip1.the_x;
		the_y += ip1.the_y;
		return *this;
	}
	/*{\Mbinop adds |ip1| to |\Mvar|. Returns
	a reference to |\Mvar|.}*/

	IPoint& operator-= (const IPoint& ip1) {
		the_x -= ip1.the_x;
		the_y -= ip1.the_y;
		return *this;
	}
	/*{\Mbinop subtracts |ip1| from |\Mvar|. Returns
	a reference to |\Mvar|.}*/

	LEDA_MEMORY(IPoint)
};

ostream& operator<< (ostream& O, const IPoint& ip);
/*{\Mbinopfunc writes |ip| to output stream $O$.}*/

istream& operator>> (istream& I, IPoint& ip);
/*{\Mbinopfunc reads the coordinates of |ip| (two $int$ numbers)
from input stream $I$.}*/


#endif
