//-------------------------------------------------------------------
//
//                        Layer.h
//
//
//    author: Carsten Gutwenger
//            Michael Krger
//    date:   february 1998
//-------------------------------------------------------------------


#ifndef _AGD_LAYER_H
#define _AGD_LAYER_H

#include <LEDA/graph.h>
#include <LEDA/array.h>


class Hierarchy;
template<class P> class leda_node_pq;

/*{\Manpage {Layer} {} {Layers of Hierarchies} {L} }*/

class Layer {

/*{\Mdefinition
An instance |\Mvar| of type |\Mname| is an ordered collection
$v_0,\ldots,v_{m-1}$ of nodes with an associated node hierarchy
|H| (see \ref{Node Hierarchies}) to which |\Mvar| belongs.

The changeable and the fixed layer of $H$ are called {\it neighbored}.
}*/

	friend Hierarchy;

public:
	
	/*{\Mcreation}*/

	Layer () {
		_hierarchy = nil; _index = -1;
	}

	Layer (Hierarchy& H, int i, const leda_list<leda_node>& nodes);
	/*{\Mcreate  creates an instance |\Mvar| of type |\Mname| initialized
	to a layer of hierarchy $H$ with nodes |nodes|.}*/


	/*{\Moperations}*/

	const Hierarchy& hierarchy() const {
		return *_hierarchy;
	}
	/*{\Mop returns a reference to the associated node hierarchy.}*/

	const Layer& neighbored_layer() const;
	/*{\Mop returns a reference to the neighbored layer. \precond
	|\Mvar| is currently the fixed or changeable layer.}*/

	int length() const {
		return _nodes.size();
	}
	/*{\Mop returns the number of nodes on |\Mvar|.}*/

	leda_node operator[] (int i) const {
		return _nodes[i];
	}
	/*{\Marrop returns the node at position $i$.
	\precond $0 \leq i \leq m-1$.}*/

	void swap (int i, int j);
	/*{\Mop swaps the $i$th and the $j$th nodes on |\Mvar|.}*/

	const leda_list<leda_node>& adj_nodes (leda_node v) const;
	/*{\Mop returns a reference to the list of nodes on the neighbored
	layer that are adjacent to $v$.}*/

	const leda_list<leda_edge>& adj_edges (leda_node v) const;
	/*{\Mop returns a reference to the list of edges leading to the
	neighbored layer that are adjacent to v.}*/

	void quicksort (const leda_node_array<double>& weight);
	/*{\Mop sorts the nodes on |\Mvar| according to |weight| using a
	randomized quicksort.
	The sort is stable, i.e., if two nodes have the same weight,
	they keep in order of their previous positions.}*/

	void set_pos (const leda_node_map<int>& position);
	/*{\Mop orders the nodes on |\Mvar| according to |position|.
	\precond $\{ |position|[v] \mid v\in |\Mvar| \} = \{0,\ldots,m-1 \}$}*/

	void shuffle();
	/*{\Mop permutes the nodes on |\Mvar| randomly.}*/

private:

	void _qs (int l, int r, const leda_node_array<double>& weight);
	int cmp_qs (double w_v, int p_v, double w_w, int p_w);
	leda_random_source rs;

	Hierarchy *_hierarchy;
	int _index;
	leda_array<leda_node> _nodes;
};

ostream& operator<< (ostream& os, const Layer& L);
istream& operator>> (istream& is, Layer& L);


#ifdef _MSC_VER
int compare (const Layer&, const Layer&);
#endif


#endif
