//-------------------------------------------------------------------
//
//                         LayoutBuffer.h
//
//    author: Carsten Gutwenger
//    date:   august 1997
//-------------------------------------------------------------------


#ifndef _AGD_LAYOUT_BUFFER_H
#define _AGD_LAYOUT_BUFFER_H


#include <AGD/LayoutInterface.h>


/*{\Manpage {LayoutBuffer} {} {Layout Buffers} {A} }*/

/*{\Mbaseclasses

\begin{hierarchy}{6.5}{2.5}
\hdclass{0}{1.5}{5}{LayoutInterface}
\arhook{5}{2}{6}
\hdclass{1.5}{0}{5}{LayoutBuffer}
\end{hierarchy}
}*/

class LayoutBuffer : public LayoutInterface {

/*{\Mdefinition
A layout buffer for graph $G$ is a layout interface that stores the
following graphical attributes for graph $G$:
\begin{itemize}
\item the position of each node.
\item the bend points of each edge.
\item the source and target anchor of each edge.
\end{itemize}
The class |\Mname| overrides the respective pure virtual methods of
|LayoutInterface| for setting and getting these attributes. The
methods set\_width, get\_width, set\_height, and get\_height must
be overridden by derived classes.
}*/

public:

	/*{\Minitialization A 5.5}*/

	LayoutBuffer(const leda_graph& G);
	/*{\Mcreate initializes a layout buffer for graph $G$.}*/

	~LayoutBuffer() { }


	/*{\Moperations}*/

	double get_x (leda_node v) const {
		return _pos[v].xcoord();
	}
	/*{\Mop returns the x-coordinate of the position of node |v|.}*/

	void set_x (leda_node v, double new_x) {
		_pos[v].xcoord(new_x);
	}
	/*{\Mop sets the x-coordinate of the position of node |v| to |new_x|.}*/

	double get_y (leda_node v) const {
		return _pos[v].ycoord();
	}
	/*{\Mop returns the y-coordinate of the position of node |v|.}*/

	void set_y (leda_node v, double new_y) {
		_pos[v].ycoord(new_y);
	}
	/*{\Mop sets the y-coordinate of the position of node |v| to |new_y|.}*/

	DPoint get_position (leda_node v) const {
		return _pos[v];
	}
	/*{\Mop returns the position of node |v|.}*/

	void set_position (leda_node v, const DPoint& pos) {
		_pos[v] = pos;
	}
	/*{\Mop sets the position of node |v| to |pos|.}*/
	
	DPolyline get_bend_points (leda_edge e) const {
		return _bends[e];
	}
	/*{\Mop returns the bend points of edge |e|.}*/

	void set_bend_points (leda_edge e, const DPolyline& l) {
		_bends[e] = l;
	}
	/*{\Mop sets the bend points of edge |e| to |l|.}*/

	DPoint get_source_anchor (leda_edge e) const {
		return _s_anchor[e];
	}
	/*{\Mop returns the source anchor position of edge |e|.}*/

	void set_source_anchor (leda_edge e, const DPoint& pos) {
		_s_anchor[e] = pos;
	}
	/*{\Mop sets the source anchor position of edge |e| to |pos|.}*/

	DPoint get_target_anchor (leda_edge e) const {
		return _t_anchor[e];
	}
	/*{\Mop returns the target anchor position of edge |e|.}*/

	void set_target_anchor (leda_edge e, const DPoint& pos) {
		_t_anchor[e] = pos;
	}
	/*{\Mop sets the target anchor position of edge |e| to |pos|.}*/


private:
	leda_node_map<DPoint> _pos;
	leda_edge_map<DPolyline> _bends;
	leda_edge_map<DPoint> _s_anchor, _t_anchor;
};


#endif
