//-------------------------------------------------------------------
//
//                         LayoutModule.h
//
//
//    author: Carsten Gutwenger
//    date:   may 1998
//-------------------------------------------------------------------


#ifndef _AGD_LAYOUT_MODULE_H
#define _AGD_LAYOUT_MODULE_H


#include <AGD/AGDModule.h>


class LayoutInterface;
class GraphCopyLI;
class GraphCopy;
class DRect;
class PlaneGraphCopy;


/*{\Manpage {LayoutModule} {} {Layout Algorithms} {L} }*/

/*{\Mbaseclasses

\begin{hierarchy}{6.5}{2.5}
\hdclass{0}{1.5}{5}{AGDModule}
\arhook{5}{2}{6}
\hdclass{1.5}{0}{5}{LayoutModule}
\end{hierarchy}
}*/

class LayoutModule : public AGDModule {

	friend class GridLayoutModule;

/*{\Mdefinition
The class |\Mname| is the base class for layout algorithms and defines
a generic interface for these algorithms. We define a layout
algorithm |\Mvar| as follows:

A layout algorithm |\Mvar| gets as input a graph $G$ and for each node $v$
of $G$ the size of the rectangle surrounding the graphics of $v$. |\Mvar|
computes for each node $v$ a point, which will be the center of the
graphics of $v$, and for each edge a list of bend points with a
source and target anchor point. Layout algorithms are also allowed to change
the size of the graphics
of nodes (this feature is only used by few algorithms).

The graphical attributes of a graph are accessed via a layout
interface (see section \ref{Layout Interfaces}).

\headerline{Input and Output Parameters}

\begin{tabular}{lcc}
input parameter: & graph & $|in_graph|=1$ \\
output parameter: & layout & $|out_layout|=1$
\end{tabular}

The integer constants |in_graph| and |out_layout| can be used to denote
the positions of the input and output parameters.

\headerline{Implementation of Layout Algorithms}

Layout algorithms are not allowed to change the input graph,
i.e., the graph is passed as a const reference to the {\sl call} function.
Therefore, the AGD provides mechanisms to work on a copy of the input
graph. The classes |GraphCopy| and |PlaneGraphCopy| maintain copies
of graphs. Implementation of layout algorithms using this mechanism
can manipulate the graph copy in certain ways, e.g., inserting
dummy edges or splitting edges. Afterwards, the layout of the graph copy is
converted automatically in a respective layout for the input graph

The parameter |use_copy| of the constructor of |\Mname| decides if
such a mechanism is used. Layout algorithms operating on plane graphs
have an extended interface in order to pass a plane graph directly. Such an
algorithm must set the general postcondition |planar_layout|.
An implementation must override the respective
\_call method depending on the graph type used. This depends on
the parameter |use_copy| and the general postcondition |planar_layout|.
The following table shows which {\sl\_call} method has to be overridden:

\begin{tabular}{cc|l}
|use_copy|  & |planar_layout| & overridable \\\hline
|false| & no  & {\sl \_call(const graph \&G, LayoutInterface \&A)} \\
|true|  & no  & {\sl \_call(GraphCopy \&GC, LayoutInterface \&A)} \\
|false| & yes & {\sl \_call(const graph \&G, face f, LayoutInterface \&A)} \\
|true|  & yes & {\sl \_call(PlaneGraphCopy \&GC, face f, LayoutInterface \&A)}
\end{tabular}
}*/

public:

	// the input and output parameters of a layout algorithm module
	static const int in_graph;
	static const int out_layout;

	/*{\Minitialization}*/

	LayoutModule (bool use_copy = false);
	/*{\Mcreate initializes a layout algorithm. Graph copies are used
	iff |use_copy| is true.}*/

	virtual ~LayoutModule () { }


	/*{\Moperations 1.2 3.5}*/

	/*{\Mtext       
	\headerline{a) Standard Interface} }*/

	virtual bool check (const leda_graph& G, leda_string& message);
	/*{\Mop returns |true| if $G$ satisfies the preconditions of |\Mvar|.
	Otherwise, |false| is returned, and |message| contains the
	error message.}*/

	virtual void call (const leda_graph& G, LayoutInterface& A);
	/*{\Mfunop calls the layout algorithm for graph $G$ with layout
	interface $A$.}*/

	/*{\Mtext       
	The following methods are only supported by a planar layout module,
	that is a layout module with general postcondition |planar_layout|,
	i.e., $|planar_layout|\in|post|(0)(\emptyset)$. }*/

	virtual void call (const leda_graph& G, leda_face f, LayoutInterface& A);
	/*{\Mop calls |\Mvar| for graph $G$ with layout interface
	|A|. \precond $G$ represents a planar map, and $f$ is |nil|
	or a face in $G$.}*/

	virtual void call (PlaneGraphCopy& PG, leda_face f, LayoutInterface& A);
	/*{\Mop calls |\Mvar| for planarized graph $PG$ with layout
	buffer |A|. \precond $f$ is |nil| or a face in $PG$.}*/


	/*{\Mtext       
	\bigskip
	\headerline{b) Step by Step Animation}
	The following methods are only supported by a layout module
	with general postcondition |animation|,
	i.e., $|animation|\in|post|(0)(\emptyset)$. }*/

	bool start_animation (const leda_graph& G, LayoutInterface& A);
	/*{\Mop initializes step by step animation for graph $G$ with
	layout interface $A$. Must be called before $next_step$ is used. Returns |false|,
	if animation is not possible.}*/

	bool next_step ();
	/*{\Mop executes the next step of the animation. Returns |false|,
	if no more steps are possible, i.e., animation is finished.}*/

	void stop_animation ();
	/*{\Mop finish explicitly animation.}*/


protected:

	/*{\Mtext       
	\bigskip
	\headerline{c) Implementation of the Module Call}
	}*/

	bool use_copy () {
		return the_use_copy;
	}

	virtual void _call (const leda_graph& G, LayoutInterface& A);
	/*{\Mop implements the computation of the layout for graph $G$
	and layout interface $A$.}*/

	virtual void _call (GraphCopy& GC, LayoutInterface& A);
	/*{\Mop implements the computation of the layout for graph copy
	|GC| and layout interface $A$.}*/

	virtual void _call (const leda_graph& G, leda_face f, LayoutInterface& A);
	/*{\Mop implements the computation of the layout for graph $G$,
	external face $f$ and layout interface $A$. $f$ may be |nil|
	indicating that the algorithm selects the external face.}*/

	virtual void _call (PlaneGraphCopy& PG, leda_face f, LayoutInterface& A);
	/*{\Mop implements the computation of the layout for planarized
	graph $PG$, external face $f$ and layout buffer $A$. $f$ may be
	|nil| indicating that the algorithm selects the external face.}*/


	/*{\Mtext       
	\bigskip
	\headerline{d) Implementation of Animation}
	The following methods are called if const graph references are used
	(|use_copy| is |false|) and |\Mvar| is not a planar layout module:
	}*/

	virtual bool _start_animation (const leda_graph& G, LayoutInterface& A);
	/*{\Mop called to initialize step by step animation for graph $G$
	and layout interface $A$. Returns |true| if animation is possible,
	|false| otherwise.}*/

	virtual bool _next_step (const leda_graph& G, LayoutInterface& A);
	/*{\Mop called to perform the next step of the animation for graph $G$
	and layout interface $A$. Returns |false|
	if this was the last step, |true| otherwise.}*/

	virtual void _stop_animation (const leda_graph& G, LayoutInterface& A);
	/*{\Mop called after the animation has finished (by request of the client
	or the algorithm).}*/

	
	/*{\Mtext\medskip
	The following methods are called if graph copies are used
	(|use_copy| is |true|).
	}*/

	virtual bool _start_animation (GraphCopy& GC, LayoutInterface& A);
	/*{\Mop called to initialize step by step animation for graph copy |GC|
	and layout interface $A$. Returns |true| if animation is possible,
	|false| otherwise.}*/

	virtual bool _next_step (GraphCopy& GC, LayoutInterface& A);
	/*{\Mop called to perform the next step of the animation for graph copy |GC|
	and layout interface $A$. Returns |false|
	if this was the last step, |true| otherwise.}*/

	virtual void _stop_animation (GraphCopy& GC, LayoutInterface& A);
	/*{\Mop called after the animation has finished (by request of the client
	or the algorithm).}*/

private:
	void _finish_call(LayoutInterface &A, LayoutInterface *ap);


	// data for step by step animation
	const leda_graph *step_graph;
	GraphCopy        *step_copy_graph;
	LayoutInterface  *step_interface;
	GraphCopyLI      *step_copy_li;
	bool running_step;

	bool the_use_copy;
};



#endif
