//-------------------------------------------------------------------
//
//                         LeveledGraph.h
//
//    author: Carsten Gutwenger
//    date:   july 1998
//-------------------------------------------------------------------


#ifndef _AGD_LEVELED_GRAPH_H
#define _AGD_LEVELED_GRAPH_H


#include <LEDA/graph.h>
#include <LEDA/map.h>


/*{\Manpage {LeveledGraph} {} {Leveled Graphs} {LG} }*/

/*{\Mbaseclasses

\begin{hierarchy}{6.5}{2.5}
\hdclass{0}{1.5}{5}{graph}
\arhook{5}{2}{6}
\hdclass{1.5}{0}{5}{LeveledGraph}
\end{hierarchy}
}*/

class LeveledGraph : public leda_graph {

/*{\Mdefinition
An instance |\Mvar| of type |\Mname| is a graph $G=(V,E)$ with an injective
mapping $|level|:V\rightarrow \{0,1,2,\ldots\}$. We call the set of nodes
$L_i := \{ v \mid |level|(v)=i\}$ the $i$th level of |\Mvar|. The number
of levels of |\Mvar| is $\max \{i\mid L_i\neq\emptyset\}+1$ if $V\neq\emptyset$,
and $0$ otherwise.
}*/

public:

	/*{\Mcreation}*/

	LeveledGraph();
	/*{\Mcreate creates an instance |\Mvar| of type |\Mname| initialized
	to an empty graph.}*/

	LeveledGraph(const LeveledGraph &LG);

	virtual ~LeveledGraph() { }

	LeveledGraph& operator=(const LeveledGraph &LG);

	/*{\Moperations}*/

	int number_of_levels() const {
		return _max_level+1;
	}
	/*{\Mop returns the number of levels of |\Mvar|.}*/

	int level(leda_node v) const {
		return _level[v];
	}
	/*{\Mop returns the index $i$ of the level $L_i$ to which $v$ belongs.}*/

	void level(leda_node v, int i);
	/*{\Mop changes the level to which $v$ belongs to $L_i$.}*/

	const leda_list<leda_node> &nodes(int i) const {
		return _nodes[i];
	}
	/*{\Mop returns the list of nodes on level $L_i$.}*/

	leda_node new_node (int i);
	/*{\Mop adds a new node on level $L_i$ to |\Mvar| and returns it.}*/

	leda_node new_node() {
		return leda_graph::new_node();
	}

protected:

	void post_new_node_handler(leda_node v);
	void pre_del_node_handler(leda_node v);

private:

	void _init(const LeveledGraph &LG);

	bool _set_level;
	int  _max_level;
	leda_node_map<int>       _level;
	leda_node_map<list_item> _pos_in_nodes;
	leda_map<int,leda_list<leda_node> > _nodes;
};

/*{\Mimplementation
All operations take expected time $O(1)$.
}*/


#endif
