//-------------------------------------------------------------------
//
//                      MixedModelLayout.h
//
//    author: Carsten Gutwenger
//    date:   august 1997
//-------------------------------------------------------------------


#ifndef _AGD_MIXED_MODEL_LAYOUT_H
#define _AGD_MIXED_MODEL_LAYOUT_H


#include <AGD/impl/IOPoints.h>
#include <AGD/GridLayoutModule.h>
#include <AGD/CanonicalOrder.h>
#include <AGD/AugmentModule.h>
#include <AGD/AGDModuleOption.h>



/*{\Manpage {MixedModelLayout} {} {The Mixed-Model Algorithm} {L} }*/

/*{\Mbaseclasses

\begin{hierarchy}{9.5}{5.5}
\hdclass{0}{4.5}{5}{AGDModule}
\arhook{5}{5}{6}
\hdclass{1.5}{3}{5}{LayoutModule}
\arhook{6.5}{3.5}{7.5}
\hdclass{2}{1.5}{6}{GridLayoutModule}
\arhook{8}{2}{9}
\hdclass{3.5}{0}{6}{MixedModelLayout}
\end{hierarchy}
\hfill\agdeps{mixed_model.epsi}{5.5cm}
}*/

class MixedModelLayout : public GridLayoutModule {

/*{\Mdefinition
The class |\Mname| represents the layout algorithm Mixed-Model
(Kant~\cite{Kan96})
extented by a generalization to connected planar graphs
(Gutwenger and Mutzel~\cite{GM97}).

This algorithm draws a $d$-planar graph |G| on a grid such that every
edge has at most three bends and the minimum angle between two
edges is at least $\frac{2}{d}$ radians. |G| may not contain self-loops
or mulitple edges. |G| must be triconnected if the option |co_type| is
|co_triconnected|, and connected if it is |co_biconnected|.
The grid size is at most $(2n-6) \times (3n-9)$, the number of bends is
at most $5n-15$, and every edge has length $O(n)$ if |G| has $n$ nodes.

\horizline
\commoninfo{Mixed-Model}{Mixed-Model (Kant,...)}{Goos Kant, ...}{Carsten Gutwenger}
{march 1997}{2.1}

\headerline{Pre- and Postcondition}
\begin{tabular}{lcl}
precondition & $=$ & $\left\{
\begin{array}{l}
 \{\, |planar|,|simple|,|no self loops| \,\} \\
\mbox{ {\hspace{3.5cm}\bf if} |co_type|}=|co_biconnected|\\
\{\, |planar|,|triconnected|,|simple|,|no self loops| \,\} \\
\mbox{ {\hspace{3.5cm}\bf if} |co_type|}=|co_triconnected|
\end{array}\right. $ \\
postcondition(|PRE|) & $=$ & $\{\, |no crossings| \,\}$
\end{tabular}\\

\headerline{Exchangeable Modules}
Instances of type |\Mname| provide the following module options:
\begin{options}
\item {\bf augmenter}\\
an |AugmentModule| option used for planar biconnected augmentation.

\begin{tabular}{ll}
guaranteed precondition: & |planar|, |simple|, |no self loops|, |directed|\\
required postcondition: & |planar|, |biconnected|, |simple|, |no self loops|\\
initial module: & |PlanAug|
\end{tabular}
\end{options}

\headerline{Optional Parameters}
Instances of |\Mname| provide the following optional parameters:
\begin{options}
\item |double| {\bf base\_ratio} $= 0.33$\\
a double value determining, how many nodes are drawn on the base line.
If |base_ratio| is $r$ and the outerface is $F$, the number of nodes
on the base line is $\min\left(2,\max\left(\ell, r\Labs{F}\right)\right)$,
where $\ell = \max\{k \mid v_1,\ldots,v_k$ is a path on $F_{out}$ with
$(v_i,v_j)\notin E$ for all $j\neq i+1 \}$.

\item |bool| {\bf do\_postprocessing} = |true|\\
a boolean value determining if a postprocessing step is used
improving the positions of outpoints.

\item |CanonicalOrderType| {\bf co\_type} = |co_biconnected|\\
an enumeration value of type |CanonicalOrderType| determining the
canonical ordering used in the algorithm (see |CanonicalOrder|).
\end{options}
\horizline
}*/

public:

	/*{\Mcreation L 5}*/

	MixedModelLayout ();
	/*{\Mcreate creates an instance |\Mvar| of type |\Mname|.}*/

	virtual ~MixedModelLayout() { }


	/*{\Moperations 1.5 2.5}*/

	leda_string name () const {
		return "Mixed-Model";
	}

	leda_string long_name () const {
		return "Mixed-Model (Kant,...)";
	}

	leda_string author () const {
		return "Goos Kant, ...";
	}

	leda_string impl_author () const {
		return "Carsten Gutwenger";
	}

	leda_string impl_version () const {
		return "2.1";
	}

	leda_string impl_date () const {
		return "july 1997";
	}


	double base_ratio() const {
		return the_base_ratio;
	}

	void base_ratio (double br) {
		the_base_ratio = br;
	}

	bool prefer_nodes() const {
		return the_prefer_nodes;
	}

	void prefer_nodes (bool pn) {
		the_prefer_nodes = pn;
	}

	bool do_postprocessing() const {
		return the_do_postprocessing;
	}

	void do_postprocessing (bool pp) {
		the_do_postprocessing = pp;
	}

	CanonicalOrderType co_type() const {
		return the_co_type;
	}

	void co_type (CanonicalOrderType ct) {
		the_co_type = ct;
		if (ct == co_biconnected) {
			rem_precondition (key::triconnected);
		} else {
			add_precondition (key::triconnected);
		}
	}

	AGDModule *clone () const;

	/*{\Moptions warnings=no}*/

	/*{\Mtext       
	\headerline{Generic Interface (Inherited Methods)}
	The detailed description of these methods can be found in the
	manual entries of the base classes (|LayoutModule|, |GridLayoutModule|).
	\setlength{\entrysep}{-1ex}}*/

	/* inherited
	bool check (const graph& G, string& message);
	*/
	/*{\Mop}*/

	/* inherited
	void call (const graph& G, LayoutInterface& A);
	*/
	/*{\Mop}*/

	/* inherited
	virtual void call (const graph& G, face f, LayoutInterface& A);
	*/
	/*{\Mop}*/

	/* inherited
	virtual void call (PlaneGraphCopy& PG, face f, LayoutInterface& A);
	*/
	/*{\Mop}*/

	/* inherited
	void call_grid (const graph& G, GridLayout &gl);
	*/
	/*{\Mop}*/

	/* inherited
	void call_grid (const graph& G, face f, GridLayout &gl);
	*/
	/*{\Mop}*/

	/* inherited
	void call_grid (PlaneGraphCopy& PG, face f, GridLayout &gl);
	*/
	/*{\Mop}*/

	/* inherited
	IRect get_grid_area ();
	*/
	/*{\Mop}*/

	/* inherited
	void set_coordinate_mapper (const GridCoordinateMapper& gcm);
	*/
	/*{\Mop}*/
	
	/*{\Mtext \setlength{\entrysep}{0.5ex}}*/

	AGD_DECLARE_MODULE (AugmentModule, augmenter)

protected:
	 void _call_grid (PlaneGraphCopy& PG, leda_face F, GridLayout &gl);

private:
	bool the_prefer_nodes;
	bool the_do_postprocessing;
	double the_base_ratio;
	CanonicalOrderType the_co_type;

	enum Outpoint_Style { OS_left, OS_middle, OS_right };

	int out (leda_node v) {
		return iop.out_points(v).length();
	}
	int in (leda_node v) {
		return iop.in_points(v).length();
	}

	int out_l (leda_node v) {
		int l = out(v)-1;
		return (l < 0) ? 0 : l/2;
	}
	int out_r (leda_node v) {
		return out(v) / 2;
	}
	int in_l (leda_node v) {
		int l = in(v)-3;
		return (l < 0) ? 0 : l/2;
	}
	int in_r (leda_node v) {
		int l = in(v)-2;
		return (l < 0) ? 0 : l/2;
	}

	// bounding box of node v
	int bb_upper (leda_node v);
	int bb_lower (leda_node v);
	int bb_ll (leda_node v);
	int bb_lr (leda_node v);
	int bb_ul (leda_node v);
	int bb_ur (leda_node v);

	void set_out_coord (leda_node v, Outpoint_Style style);
	void postprocessing (PlaneGraphCopy &G, leda_node_map<int>& x,
	leda_node_map<int>& y);

	void init_bends (PlaneGraphCopy &G, leda_node_map<int>& x,
		leda_node_map<int>& y, leda_edge_map<IPolyline>& bends,
		LmcOrder& lmc);

	void perform_backshift (leda_node from, leda_node to, LmcOrder& lmc,
		leda_node_map<int>& x, leda_node_map<int>& x_rel);
	void compute_comp (PlaneGraphCopy &G, LmcOrder& lmc);
	int horiz_height (PlaneGraphCopy& G, leda_edge e,
		leda_node_map<int>& x, leda_node_map<int>& y);
	bool exists (PlaneGraphCopy& G, leda_edge e);

	IOPoints                            iop;
	leda_array<int>                     space;
	leda_node_map<int>                  shift, comp;
	leda_list<pair_v_d>                 dummy;
	leda_node_map<leda_list<pair_v_d> > compL;
	leda_node_map<leda_node>            prev, next, upper;
	leda_list<leda_node>                S;
	leda_node_map<int>                  the_bb_upper;

	void compute_coordinates (PlaneGraphCopy &G,
		LmcOrder& lmc,
		leda_node_map<int>&  x,
		leda_node_map<int>&  y,
		leda_edge_map<IPolyline>& bends);
};


#endif
