
#line 96 "Ovid.lw"
#ifndef _AGD_OVID_H
#define _AGD_OVID_H

#include <LEDA/face_map.h>
#include <AGD/OrthoShape.h>

/*{\Manpage {Ovid} {} {The Orthogonal Visibility Drawing Base Class} {L} }*/

/*{\Mbaseclasses
\arhook{9.5}{2}{10.5}
\hdclass{5}{0}{6}{Ovid}
\end{hierarchy}
}*/

class Ovid {

/*{\Mdefinition
\horizline

\commoninfo{Ovid}{Ovid}{Gunnar W. Klau}{Gunnar W. Klau}{III 1998}{0.3}

\headerline{Optional Parameters} \reversemarginpar
Instances of |\Mname| provide the following optional parameters:
\begin{options}
\item |bool| {\bf check} $= true$\\
If set, the checking routines are activated. Especially the correctness of the
orthogonal representation is verified, which is helpful for debugging and
guarantees the correctness of the computation.  
\item |int| {$\mathbf k$} $= 2$\\
Determines the maximal set of possible angles, i.e. every angle $\alpha$ that 
occurs in the representation holds $\alpha = \frac{i}{2^k}\cdot 360^\circ$ with $i \in \{-2^k, \ldots, 0, \ldots, 2^k\}$.
\item |bool| {\bf allow\_bends} $ = true$\\
Makes it possible to place bends on edges. 
\item |bool| {\bf allow\_nonpos\_deg} $= true$\\
For graphs $G$ with maximal vertex degree $\Delta(G) > 4$ this variable
has to be set. It is possible to set it for 4--planar graphs, too. Then
the number of bends will possibly decrease at the cost of losing the property
that there is at most one edge per side.
\item |bool| {\bf forbid\_small\_zero\_deg} $= \ldots$\\
\marginpar{\fbox{\tiny NOT IMPLEMENTED.}} Causes zero degrees only to appear at high degree nodes.
\item |bool| {\bf forbid\_cross\_zero\_deg} $= true$\\
\marginpar{\fbox{\tiny NOT IMPLEMENTED.}} Forbids zero degrees at dummy vertices 
standing for crossings in the drawing. Should always be true.
\item |bool| {\bf allow\_neg\_deg} $= true$\\
Extends the set of possible angle values by $\{-90^\circ, -180^\circ, -270^\circ, -360^\circ\}$.
\item |bool| {\bf restrict\_to\_L\_shape} $= true$\\
Only one $-90^\circ$ angle can appear at each vertex. Thus their form is
restricted to L--shape.
\item |bool| {\bf contains\_cages} $= false$\\
Some algorithms replace high degree vertices by faces--internally called {\em faces}--in order to proceed with a representative graph $G'$ with maximal degree
$\Delta(G') \leq 4$. In this case, the cages have to get rectangular shape which can be ensured by modifying the network. Such special faces are marked by a
[[face_map<bool]].
\end{options} \horizline }*/
 
  public:
  /*{\Mcreation}*/
    
  Ovid();
  /*{\Mcreate creates an instance |\Mvar| of type |\Mname|.}*/
  
  virtual ~Ovid() {}
  
  /*{\Moperations 1.5 2.5}*/
  
  leda_string name() const { return leda_string("Orthogonal visibility representation"); }
  
  leda_string long_name() const { 
    return leda_string("Orthogonal visibility representation (fixed topology)"); 
  }
  
  leda_string author() const { return leda_string("Gunnar W. Klau"); }
  
  virtual leda_string impl_author() const { return leda_string("Gunnar W. Klau"); }
  
  virtual leda_string impl_version() const { return leda_string("0.3"); }
  
  virtual leda_string impl_date() const { return leda_string("III 1998"); }

  /*{\Moptions warnings=no}*/

  //virtual bool check(const graph& G, string& message); 
  /*hier noch die abh"angigkeiten der options pr"ufen. */
  
  /*{\Mop returns |true| if $G$ satisfies the precondition of |\Mvar|.
  Otherwise, |false| is returned, and |message| contains the
  error message.}*/

  bool adopt() const { return _adopt; }
  /*{\Mop returns the optional parameter |adopt|. }*/ 

  void adopt(bool a) { _adopt = a; }
  /*{\Mop sets the parameter |adopt| to |a|. }*/ 

  int number_of_bends() const { return _number_of_bends; }
  /*{\Mop returns the number of bends. }*/



#line 207 "Ovid.lw"
private:


  enum n_type { high, low, inner, outer };
  enum a_type { angle, bend, cage_bend, neg_angle, cage_angle };
  
  inline int pow2(int n) { return 1 << n; }

protected:
  
  leda_edge_map<int> s;		
  leda_edge_map<int> ang;

  int _k;

  bool _allow_bends, _allow_nonpos_deg, _allow_neg_deg,
    _forbid_small_zero_deg, _forbid_cross_zero_deg, 
    _restrict_to_L_shape, _contains_cages, _doorley;

  int _number_of_bends;
  int _number_of_zero_degs;
  bool _check, _debug;
  bool _adopt; 	// this should be in a mother class for planar algs.
		// if set, the givewn planar embeding should be adopted.
		// this has to be realized by the LayoutInterface

  face_map<bool> _is_cage;

  void compute_optimal_shape(OrthoShape& H);

  inline void INF(leda_string s) { cout << s; cout. flush(); }
  inline void DONE() { cout << "done. " << endl; }
  inline void LINE() { 
    cout << "----------------------------------------------------" << endl; 
  }

};

/*{\Mimplementation
Creation of the network and computation of a min cost flow takes time
$O(\ldots)$.
}*/

#endif

