//-------------------------------------------------------------------
//
//                    PlanarizationGridLayout.h
//
//    author: Carsten Gutwenger
//    date:   may 1998
//-------------------------------------------------------------------


#ifndef _AGD_PLANARIZATION_GRID_LAYOUT_H
#define _AGD_PLANARIZATION_GRID_LAYOUT_H


#include <AGD/GridLayoutModule.h>
#include <AGD/PlanarizerModule.h>
#include <AGD/AGDModuleOption.h>


/*{\Manpage {PlanarizationGridLayout} {} {Planarization on the Grid} {L} }*/

/*{\Mbaseclasses

\begin{hierarchy}{9.5}{5.5}
\hdclass{0}{4.5}{5}{AGDModule}
\arhook{5}{5}{6}
\hdclass{1.5}{3}{5}{LayoutModule}
\arhook{6.5}{3.5}{7.5}
\hdclass{2}{1.5}{6}{GridLayoutModule}
\arhook{8}{2}{9}
\hdclass{2}{0}{7.5}{PlanarizationGridLayout}
\end{hierarchy}
\hfill
\agdeps{planarization.epsi}{5.5cm}
}*/

class PlanarizationGridLayout : public GridLayoutModule {

/*{\Mdefinition
The class |\Mname| represents a general framework for drawing with planarization
on the grid.
It transforms a general graph into a planar graph using a planarization module, and
draws the resulting planar graph using a grid layout algorithm for planar graphs.
The planarization and planar layout algorithm are exchangeable modules allowing
to use |\Mname| with different methods for planarization and planar drawing.

\horizline
\commoninfo{Planarization}{Grid Planarization}{}
{Carsten Gutwenger}{may 1998}{1.0}

\headerline{Pre- and Postcondition}
\begin{tabular}{lcl}
precondition & $=$ & $\{\, |simple|,|no self loops| \,\}$ \\
postcondition(|PRE|) & $=$ & $\emptyset$
\end{tabular}

\headerline{Exchangeable Modules}
Instances of type |\Mname| provide the following module options:
\begin{options}
\item {\bf planarizer}\\
a |PlanarizerModule| option used to compute the planarization of the input graph.

\begin{tabular}{ll}
guaranteed precondition: & $\{\, |simple|, |no self loops|, |directed| \,\}$\\
required postcondition: & $\{\, |planar|, |simple|, |no self loops| \,\}$\\
initial module: & |PlanarSubgraph|
\end{tabular}

\item {\bf planar\_layouter}\\
a |GridLayoutModule| option used to draw the planarized graph.

\begin{tabular}{ll}
guaranteed precondition: & $\{\, |simple|, |no self loops|, |planar| \,\}$\\
required postcondition: & $\{ |planar layout| \} , \{\, |no crossings| \,\}$\\
initial module: & |Quod|
\end{tabular}
\end{options}
\horizline
}*/

AGD_DECLARE_MODULE(PlanarizerModule,planarizer)

protected:
	AGDModuleOption<GridLayoutModule> planar_layouter;
public:
	void set_planar_layouter (const GridLayoutModule& M);

public:

	PlanarizationGridLayout();
	~PlanarizationGridLayout() { }

	/*{\Moperations}*/

	leda_string name () const {
		return "Planarization";
	}

	leda_string long_name () const {
		return "Grid Planarization";
	}

	leda_string impl_author () const {
		return "Carsten Gutwenger";
	}

	leda_string impl_version () const {
		return "1.0";
	}

	leda_string impl_date () const {
		return "may 1998";
	}

	AGDModule *clone () const;

	/*{\Moptions warnings=no}*/

	/*{\Mtext       
	\headerline{Generic Interface (Inherited Methods)}
	The detailed description of these methods can be found in the
	manual entries of the base classes (|LayoutModule|, |GridLayoutModule|).
	\setlength{\entrysep}{-1ex}}*/

	/* inherited
	bool check (const graph& G, string& message);
	*/
	/*{\Mop}*/

	/* inherited
	void call (const graph& G, LayoutInterface& A);
	*/
	/*{\Mop}*/

	/* inherited
	void call_grid (const graph& G, GridLayout &gl);
	*/
	/*{\Mop}*/

	/* inherited
	IRect get_grid_area ();
	*/
	/*{\Mop}*/

	/* inherited
	void set_coordinate_mapper (const GridCoordinateMapper& gcm);
	*/
	/*{\Mop}*/

	/*{\Mtext \setlength{\entrysep}{0.5ex}}*/

protected:

	void _call_grid (const leda_graph& G, GridLayout &gl);
};


#endif
