
#ifndef _AGD_QUOD_H
#define _AGD_QUOD_H

#include <AGD/GridLayoutModule.h>
#include <AGD/Ortho.h>

/*{\Manpage {Quod} {} {The Quasi Orthogonal Drawing Algorithm} {L} }*/

/*{\Mbaseclasses
\begin{hierarchy}{9.5}{5.5}
\hdclass{0}{4.5}{5}{AGDModule}
\arhook{5}{5}{6}
\hdclass{1.5}{3}{5}{LayoutModule}
\arhook{6.5}{3.5}{7.5}
\hdclass{2}{1.5}{6}{GridLayoutModule}
\arhook{8}{2}{9}
\hdclass{3.5}{0}{6}{Quod}
\end{hierarchy}
\hfill\agdeps{Quod.eps}{5.5cm}
}*/

class Quod : public Ortho {

/*{\Mdefinition
The class |\Mname| is an implementation of an algorithm extending
Tamassia's bend minimizing algorithm to the class of planar graphs with
arbitrary degree. Its implementation is very similar to the one for the [[Ortho]] class, and also the operations it provides are identical. The only difference to the user is the extended applicability and the slightly different output style.

\horizline

\commoninfo{Quod}{Quasi Orthogonal Drawing Algorithm}{Gunnar W. Klau}{Gunnar W. Klau}{June 1998}{0.3}

\headerline{Pre- and Postcondition}

\begin{tabular}{lcl}
precondition & $=$ & $\{\,$ {\em planar, simple, no self loops} $\,\}$ \\
postcondition(|PRE|) & $=$ & $\{\,$ {\em no crossings} $\,\}$
\end{tabular}\\

\headerline{Optional Parameters}
Instances of |\Mname| provide the following optional parameters:
\begin{options}
\item |bool| {\bf dissect} $= true$\\ 
Truth value indicating if the faces should be dissected in rectangles during the compaction phase. This is the original method proposed in \cite{t-eggmn-87}. Switching this off may lead to violated embeddings in the current version.
\item |bool| {\bf check} $= true$\\
If set, the checking routines are activated. Especially the 
orthogonal representation is verified, which is helpful for debugging and
certificates the correctness of the computation.  
\end{options} \horizline }*/
  
public:
  /*{\Mcreation}*/
    
  Quod();
  /*{\Mcreate creates an instance |\Mvar| of type |\Mname|.}*/
  
  virtual ~Quod() {}
  
  /*{\Moperations 1.5 2.5}*/
  
  leda_string name() const { return leda_string("Quod"); }
  
  leda_string long_name() const { 
    return leda_string("Quasi orthogonal drawing algorithm (fixed topology)"); 
  }
  
  leda_string author() const { return leda_string("Gunnar W. Klau"); }
  
  virtual leda_string impl_author() const { return leda_string("Gunnar W. Klau"); }
  
  virtual leda_string impl_version() const { return leda_string("0.3"); }
  
  virtual leda_string impl_date() const { return leda_string("June 1998"); }

  AGDModule* clone() const;

  bool dissect() const { return _dissect; }
  /*{\Mop returns the optional parameter |dissect|. }*/ 

  void dissect(bool d) { _dissect = d; }
  /*{\Mop sets the parameter |dissect| to |d|. }*/ 

  bool verify() const { return _check; }
  /*{\Mop returns the verify status. }*/ 

  void verify(bool c) { _check = c; }
  /*{\Mop sets the verify status to |c|. }*/ 

  bool variant() const { return _doorley; }
  /*{\Mop \ldots }*/ 

  void variant(bool d) { _doorley = d; }
  /*{\Mop \ldots }*/ 



  /*{\Moptions warnings=no}*/

	/*{\Mtext       
	\headerline{Generic Interface (Inherited Methods)}
	The detailed description of these methods can be found in the
	manual entries of the base classes (|LayoutModule|, |GridLayoutModule|).
	\setlength{\entrysep}{-1ex}}*/

	/* inherited
	bool check (const graph& G, string& message);
	*/
	/*{\Mop}*/

	/* inherited
	void call (const graph& G, LayoutInterface& A);
	*/
	/*{\Mop}*/

	/* inherited
	virtual void call (const graph& G, face f, LayoutInterface& A);
	*/
	/*{\Mop}*/

	/* inherited
	virtual void call (PlaneGraphCopy& PG, face f, LayoutInterface& A);
	*/
	/*{\Mop}*/

	/* inherited
	void call_grid (const graph& G, GridLayout &gl);
	*/
	/*{\Mop}*/

	/* inherited
	void call_grid (const graph& G, face f, GridLayout &gl);
	*/
	/*{\Mop}*/

	/* inherited
	void call_grid (PlaneGraphCopy& PG, face f, GridLayout &gl);
	*/
	/*{\Mop}*/

	/* inherited
	IRect get_grid_area ();
	*/
	/*{\Mop}*/

	/* inherited
	void set_coordinate_mapper (const GridCoordinateMapper& gcm);
	*/
	/*{\Mop}*/
	
	/*{\Mtext \setlength{\entrysep}{0.5ex}}*/
  
protected:
         
  void _call_grid(PlaneGraphCopy& G, leda_face f_0, GridLayout& I);

};

/*{\Mimplementation
The computation of a quasi orthogonal grid embedding by algorithm [[Quod]] 
takes time $O(n^2 \log n \log n)$ where $n$ is the number of nodes of the graph to be drawn. The running time is asymptotically equal to the time to compute
an [[Ortho]] layout.
}*/

#endif



