//-------------------------------------------------------------------
//
//                        SugiyamaLayout.h
//
//
//    author: Carsten Gutwenger
//            Michael Krger
//    date:   february 1998
//-------------------------------------------------------------------


#ifndef _AGD_SUGIYAMA_LAYOUT1_H
#define _AGD_SUGIYAMA_LAYOUT1_H


#include <AGD/LayoutModule.h>
#include <AGD/AGDModuleOption.h>
#include <AGD/RankAssignment.h>
#include <AGD/HierarchyLayoutModule.h>
#include <AGD/TwoLayerCrossMin.h>


/*{\Manpage {SugiyamaLayout} {} {Sugiyama's Algorithm} {L} }*/

/*{\Mbaseclasses

\begin{hierarchy}{8}{4}
\hdclass{0}{3}{5}{AGDModule}
\arhook{5}{3.5}{6}
\hdclass{1.5}{1.5}{5}{LayoutModule}
\arhook{6.5}{2}{7.5}
\hdclass{2}{0}{6}{SugiyamaLayout}
\end{hierarchy}
\hfill\agdeps{sugiyama.epsi}{4cm}
}*/

class SugiyamaLayout : public LayoutModule {

/*{\Mdefinition
The class |\Mname| represents a general framework for drawing graphs with
the hierarchical drawing method suggested by Sugiyama, Tagawa, and
Toda~\cite{stt-mvuhs-81}. This method works in three phases:
\begin{enumerate}
\item We partition the nodes into levels using a |RankAssignment| module.
Then, we build a proper hierarchy, that is, we reverse and split edges until
each edge is directed downwards and connects two nodes on neighbored layers.
The newly introduced nodes are called {\it dummy nodes} and represent
edge bends in the final drawing.
\item We reorder the nodes on each layer in order to reduce the number
of edge crossings. Therefore, we iteratively call a |TwoLayerCrossMin|
module.
\item We call a |HierarchyLayoutModule| to compute the final layout of the
hierarchy. This module assigns coordinates to nodes and bend points to edges.
\end{enumerate}

\horizline
\commoninfo{Sugiyama}{Sugiyama}{Sugiyama, Tagawa, Toda}
{Carsten Gutwenger, Michael Kr\"uger}{march 1998}{2.0}

\headerline{Pre- and Postcondition}
\begin{tabular}{lcl}
precondition & $=$ & $\{\, |simple|,|no self loops|,|no reversals| \,\}$ \\
postcondition(|PRE|) & $=$ & $\emptyset$
\end{tabular}

\headerline{Exchangeable Modules}
Instances of type |\Mname| provide the following module options:
\begin{options}
\item {\bf ranking}\\
a |RankAssignment| option used to compute the ranking.

\begin{tabular}{ll}
guaranteed precondition: & $\{\, |simple|, |no self loops|, |no reversals|, |directed| \,\}$\\
required postcondition: & $\emptyset$\\
initial module: & |HierarchicalRanking|
\end{tabular}

\item {\bf cross\_min}\\
a |TwoLayerCrossMin| option used in the crossing reduction phase.

\begin{tabular}{ll}
guaranteed precondition: & $\{\, |simple|, |no self loops|, |no reversals|, |directed| \,\}, \emptyset$\\
required postcondition: & $\emptyset$\\
initial module: & |BarycenterHeuristic|
\end{tabular}

\item {\bf comp\_coord}\\
a |HierarchyLayoutModule| option used to compute the final layout.

\begin{tabular}{ll}
guaranteed precondition: & $\{\, |simple|, |no self loops|, |no reversals|, |directed| \,\}, \emptyset$\\
required postcondition: & $\emptyset$\\
initial module: & |FastHierarchyLayout|
\end{tabular}
\end{options}
\horizline

\headerline{Optional Parameters}
Instances of |\Mname| provide the following optional parameters:
\begin{options}
\item |int| {\bf runs} $= 3$\\
determines, how many times the crossing minimization is repeated.
Each repetition (except for the first) starts with randomly
permuted nodes on each layer. Deterministic behaviour can be achieved
by setting |runs| to $1$.
\end{options}
\horizline
}*/

	// declare module options for rank assignment
	AGD_DECLARE_MODULE(RankAssignment,ranking)

	// two layer crossing minimization
	AGD_DECLARE_MODULE(TwoLayerCrossMin,cross_min)

	// and computation of coordinates
	AGD_DECLARE_MODULE(HierarchyLayoutModule,comp_coord)

public:

	/*{\Mcreation}*/

	SugiyamaLayout();
	/*{\Mcreate creates an instance |\Mvar| of type |\Mname|.}*/

	/*{\Moperations}*/

	// common informations
	leda_string name() const {
		return "Sugiyama";
	}
	leda_string author() const {
		return "Sugiyama, Tagawa, Toda";
	}
	leda_string impl_author() const {
		return "Carsten Gutwenger, Michael Krger";
	}
	leda_string impl_version() const {
		return "2.0";
	}
	leda_string impl_date() const {
		return "march 1998";
	}

	// options
	int runs() const {
		return _runs;
	}

	void runs(int n) {
		_runs = n;
	}

	void ext_call (const leda_graph& G, leda_node_array<int>& rank,
		LayoutInterface& A);
	/*{\Mop calls |\Mvar| for graph $G$ with layout interface $A$.
	Uses the level assignment defined by |rank|.}*/

	void reduce_crossings (GraphCopy& GC, Hierarchy& H);
	/*{\Mop calls only the crossing reduction step for graph copy
	|GC| and hierarchy |H|.}*/


	AGDModule *clone() const;

	/*{\Moptions warnings=no}*/

	/*{\Mtext       
	\headerline{Generic Interface (Inherited Methods)}
	The detailed description of these methods can be found in the
	manual entries of the base class (|LayoutModule|).
	\setlength{\entrysep}{-1ex}}*/

	/* inherited
	virtual bool check (const graph& G, string& message);
	*/
	/*{\Mop}*/

	/* inherited
	void call (const graph& G, LayoutInterface& A);
	*/
	/*{\Mop}*/
	
	/*{\Mtext \setlength{\entrysep}{0.5ex}}*/

protected:

	void _call (GraphCopy& GC, LayoutInterface& A);

private:
	int _runs;

	int best_nr_crossings;

	void minimize_crossings(const leda_graph& G, Hierarchy& H);

	int traverse_top_down(const leda_graph& G, Hierarchy& H,
		TwoLayerCrossMin& minimizer);
	int traverse_bottom_up(const leda_graph& G, Hierarchy& H,
		TwoLayerCrossMin& minimizer);

};

/*{\Mexample
{\bf Example 1:} We use only the crossing reduction phase of the
Sugiyama algorithm with the Split heuristic and our own level assignment.
{\small\tt
\begin{verbatim}
#include <LEDA/graphwin.h>
#include <AGD/SugiyamaLayout.h>
#include <AGD/SplitHeuristic.h>
#include <AGD/DfsRanking.h>
#include <AGD/Hierarchy.h>
#include <AGD/GraphCopy.h>

void reduce crossings(graph& G)
{
  node_array<int> rank(G);

  // assign a non-negative level to each node in G
  // we simply use a rank assignment module
  DfsRanking().call(G,rank);

  // build the hierarchy
  GraphCopy GC(G);
  Hierarchy H(GC,rank);

  // call crossing minimization
  SugiyamaLayout sugiyama;
  sugiyama.set_cross_min(SplitHeuristic());
  sugiyama.reduce_crossings(GC,H);

  // the positions of the nodes (in GC) on each layer
  // can now be accessed via H
}
\end{verbatim}}

\bigskip
{\bf Example 2:} We compute a layout in a {\it GraphWin} with
our own level assignment.
{\small\tt
\begin{verbatim}
#include <LEDA/graphwin.h>
#include <AGD/SugiyamaLayout.h>
#include <AGD/DfsRanking.h>

void call_with_ranking(GraphWin& GW)
{
  graph& G = GW.get_graph();
  node_array<int> rank(G);

  // assign a non-negative level to each node in G
  // we simply use a rank assignment module
  DfsRanking().call(G,rank);

  // compute layout in GW
  GraphWinInterface A(GW);
  SugiyamaLayout sugiyama;
  sugiyama.ext_call(G,rank,A);
}
\end{verbatim}}

}*/

#endif
