//-------------------------------------------------------------------
//
//                       TutteLayout.h
//
//    author: David Alberts
//    date:   April 1997
//-------------------------------------------------------------------


#ifndef _AGD_TUTTE_LAYOUT_H
#define _AGD_TUTTE_LAYOUT_H


#include <AGD/LayoutModule.h>
#include <AGD/DRect.h>

/*{\Manpage {TutteLayout} {} {Tutte's Algorithm} {L} }*/

/*{\Mbaseclasses

\begin{hierarchy}{8}{4}
\hdclass{0}{3}{5}{AGDModule}
\arhook{5}{3.5}{6}
\hdclass{1.5}{1.5}{5}{LayoutModule}
\arhook{6.5}{2}{7.5}
\hdclass{3}{0}{5}{TutteLayout}
\end{hierarchy}
}*/

class TutteLayout : public LayoutModule
{

/*{\Mdefinition
The class |\Mname| represents the layout algorithm by
Tutte\cite{t-hdg-63}.

This algorithm draws a planar graph |G| straight-line
without crossings. It can also draw non-planar graphs.

\horizline
\commoninfo{Tutte}{Tutte}{Tutte}
{David Alberts}{April 1997}{}

\headerline{Pre- and Postcondition}
\begin{tabular}{lcl}
precondition & $=$ & $\{\, |connected|,|no self loops| \,\}$ \\
postcondition(|PRE|) & $=$ & $\{\, \mbox{|straight|-|line|} \,\}$
\end{tabular}

\headerline{Optional Parameters}
Instances of |\Mname| provide the following optional parameters:
\begin{options}
\item |DRect| {\bf bbox} = (0.0, 250.0, 0.0, 250.0)\\
a double rectangle determining the bounding box of the layout.
\end{options}\horizline
}*/

  public:

	/*{\Mcreation}*/

    TutteLayout();
	/*{\Mcreate creates an instance |\Mvar| of type |\Mname|.}*/

    ~TutteLayout() { }

  /*{\Moperations}*/

    leda_string name () const { return "Tutte"; }

    leda_string long_name () const { return "Tutte"; }

    leda_string author () const { return "Tutte"; }

    leda_string impl_author () const
    { return "David Alberts"; }

    leda_string impl_date () const
    { return "April 1997"; }


	DRect bbox () const {
		return the_bbox;
	}

	void bbox (const DRect& bb) {
		the_bbox = bb;
	}

	AGDModule *clone () const;

	/*{\Moptions warnings=no}*/

	/*{\Mtext       
	\headerline{Generic Interface (Inherited Methods)}
	The detailed description of these methods can be found in the
	manual entries of the base class (|LayoutModule|).
	\setlength{\entrysep}{-1ex}}*/

	/* inherited
	virtual bool check (const graph& G, string& message);
	*/
	/*{\Mop}*/

	/* inherited
	void call (const graph& G, LayoutInterface& A);
	*/
	/*{\Mop}*/
	
	/*{\Mtext \setlength{\entrysep}{0.5ex}}*/


	/*{\Mtext       
	\bigskip
	\headerline{c) Special Functionality} }*/

    bool do_tutte(const leda_graph& G, LayoutInterface& a,
		leda_list<leda_node>& fixed_nodes,
        leda_list<DPoint>& fixed_positions);
      /*{\Mop does the actual computation. |fixed_nodes| and |fixed_positions|
              contain the nodes with fixed positions.

              \precond All nodes in |fixed_nodes| are distinct and belong to
              |G|. The sizes of |fixed_nodes| and |fuxed_positions| match.
      }*/

  protected:

    void _call(const leda_graph& G, LayoutInterface& a);
	void _call (GraphCopy& GC, LayoutInterface& A)
	{ LayoutModule::_call (GC,A); }

  private:

    void set_fixed_nodes(const leda_graph& G, leda_list<leda_node>& nodes,
                         leda_list<DPoint>& pos, double radius = 1.0);
      // sets the positions of the nodes in a largest face of G in the
      // form of a regular k-gon with the prescribed radius. The
      // corresponding nodes and their positions are stored in nodes
      // and pos, respectively. G does not have to be planar!

    DRect the_bbox;
};

/*{\Mimplementation
  The algorithm solves a system
  of linear equations and takes $O(n^3)$ time.
}*/

#endif
