//-------------------------------------------------------------------
//
//                        TwoLayerCrossMin.h
//
//
//    author: Carsten Gutwenger
//    date:   february 1998
//-------------------------------------------------------------------


#ifndef _AGD_TWO_LAYER_CROSS_MIN_H
#define _AGD_TWO_LAYER_CROSS_MIN_H

#include <AGD/AGDModule.h>


class Layer;
class Hierarchy;

/*{\Manpage {TwoLayerCrossMin} {} {Two Layer Crossing Minimization} {M} }*/

/*{\Mbaseclasses

\begin{hierarchy}{6.5}{2.5}
\hdclass{0}{1.5}{5.5}{AGDModule}
\arhook{5.5}{2}{6.5}
\hdclass{1}{0}{6}{TwoLayerCrossMin}
\end{hierarchy}
}*/

class TwoLayerCrossMin : public AGDModule {

/*{\Mdefinition
Modules of type |\Mname| perform a two layer crossing minimization
in a node hierarchy |H|, that is, the nodes of a graph |G|
on a layer |L|, called the changeable layer, are reordered according
to a neighbored layer of |L|, called the fixed layer, in order
to reduce the number of edge crossings between these two layers.

Modules of type |\Mname| are used in a process, which
iteratively performs two layer crossing minimization
between two neighbored layers in order
to reduce the total number of edge crossings in the hierarchy.
For efficiency reasons, the class |\Mname| contains two
notification methods init and cleanup.
A user must first call the init method with the graph |G|
and the node hierarchy |H|. Then, the call method can be called
several times with layers belonging to |H|. Finally, the
cleanup method should be called to free resources.

\headerline{Implementing Two Layer Crossing Minimization Modules}

An implementation of a two layer crossing minimization module
must override the method call($G$,$L$), which gets as
input a graph $G$ and a layer $L$. The class |Layer| provides
methods to access the fixed and changeable layer, and to
manipulate the order of the nodes on the changeable layer.
The method init can be overridden to allocate or initialize
internal data structures, and the method cleanup can be overridden
to free resources allocated by init.

\headerline{Input and Output Parameters}

\begin{tabular}{lcc}
input parameter: & graph & $|in_graph|=1$ \\
 & layer & $|in_layer|=2$ \\
output parameter: & layer & $|out_layer|=1$
\end{tabular}

The integer constants |in_graph|, |in_layer| and |out_layer| can be used to denote
the positions of the input and output parameters.
}*/

public:

	// the input and output parameters of a two layer crossing
	// minimization module
	static const int in_graph, in_layer;
	static const int out_layer;

	/*{\Minitialization M 5}*/

	TwoLayerCrossMin () : AGDModule(2,1) { }

	/*{\Mcreate initializes a two layer crossing minimization module.}*/

	virtual ~TwoLayerCrossMin () { }


	/*{\Moperations}*/

	virtual void init (const leda_graph& G, const Hierarchy& H);
	/*{\Mop called before the crossing minimization.}*/

	virtual void call (const leda_graph& G, Layer& L) = 0;
	/*{\Moppure implements the module call for graph $G$. Reorders the nodes
	on layer $L$.}*/

	virtual void cleanup () { }
	/*{\Mop called after the crossing minimization.}*/
};

inline void TwoLayerCrossMin::init (const leda_graph& /*G*/, const Hierarchy& /*H*/)
{ }


#endif
