//-------------------------------------------------------------------
//
//                      VisibilityMapper.h
//
//    author: Carsten Gutwenger
//    date:   octobre 1997
//-------------------------------------------------------------------


#ifndef _AGD_VISIBILITY_MAPPER_H
#define _AGD_VISIBILITY_MAPPER_H


#include <AGD/GridCoordinateMapper.h>


/*{\Manpage {VisibilityMapper} {} {The Grid Coordinate Mapper Visibility} {M}}*/

/*{\Mbaseclasses

\begin{hierarchy}{7.5}{2.5}
\hdclass{0}{1.5}{7.5}{GridCoordinateMapper}
\arhook{7.5}{2}{8.5}
\hdclass{3}{0}{6}{VisibilityMapper}
\end{hierarchy}
}*/

class VisibilityMapper : public GridCoordinateMapper {

/*{\Mdefinition
The Visibility mapper is a grid coordinate mapper that can be used for
visibility representations.

A visibility representation represents each node |v| as a horizontal
segment. The visibility mapper chooses the width of |v| such that 
this segment overlaps all horizontal edge segments outgoing from |v|.

We define the length |len(v)| of a node |v| as follows:
\[ |len|(v) = \max |Segment|(v) - \min |Segment|(v) + 1 \]
where the set |Segment|(v) is defined as
\begin{eqnarray*}
	|Segment|(v) = \{|column|(v)\} & \cup & \left\{ |col|_2^e \mid e=(v,w)
	\mbox{ and } |row|_1^e = |row|_2^e \right\} \\
	& \cup & \left\{ |col|_{k(e)-1}^e \mid e=(w,v) \mbox{ and } |row|_{k(e)}^e =
	|row|_{k(e)-1}^e \right\}
\end{eqnarray*}

The mapping is done in such a way, that each column has size $|size|_x$,
and each row has size $|size|_y$. These are given by
\begin{eqnarray*}
	|size|_x & = & \max\left\{\left. \frac{|width|(v)-(|len|(v)-1)*|sep|_x}
	{|len|(v)} \;\right| v\in V\right\} \cup \{|min_size|_x\} \\[1ex]
	|size|_y & = & \max \left\{ |height|(v)\;\mid v\in V\right\}
\end{eqnarray*}

We can now define the mapping by
	\[ \varphi_x(|column|) = (|size|_x + |sep|_x) * (|column|-|xmin|) +
	|left_margin| + \frac{|size|_x}{2} \]
	\[ \varphi_y(|row|) = (|size|_y + |sep|_y) * (|ymax|-|row|) +
	|upper_margin| + \frac{|size|_y}{2} \]

Since the mapper changes the width of nodes, the x-coordinate must be
adjusted:
\begin{eqnarray*}
	x(v) & = & \widetilde{\varphi}_x \left( \frac{\min |Segment|(v) + \max |Segment|(v)
	}{2} \right) \\
	|width|'(v) & = & \left(|len|(v)-1\right) * |sep|_x + |len|(v) * |size|_x
\end{eqnarray*}
where $\widetilde{\varphi}_x$ is the extension of $\varphi_x$ to
$\real\rightarrow\real$.
}*/
public:

	/*{\Mcreation}*/

	VisibilityMapper (double sepx, double sepy, double msx,
		double lm, double um) {
		sep_x (sepx);
		sep_y (sepy);
		left_margin (lm);
		upper_margin (um);
		min_size_x (msx);
	}
	/*{\Mcreate creates an instance |\Mvar| of type |\Mname|, and sets
	$|sep|_x$ to |sepx|, $|sep|_y$ to |sepy|, $|min_size|_x$ to |msx|,
	|left_margin| to |lm|, and |upper_margin| to |um|.}*/

	virtual ~VisibilityMapper() { }


	/*{\Moperations}*/

	/*{\Mtext       
	\headerline{a) Initialization} }*/

	void init ();
	/*{\Mop computes the mapping for the grid layout given by the graph |G|, 
	the node coordinates |xpos|, |ypos|, the bendpoints |bends| of the edge
	polylines, the layout interface |a|, and the bounding box of the grid
	layout |grid_bbox|.}*/


	/*{\Mtext       
	\bigskip
	\headerline{b) Mapping of coordinates} }*/

	double map_x (int column) const {
		return ax * column + bx;
	}
	/*{\Mop returns $\varphi_x(|column|)$.}*/

	double map_y (int row) const {
		return ay * row + by;
	}
	/*{\Mop returns $\varphi_y(|row|)$.}*/

	DPoint map_node (leda_node v);
	/*{\Mop returns $\varphi(|ip|)$.}*/

	void map_edge (leda_edge e, DPolyline& points, DPoint& s_a, DPoint& t_a);
	/*{\Mop }*/

	void get_layout_size (DRect& bbox) {
		bbox = the_bbox;
	}
	/*{\Mop returns the bounding box of the mapped layout.}*/

	/*{\Mtext       
	\bigskip
	\headerline{c) Further operations} }*/

	void cleanup ();
	/*{\Mop frees resources allocated by |init|.}*/

	GridCoordinateMapper *clone() const {
		return new VisibilityMapper (sep_x(), sep_y(), min_size_x(),
			left_margin(), upper_margin());
	}
	/*{\Mop returns a pointer to a new |\Mname| object that behaves like
	|\Mvar|, i.e. |clone| only copies options, not computed informations.}*/

	double sep_x () const {
		return the_sep_x;
	}
	/*{\Mop returns the current setting of $|sep|_x$.}*/

	void sep_x (double sep) {
		the_sep_x = sep;
	}
	/*{\Mop sets $|sep|_x$ to |sep|.}*/

	double sep_y () const {
		return the_sep_y;
	}
	/*{\Mop returns the current setting of $|sep|_y$.}*/

	void sep_y (double sep) {
		the_sep_y = sep;
	}
	/*{\Mop sets $|sep|_y$ to |sep|.}*/

	double min_size_x () const {
		return the_min_size_x;
	}
	/*{\Mop returns the current setting of $|min_size|_x$.}*/

	void min_size_x (double msx) {
		the_min_size_x = msx;
	}
	/*{\Mop sets $|min_size|_x$ to |msx|.}*/

	double left_margin() const {
		return the_lm;
	}
	/*{\Mop returns the current setting of |left_margin|.}*/

	void left_margin (double lm) {
		the_lm = lm;
	}
	/*{\Mop sets |left_margin| to |lm|.}*/

	double upper_margin() const {
		return the_um;
	}
	/*{\Mop returns the current setting of |upper_margin|.}*/

	void upper_margin (double um) {
		the_um = um;
	}
	/*{\Mop sets |upper_margin| to |um|.}*/

protected:
	void get_size_x_y ();

	void set_coeff ();

	double the_sep_x, the_sep_y, the_min_size_x, the_lm, the_um;

	leda_node_array<int> left, right;
	leda_node_array<DPoint> pos;

	double size_x, size_y;
	double ax, bx, ay, by;
	DRect the_bbox;
};


#endif
