// ---------------------------------------------------------------- //
// compute a visibility representation of the planar graph G        //
// using the algorithm by Rosenstiehl and Tarjan                    //
//                                                                  //
// David Alberts (1996), AGD Version (1997)                         //
// ---------------------------------------------------------------- //

/* {\Mfuncl VISIBILITY\_REPRESENTATION computes a visibility representation
    of the graph $G$, that is, each node is represented by a horizontal
    segment and each edge is represented by a vertical segment, and incidence
    between nodes and edges translates to intersection of horizontal and
    vertical segments. The segments have integer coordinates. The left
    (right) x-coordinates of the node segments are stored in the arrays
    $leftx$ ($rightx$). The y-coordinates of the node segments are stored
    in $ypos$. The lower (upper) y-coordinates of the edge segments are
    stored in $lowy$ ($topy$). The x-coordinates of the edge segments are
    stored in $xpos$.

    The algorithm (\cite{...}) has running time $O(\Labs{V}+\Labs{E})$.
    The x-coordinates lie in the range of 0 to $2\Labs{V}-4$ and the 
    y-coordinates lie in the range of 0 to $\Labs{V}-1$.

    \precond $G$ is simple, biconnected, planar and contains at least
    three nodes. }*/


#ifndef _AGD_VISREP_H
#define _AGD_VISREP_H


#include<AGD/GridLayoutModule.h>
#include<LEDA/map.h>


/*{\Manpage {VisibilityRepresentation} {} {Visibility Representation} {L} }*/

/*{\Mbaseclasses

\begin{hierarchy}{11}{5.5}
\hdclass{0}{4.5}{5}{AGDModule}
\arhook{5}{5}{6}
\hdclass{1.5}{3}{5}{LayoutModule}
\arhook{6.5}{3.5}{7.5}
\hdclass{2}{1.5}{6}{GridLayoutModule}
\arhook{8}{2}{9}
\hdclass{2}{0}{7.5}{VisibilityRepresentation}
\end{hierarchy}
\hfill\agdeps{visibility.epsi}{5.5cm}
}*/

class VisibilityRepresentation : public GridLayoutModule
{

/*{\Mdefinition
The class |\Mname| represents the visibility representation algorithm
by Rosenstiehl and Tarjan~\cite{rt-rplbo-86}.

This algorithm draws a planar, biconnected graph |G| such that
each node is represented by a horizontal
segment and each edge is represented by a vertical segment, and incidence
between nodes and edges translates to intersection of horizontal and
vertical segments.

The grid size is at most $(2n-4) \times (n-1)$ for a graph with $n$ nodes.
|G| may not contain self-loops or multiple edges.

\horizline
\commoninfo{Visibility Representation}{Weak Visibility Representation}{Rosenstiehl, Tarjan}
{David Alberts}{1996/1997}{}

\headerline{Pre- and Postcondition}
\begin{tabular}{lcl}
precondition & $=$ & $\{\, |planar|,|biconnected|,|simple|,|no self loops| \,\}$ \\
postcondition(|PRE|) & $=$ & $\{\, \mbox{|straight|-|line|},|no crossings| \,\}$
\end{tabular}

\horizline
}*/

  public:

	/*{\Mcreation}*/

    VisibilityRepresentation();
	/*{\Mcreate creates an instance |\Mvar| of type |\Mname|.}*/

    ~VisibilityRepresentation() { }

	/*{\Moperations}*/

    leda_string name() const {
		return "Visibility Representation";}

    leda_string long_name() const {
		return "Weak Visibility Representation";
	}

    leda_string author () const {
		return "Rosenstiehl, Tarjan";
	}

	leda_string impl_author () const {
		return "David Alberts";
	}

	leda_string impl_date () const {
		return "1996/1997";
	}

	AGDModule *clone() const;

	/*{\Moptions warnings=no}*/

	/*{\Mtext       
	\headerline{Generic Interface (Inherited Methods)}
	The detailed description of these methods can be found in the
	manual entries of the base classes (|LayoutModule|, |GridLayoutModule|).
	\setlength{\entrysep}{-1ex}}*/

	/* inherited
	bool check (const graph& G, string& message);
	*/
	/*{\Mop}*/

	/* inherited
	void call (const graph& G, LayoutInterface& A);
	*/
	/*{\Mop}*/

	/* inherited
	void call_grid (const graph& G, GridLayout &gl);
	*/
	/*{\Mop}*/

	/* inherited
	IRect get_grid_area ();
	*/
	/*{\Mop}*/

	/* inherited
	void set_coordinate_mapper (const GridCoordinateMapper& gcm);
	*/
	/*{\Mop}*/

  protected:

    void _call_grid (GraphCopy& G, GridLayout &gl);

  private:

    int compute_depth(leda_graph& G, leda_node s, leda_node_array<int>& depth,
                      int offset=0);

    int VISIBILITY_REPRESENTATION(leda_graph& G);

    leda_node_map<int> leftx;
    leda_node_map<int> rightx;
    leda_node_map<int> ypos;
    leda_edge_map<int> xpos;
    leda_edge_map<int> lowy;
    leda_edge_map<int> topy;
};

/*{\Mimplementation
The computation of
the layout takes time $O(n)$ where $n$ is the number of nodes
of the drawn graph.}*/


#endif
