//-------------------------------------------------------------------
//
//                           AGD.cpp
//
//    author: Carsten Gutwenger
//    date:   may 1998
//-------------------------------------------------------------------


#include <AGD/AGDModule.h>


// initialize static variables
class Keymapper AGD::keymapper;
const int AGDModule::out_general = 0;


// NOTE: the order of the declarations below from planar until 
//   << single sink >>
//   is used in AGD::check (for efficency reasons) and may NOT be
//   changed!

const agd_key
	AGD::key::planar        = AGD::keymapper.get_key("planar"),
	AGD::key::simple        = AGD::keymapper.get_key("simple"),
	AGD::key::connected     = AGD::keymapper.get_key("connected"),
	AGD::key::biconnected   = AGD::keymapper.get_key("biconnected"),
	AGD::key::triconnected  = AGD::keymapper.get_key("triconnected"),
	AGD::key::four_graph    = AGD::keymapper.get_key("four-graph"),
	AGD::key::no_self_loops = AGD::keymapper.get_key("no self loops"),
	AGD::key::directed      = AGD::keymapper.get_key("directed"),
	AGD::key::undirected    = AGD::keymapper.get_key("undirected"),
	AGD::key::no_reversals  = AGD::keymapper.get_key("no reversals"),
	AGD::key::acyclic       = AGD::keymapper.get_key("acyclic"),
	AGD::key::tree          = AGD::keymapper.get_key("tree"),
	AGD::key::forest        = AGD::keymapper.get_key("forest"),
	AGD::key::single_source = AGD::keymapper.get_key("single source"),
	AGD::key::single_sink   = AGD::keymapper.get_key("single sink"),
	AGD::key::bipartite     = AGD::keymapper.get_key("bipartite"),
	AGD::key::straight_line = AGD::keymapper.get_key("straight-line"),
	AGD::key::orthogonal    = AGD::keymapper.get_key("orthogonal"),
	AGD::key::no_crossings  = AGD::keymapper.get_key("no crossings"),
	AGD::key::upward_planar = AGD::keymapper.get_key("upward planar"),
	AGD::key::convex        = AGD::keymapper.get_key("convex"),
	AGD::key::animation     = AGD::keymapper.get_key("animation"),
	AGD::key::grid_animation= AGD::keymapper.get_key("grid animation"),
	AGD::key::planar_layout = AGD::keymapper.get_key("planar layout");

AGD::AGD ()
{
}


AGDModule::AGDModule (int n, int m)
{
	_n = n; _m = m;

	_pre  = array<KeySet>(1,n);
	_post = array<list<TaggedRule> >(0,m);
}

bool AGDModule::check_graph (int i, const graph& G, string& message) const
{
	agd_key k;

	forall (k, _pre[i]) {
		switch (k->id()) {
		case 0:  // planar
			if (is_planar(G) == false) {
				message = "The graph is not planar.";
				return false;
			}
			break;
		case 1: // simple
			if (is_simple(G) == false) {
				message = "The graph ist not simple.";
				return false;
			}
			break;
		case 2: // connected
			if (is_connected(G) == false) {
				message = "The graph is not connected.";
				return false;
			}
			break;
		case 3: // biconnected
			if (is_biconnected(G) == false) {
				message = "The graph is not biconnected.";
				return false;
			}
			break;
		case 4: // triconnected
			if (is_triconnected(G) == false) {
				message = "The graph is not triconnected.";
				return false;
			}
			break;
		case 5: // four-graph
			if (is_four_graph(G) == false) {
				message = "The graph is not a four-graph.";
				return false;
			}
			break;
		case 6: // no self loops
			if (has_no_self_loops(G) == false) {
				message = "The graph has self loops.";
				return false;
			}
			break;
		case 7: // directed
			if (G.is_directed() == false) {
				message = "The graph is not directed.";
				return false;
			}
			break;
		case 8: // undirected
			if (G.is_undirected() == false) {
				message = "The graph is not undirected.";
				return false;
			}
			break;
		case 9: // np_reversals
			if (has_no_reversals(G) == false) {
				message = "The graph has reversal edges.";
				return false;
			}
			break;
		case 10: // acyclic
			if (G.is_directed() && is_acyclic(G) == false) {
				message = "The graph is not acyclic.";
				return false;
			}
			break;
		case 11: // tree
			if (is_tree(G) == false) {
				message = "The graph does not represent a tree.";
				return false;
			}
			break;
		case 12: // forest
			if (is_forest(G) == false) {
				message =
					"The graph does not represent a forest.";
				return false;
			}
			break;
		case 13: // single_source
			if (has_single_source(G) == false) {
				message =
					"The graph has not a single source.";
				return false;
			}
			break;
		case 14: // single_sink
			if (has_single_sink(G) == false) {
				message =
					"The graph has not a single sink.";
				return false;
			}
			break;
		case 15: // bipartite
			if (is_bipartite(G) == false) {
				message = "The graph is not bipartite.";
				return false;
			}
			break;
		default:
			if (check_graph_key (k, G, message) == false) {
				if (message == "")
					message = "unknown key: " + k->info();
				return false;
			}
		}
	}
	return true;
}

agd_key AGDModule::_add_precondition  (const string& p, int i)
{
	agd_key k = keymapper.get_key (p);

	add_precondition (k, i);
	return k;
}

void AGDModule::_add_precondition  (agd_key k, int i)
{
	_pre[i].insert (k);
}

void AGDModule::_add_precondition (agd_key k, bool status, int i)
{
	if (status)
		_pre[i].insert (k);
	else
		_pre[i].del (k);
}

void AGDModule::_rem_precondition (agd_key k, int i)
{
	_pre[i].del (k);
}

KeySet AGDModule::_postconditions (int i, const array<KeySet>& PRE) const
{
	if (i < 0 || i > _m)
		error_handler(0,"AGDModule::postcondition(i,PRE): postcondition index out of range");

	KeySet POST;
	list_item it;
	tr_literal l;

	forall_items (it, _post[i]) {
		bool add_key = true;
		const list<tr_literal>& L = (_post[i])[it].left();
		forall (l, L) {
			if (!PRE[l.index].member (l.key)) {
				add_key = false; break;
			}
		}

		if (add_key)
			POST.insert((_post[i])[it].right());
	}

	return POST;
}


AGDModule::pr_item AGDModule::_add_post_rule (const TaggedRule& r, int i)
{
	return _post[i].append(r);
}

AGDModule::pr_item AGDModule::_add_post_rule (agd_key k, int i)
{
	return _post[i].append(TaggedRule(k));
}

AGDModule::pr_item AGDModule::_add_post_rule (agd_key p1, int i1, agd_key p, int i)
{
	TaggedRule r(p);

	r.and (p1, i1);
	return _post[i].append (r);
}

void AGDModule::_rem_post_rule (pr_item it, int i)
{
	_post[i].del_item ((list_item)it);
}

bool AGD::is_four_graph (const graph& G)
{
	node v;
	const int four = (G.is_map()) ? 8 : 4;

	forall_nodes (v, G) {
		if (degree(v) > four) return false;
	}
	return true;
}

bool AGD::has_no_self_loops (const graph& G)
{ 
	edge e;

	forall_edges (e, G) {
		if (source(e) == target(e))
			return false;
	}

	return true;
}

bool AGD::is_planar (const graph& G)
{
	graph directed_G (G);

	directed_G.make_directed();

	return PLANAR (directed_G);
}

bool AGD::is_triconnected (const graph& G)
{
	graph g(G);
	node v, w;
	
	slist<node> vertices;
	slist<node> neighbours;

	if (is_biconnected(g) == false)
		return false;

	Delete_Loops(g);
	g.make_undirected();
	Make_Simple(g);

	forall_nodes (v, g)
		vertices.append(v);

	while (!vertices.empty()) {
		v = vertices.pop();

		forall_adj_nodes (w, v)
			neighbours.append(w);

		g.del_node(v);

		if (is_biconnected(g) == false)
			return false;

		v = g.new_node();
		while (!neighbours.empty())
			g.new_edge (v, neighbours.pop());
	}

	return true;
}

bool AGD::dfs_iua (node v, node father,
	node_array<bool>& visited)
{
	node w;
	edge e;

	visited [v] = true;

	forall_inout_edges (e, v) {
		w = opposite(v,e);
		if (w != father) {
			if (visited [w] == true)
				return false;
			if (dfs_iua (w, v, visited) == false)
				return false;
		}
	}
	return true;
}

bool AGD::is_forest (const graph& G)
{
	node_array<bool> visited (G, false);

	if (G.number_of_nodes() == 0)
		return true;

	if (has_no_self_loops(G) == false) return false;
	if (has_no_reversals(G) == false) return false;
	if (is_simple(G) == false) return false;

	return dfs_iua (G.first_node(), nil, visited);
}

bool AGD::is_tree (const graph& G)
{
	if (G.number_of_nodes() == 0)
		return true;

	if (has_no_self_loops(G) == false) return false;
	if (has_no_reversals(G) == false) return false;
	if (is_simple(G) == false) return false;

	return (G.number_of_nodes() == G.number_of_edges()+1);
}

bool AGD::has_single_source (const graph& G)
{
	node v;
	bool source_found = false;

	if (G.is_directed() == false) return false;

	forall_nodes (v, G) {
		if (G.indeg(v) == 0) {
			if (source_found)
				return false;
			else
				source_found = true;
		}
	}
	return source_found;
}

bool AGD::has_single_sink (const graph& G)
{
	node v;
	bool sink_found = false;

	if (G.is_directed() == false) return false;

	forall_nodes (v, G) {
		if (G.outdeg(v) == 0) {
			if (sink_found)
				return false;
			else
				sink_found = true;
		}
	}
	return sink_found;
}

bool AGD::has_no_reversals (const graph& G)
{
	if (G.is_undirected())
		return true;
	
	edge_array<edge> rev(G);
	edge e;

	Is_Bidirected (G, rev);

	forall_edges (e, G)
		if (rev[e] != nil && source(e) != target(e)) return false;

	return true;
}

KeySet AGD::get_properties (const graph& G)
{
	KeySet props;

	if (G.is_directed())
		props.insert(key::directed);
	else
		props.insert(key::undirected);

	if (is_connected(G)) {
		props.insert(key::connected);
		if (is_biconnected(G)) {
			props.insert(key::biconnected);
			if (is_triconnected(G))
				props.insert(key::triconnected);
		}
	}

	if (is_planar(G)) {
		props.insert(key::planar);
		if (is_forest(G)) {
			props.insert(key::forest);
			if (props.member(key::connected)) props.insert(key::tree);
		}
	}

	if (is_simple(G)) props.insert(key::simple);
	if (has_no_self_loops(G)) props.insert(key::no_self_loops);
	if (has_no_reversals(G)) props.insert(key::no_reversals);
	if (is_four_graph(G)) props.insert(key::four_graph);
	if (is_acyclic(G)) props.insert(key::acyclic);
	if (has_single_source(G)) props.insert(key::single_source);
	if (has_single_sink(G)) props.insert(key::single_sink);
	if (is_bipartite(G)) props.insert(key::bipartite);

	return props;
}


bool AGDModule::_need_precondition (int i, const string & p) const
{
	agd_key k = keymapper.ask_key (p);
	return (k != nil) ? _pre[i].member(k) : false;
}

const KeySet& AGDModule::_preconditions (int i) const
{
	return _pre[i];
}

bool AGDModule::_usable (const array<KeySet>& PRE, const array<KeySet>& POST) const
{
	int i;

	for (i = 1; i <= _n; i++)
		if (!(PRE[i] >= _pre[i])) return false;

	for (i = 0; i <= _m; i++)
		if (!(_postconditions(i,PRE) >= POST[i])) return false;

	return true;
}

void AGD::extract_list (const string& str, char delim, list<string>& L)
{
	const int len = str.length();
	int start = 0, current = 0;

	L.clear();
	while (current < len) {
		if (str[current] == delim) {
			L.append (str(start,current-1));
			start = current+1;
		}
		current++;
	}
	L.append (str(start,current-1));
}

void AGDModule::_add_preconditions (const string& ps, char delim, int i)
{
	list<string> L;
	list_item it;

	extract_list (ps, delim, L);
	forall_items (it, L) {
		_add_precondition (L[it], i);
	}
}
