//-------------------------------------------------------------------
//
//                       LayoutModule.cpp
//
//    author: Carsten Gutwenger
//    date:   may 1998
//-------------------------------------------------------------------


#include <AGD/LayoutModule.h>
#include <AGD/DRect.h>
#include <AGD/GraphCopyLI.h>
#include <AGD/PlaneGraphCopy.h>
#include <LEDA/slist.h>


//-------------------------------------------------------------------
//                           LayoutModule
//-------------------------------------------------------------------

const int LayoutModule::in_graph = 1;
const int LayoutModule::out_layout = 1;


LayoutModule::LayoutModule (bool use_copy) : AGDModule(1,1)
{
	running_step = false;
	the_use_copy = use_copy;
}

void LayoutModule::_finish_call(LayoutInterface &A, LayoutInterface *ap)
{
	LayoutInterface *_ap = (ap != 0) ? ap : &A;

	_ap->update();

	delete ap;
}

bool LayoutModule::check (const graph& G, string& message)
{
	return check_graph (in_graph, G, message);
}


void LayoutModule::call (const graph& G, LayoutInterface& A)
{
	if (G.empty()) return;

	LayoutInterface *ap = 0;
	GraphCopy *gcp = 0;

	if (postcondition(0).member(key::planar_layout)) {
		PlaneGraphCopy PG(G);
		ap = new GraphCopyLI(PG,A);

		if (use_copy()) {
			_call (PG, nil, *ap);
		} else {
			_call ((const graph&)PG, nil, *ap);
		}

	} else {
		if (use_copy()) {
			gcp = new GraphCopy(G);
			ap = new GraphCopyLI (*gcp, A);
			_call (*gcp, *ap);
		} else {
			_call (G, A);
		}
	}
	_finish_call(A,ap);
	delete gcp;
}

bool LayoutModule::start_animation (const graph& G, LayoutInterface& A)
{
	if (!postcondition(0).member(key::animation)) {
		error_handler(0,"LayoutModule::start_animation(G,A): algorithm does'nt support animation.");
		return false;
	}

	step_graph     = &G;
	step_interface = &A;

	if (use_copy()) {
		step_copy_graph = new GraphCopy(G);
		step_copy_li    = new GraphCopyLI (*step_copy_graph, *step_interface);

		if (_start_animation (*step_copy_graph, *step_copy_li) == false) {
			delete step_copy_li;
			delete step_copy_graph;
		}
	} else {
		if (_start_animation (G, A) == false) {
			return false;
		}
	}

	running_step = true;
	return true;  // everything ok!
}


bool LayoutModule::next_step ()
{
	bool result;
	DRect bbox;

	if (!running_step) {  // we're not running animation ?
		error_handler(0,"LayoutModule::next_step: no animation started.");
		return false;
	}

	if (use_copy()) {
		result = _next_step (*step_copy_graph, *step_copy_li);
		if (result) {
			step_copy_li->update();
		}
	} else {
		result = _next_step (*step_graph, *step_interface);
		if (result) {
			step_interface->update();
		}
	}

	if (result == false)
		stop_animation();
	
	return result;
}


void LayoutModule::stop_animation ()
{
	if (!running_step) {  // we're not running animation ?
		error_handler(0,"LayoutModule::next_step: no animation started.");
		return;
	}

	if (use_copy()) {
		_stop_animation (*step_copy_graph, *step_copy_li);
		delete step_copy_li;
		delete step_copy_graph;
	} else {
		_stop_animation (*step_graph, *step_interface);
	}

	running_step = false;
}


void LayoutModule::call (const graph& G, face F, LayoutInterface& A)
{
	if (!postcondition(0).member(key::planar_layout)) {
		error_handler(0,"LayoutModule::call(G,F,A): not a planar layout algorithm.");
		return;
	}
	if (G.empty()) return;

	LayoutInterface *ap = 0;

	if (use_copy()) {
		PlaneGraphCopy PG (G);
		ap = new GraphCopyLI(PG, A);

		_call (PG, (F != nil) ? PG.copy_of_face(F) : nil, *ap);

	} else {
		_call (G, F, A);
	}

	_finish_call(A,ap);
}

void LayoutModule::call (PlaneGraphCopy& PG, face F, LayoutInterface& A)
{
	if (!postcondition(0).member(key::planar_layout)) {
		error_handler(0,"LayoutModule::call(PG,F,A): not a planar layout algorithm.");
		return;
	}
	if (PG.empty()) return;

	if (use_copy()) {
		if (!PG.empty()) _call (PG, F, A);
	} else {
		if (!PG.empty())
			_call ((const graph&)PG, F, A);
	}

	_finish_call(A,0);
}



inline void LayoutModule::_call (const graph& /*G*/, LayoutInterface& /*A*/)
{
	error_handler(0,"LayoutModule::_call(const graph&,LayoutInterface&) not overridden");
}

inline bool LayoutModule::_start_animation (const graph& /*G*/, LayoutInterface& /*A*/) {
	error_handler(0,"LayoutModule::_start_animation(const graph&,LayoutInterface&) not overridden");
	return false;
}

inline bool LayoutModule::_next_step (const graph& /*G*/, LayoutInterface& /*A*/) {
	error_handler(0,"LayoutModule::_next_step(const graph&,LayoutInterface&) not overridden");
	return false;
}

inline void LayoutModule::_stop_animation (const graph& /*G*/, LayoutInterface& /*A*/) {
	error_handler(0,"LayoutModule::_stop_animation(const graph&,LayoutInterface&) not overridden");
}

inline void LayoutModule::_call (GraphCopy& /*G*/, LayoutInterface& /*A*/) {
	error_handler(0,"LayoutModule::_call(GraphCopy&,LayoutInterface&) not overridden");
}

inline bool LayoutModule::_start_animation (GraphCopy& /*G*/, LayoutInterface& /*A*/) {
	error_handler(0,"LayoutModule::_start_animation(GraphCopy&,LayoutInterface&) not overridden");
	return false;
}

inline bool LayoutModule::_next_step (GraphCopy& /*G*/, LayoutInterface& /*A*/) {
	error_handler(0,"LayoutModule::_next_step(GraphCopy&,LayoutInterface&) not overridden");
	return false;
}

inline void LayoutModule::_stop_animation (GraphCopy& /*G*/, LayoutInterface& /*A*/) {
	error_handler(0,"LayoutModule::_stop_animation(GraphCopy&,LayoutInterface&) not overridden");
}


inline void LayoutModule::_call (const graph& /*G*/,
	face /*F*/, LayoutInterface& /*A*/)
{
	error_handler(0,"LayoutModule::_call(const graph&,face,LayoutInterface&) not overridden");
}

inline void LayoutModule::_call (PlaneGraphCopy& /*PG*/,
	face /*F*/, LayoutInterface& /*A*/)
{
	error_handler(0,"LayoutModule::_call(PlaneGraphCopy&,face,LayoutInterface&) not overridden");
}

