//-------------------------------------------------------------------
//
//                     MixedModelLayout.cpp
//
//    author: Carsten Gutwenger
//    date:   august 1997
//-------------------------------------------------------------------


#include <AGD/MixedModelLayout.h>
#include <AGD/PlaneGraphCopy.h>
#include <AGD/impl/misc.h>
#include <AGD/PlanAug.h>


/********************************************************************
							constructor

  set precondition and properties, set options to defaults
********************************************************************/

MixedModelLayout::MixedModelLayout () :
	GridLayoutModule(true)
{
	add_precondition (key::planar);
	add_precondition (key::simple);
	add_precondition (key::no_self_loops);

	add_post_rule (key::planar_layout, 0);
	add_post_rule (key::no_crossings);

	prefer_nodes (false);
	do_postprocessing (true);
	co_type (co_biconnected);
	base_ratio (0.33);

	array<KeySet> PRE(1,1), POST(0,1);
	PRE [1] << key::planar << key::simple << key::no_self_loops
		    << key::directed;
	POST[1] << key::planar << key::biconnected << key::simple
		    << key::no_self_loops;
	augmenter.init (PRE,POST,PlanAug());
}


AGDModule *MixedModelLayout::clone() const
{
	MixedModelLayout *M = new MixedModelLayout;

	M->prefer_nodes (prefer_nodes());
	M->do_postprocessing (do_postprocessing());
	M->co_type (co_type());
	M->base_ratio (base_ratio());

	M->set_augmenter (augmenter.get());
	M->set_coordinate_mapper (get_coordinate_mapper());

	return M;
}


/********************************************************************
						   _call_grid

  implementation of the algorithm call
********************************************************************/

void MixedModelLayout::_call_grid (PlaneGraphCopy& G, face F, GridLayout &gl)
{
	node v1, v2;

	// handle graphs with less than 3 nodes
	switch (G.number_of_nodes()) {
	case 0:
		return;
	case 1:
		v1 = G.first_node();
		gl.x(v1) = gl.y(v1) = 0;
		return;
	case 2:
		v1 = G.first_node();
		v2 = G.succ_node (v1);
		gl.x(v1) = gl.y(v1) = gl.y(v2) = 0;
		gl.x(v2) = 1;
		return;
	}

	augmenter.get().call (G);

	make_map (G);
	G.make_planar_map();

	LmcOrder lmc (G, co_type(), G.max_face(), prefer_nodes(), base_ratio());
	compute_coordinates (G, lmc, gl.x(), gl.y(), gl.bends());
}


/********************************************************************
						compute_coordinates

  the actual mixed-model algorithm
********************************************************************/

void MixedModelLayout::compute_coordinates (PlaneGraphCopy &G,
	LmcOrder& lmc,
	node_map<int>&  x,
	node_map<int>&  y,
	edge_map<IPolyline>& bends)
{
	node_map<list_item> next_left(G), next_right(G);
	node_map<int> x_rel(G);

	iop.init(G);

	const int m = lmc.length();

	node		v, cl, cr, z1;
	edge		e = nil;
	int			i, k, p;
	list_item	it;

	the_bb_upper.init(G);

	node v1 = lmc(1,1), v2 = lmc(1,lmc.len(1));

	for (k = 1; k <= m; k++) {
		const OrderSet &V = lmc[k];
		p = V.len();
		cl = V.left(); cr = V.right();
		for (i = 1; i <= p; i++) {
			v = V[i];
			node v_r = (i == p) ? cr : V[i+1],
			v_l = (i == 1) ? cl : V[i-1];
			edge el = nil, er = nil;
			forall_out_edges (e, v) {
				if (target(e) == v_l) el = e;
				if (target(e) == v_r) er = e;
			}
			if (el == nil) el = er;
			if (er == nil) er = el;
			for (e = G.cyclic_adj_pred(el); e != er; e = G.cyclic_adj_pred(e))
				iop.append_out_point (e);
			e = er;
			while (true) {
				iop.push_in_point (e);
				if (e == el)
					break;
				e = G.cyclic_adj_pred(e);
			}

			// optimization (dummy node for outpoint)
			if (i == p && V.has_right() == false && out(v) >= 2) {
				edge e1 = G.cyclic_adj_succ(el), e2;
				node tgt = target(e1);
				e2 = G.split(e1);
				iop.change_edge (v, e1);
				lmc.push (k, target(e1), tgt);
				iop.append_out_point(e2);
				iop.append_in_point(G.reversal(e1));
				if (k == 1) v2 = target(e1);
			}
		}
	}

	forall_nodes (v, G) {
		const list<InOutPoint> &L_out = iop.out_points (v),
			&L_in = iop.in_points (v);
		it = next_left [v] = L_out.first();
		next_right [v] = L_out.last();

		if (v == v1)
			set_out_coord (v, OS_left);
		else if (v == v2)
			set_out_coord (v, OS_right);
		else {
			const OrderSet &V  = lmc[lmc.rank(v)];
			if (v == V[1] && V.has_left() == false)
				set_out_coord (v, OS_left);
			else if (v == V[V.len()] && V.has_right() == false)
				set_out_coord (v, OS_right);
			else
				set_out_coord (v, OS_middle);
		}
		if (in(v) > 3) {
			it = L_in.first();
			iop.set_in_coord (v, it, -in_l(v), 0);
			for (i = 1, it = L_in.succ(it); i <= in_l(v); i++, it = L_in.succ(it)) {
				iop.set_in_coord (v, it, -in_l(v)+i-1, -i);
			}
			iop.set_in_coord (v, it, 0, -in_r(v));
			for (i = 1, it = L_in.succ(it); i <= in_r(v); i++, it = L_in.succ(it)) {
				iop.set_in_coord (v, it, i, -in_r(v)+i-1);
			}
			iop.set_in_coord (v, it, in_r(v), 0);
		}
	}

	space = array<int>(1, m);
	for (i = 1; i <= m; i++)
		space [i] = 0;
	comp .init (G, 0);
	shift.init(G, 0);
	compL.init(G);
	prev .init(G);
	next .init(G);
	upper.init(G, nil);
node_map<bool> put_back(G, true);

	node c_start, cr2;
	int sum, u1dx, u1dy, u2dx, u2dy;
	bool has_left, has_right;

	const OrderSet &V1 = lmc[1];
	p = V1.len();
	for (i = 2, sum = 0; i <= p; i++) {
		x[V1[i]] = bb_ur(V1[i-1]) + bb_ul(V1[i]) + 1;
	}
	for (i = 1; i <= p; i++) {
		if (i < p) next[V1[i]] = V1[i+1];
		if (i > 1) prev[V1[i]] = V1[i-1];
		y[V1[i]] = 0;
	}
	x [v1] = bb_ul(v1);
	y [v1] = 0;
	prev [v1] = nil;
	next [v2] = nil;

	c_start = v1;

	for (k = 2; k <= m; k++) {
		const OrderSet &Vk = lmc[k];
		p  = Vk.len();
		z1 = Vk[1];
		cl = lmc.left(k); cr = lmc.right(k);
		has_left  = Vk.has_left();
		has_right = Vk.has_right();

		if (has_left == true) {
			const InOutPoint &u = iop.out_points (cl, next_right [cl]);
			u1dx = u.dx; u1dy = u.dy;
		} else {
			u1dx = (next_right[cl] == nil) ? -bb_ul(cl) :
				iop.out_points (cl, next_right[cl]).dx+1;
			u1dy = 1;
		}
		if (has_right == true) {
			const InOutPoint &u = iop.out_points (cr, next_left [cr]);
			u2dx = u.dx; u2dy = u.dy;
		} else {
			 u2dx = (next_left[cr] == nil) ? bb_ur(cr) :
				iop.out_points(cr, next_left[cr]).dx-1;
			 u2dy = 1;
		}
		 perform_backshift (c_start, cr, lmc, x, x_rel);
		 c_start = z1;

		// compute relative x-distance from ci to cl for i = p+1, ..., r
		for (sum = 0, v = next[cl]; v != cr; v = next[v]) {
			sum += x[v]; x[v] = sum;
		}
		x[cr] += sum;

		int y_max = Max ( (u1dx > 0) ? y[cl]+u1dy : (y[cl] + Max(1,u1dy)) ,
						  (u2dx < 0) ? y[cr]+u2dy : (y[cr] + Max(1,u2dy)));

		for (i = 2, sum = 0; i <= p; i++) {
			sum += (x[Vk[i]] = bb_ur(Vk[i-1]) + bb_ul(Vk[i]) + 1);
		}

		if (in(z1) >= 3) {
			node w = (in(z1) >= 3) ? target (iop.in_points (z1, iop.
				in_points (z1).get_item (in_l(z1)+1)).e) : cl;
			const InOutPoint &u_w = iop.out_points(w, next_right [w]);

			// compute y_max
			y_max = Max (y_max, y[w] + Max (1, u_w.dy) - iop.point_of (G.
				reversal(u_w.e)).dy);
			int curr_y = 1;
			for (v = next[cl]; v != w; v = next[v]) {
				if (next_right[v] == next_left[v] && next_left[v] != nil) {
					const InOutPoint &u_v = iop.out_points(v, next_right[v]);
					y_max = Max (y_max, y[v] + Max (1, u_v.dy) - iop.point_of (
						G.reversal(u_v.e)).dy);
					y_max = Max (y_max, Max (horiz_height (G, G.
						cyclic_adj_succ(u_v.e), x, y), horiz_height (G, G.
						cyclic_adj_pred(u_v.e), x, y)) + curr_y);
					curr_y ++;
				} else {
					y_max = Max (y_max, y[v] + bb_upper(v) + curr_y);
				}
			}
			y_max = Max (y_max, horiz_height (G, G.cyclic_adj_succ(
				u_w.e), x, y) + curr_y);
			curr_y = 1;
			for (v = prev[cr]; v != w; v = prev[v]) {
				if (next_right[v] == next_left[v] && next_left[v] != nil) {
					const InOutPoint &u_v = iop.out_points(v, next_right[v]);
					y_max = Max (y_max, y[v] + Max (1, u_v.dy) - iop.point_of (
						G.reversal(u_v.e)).dy);
					y_max = Max (y_max, Max (horiz_height (G, G.
						cyclic_adj_succ(u_v.e), x, y), horiz_height (G, G.
						cyclic_adj_pred(u_v.e), x, y)) + curr_y);
					curr_y ++;
				} else {
					y_max = Max (y_max, y[v] + bb_upper(v) + curr_y);
				}
			}
			y_max = Max (y_max, horiz_height (G, G.cyclic_adj_pred(
				u_w.e), x, y) + curr_y);

			x[z1] = Max (x[w]+u_w.dx, Max(bb_ll(z1),bb_ul(z1))+u1dx);
			int delta = x[z1] - (x[w]+u_w.dx);
			x[cr] = Max (Max(bb_lr(z1), bb_ur(z1))-u2dx, x[cr]+delta-x[z1]);
			int x_w = x[w];
			for (v = next[cl]; v != w; v = next[v]) {
				if (upper[v] == nil) {upper[v] = z1; x_rel[v] = x[v] - x[z1];}
			}
			for ( ; v != cr; v = next[v]) {
				if (upper[v] == nil) {upper[v] = z1; x_rel[v] = x[v] - (x_w + u_w.dx);}
			}
		} else {
			// compute y_max
			for (v = next[cl]; v != cr; v = next[v])
				if (y[v]+bb_upper(v)+1 > y_max) y_max = y[v]+bb_upper(v)+1;

			x[z1] = bb_ul(z1) + u1dx;
			int s1 = bb_ur(Vk[p])-u2dx, s2 = x[cr]-x[z1]-sum;

			x[cr] = Max (s1, s2);

			space [k] = Max (0, x[cr]-s1);
			if (s1-s2 > 0)
				shift[cr] += (s1-s2);

			for (v = next[cl]; v != cr; v = next[v]) {
				if (upper[v] == nil) {upper[v] = z1; x_rel[v] = x[v]-x[z1];}
			}
		}

		if (has_left == true)
			next_right [cl] = iop.out_points (cl).pred (next_right[cl]);
		if (has_right == true)
			next_left  [cr] = iop.out_points (cr).succ (next_left [cr]);

		for (v = cr, sum = 0; v != nil && upper[v] == nil &&
			space[lmc.rank(v)] == 0 && (next_left[v] == nil ||
			iop.out_points(v).pred(next_left[v]) == next_right[v]); v = next[v]) {
			sum += x[v]; x_rel[v] = sum;
			upper[v] = Vk[p]; put_back[v] = false;
		}
		cr2 = v;


		for (i = 1; i <= p; i++) {
			y [Vk[i]] = y_max;
		}

		// update contour after insertion of z1,...,zp
		for (i = 1; i <= p; i++) {
			if (i < p) next[Vk[i]] = Vk[i+1];
			if (i > 1) prev[Vk[i]] = Vk[i-1];
		}

		next [cl] = z1; next [Vk[p]] = cr;
		prev [cr] = Vk[p]; prev [z1] = cl;
	}

	perform_backshift (c_start, v2, lmc, x, x_rel);

	// compute final comp values:
	compute_comp (G, lmc);

	// compute final x-coordinates for nodes on final contour
	for (sum = 0, v = v1; v != nil; v = next[v])
		x [v] = (sum += x[v]);

	// compute final x-coordinates for inner nodes
	for (k = m; k >= 1; k--) {
		for (i = 1; i <= lmc.len(k); i++) {
			node zi = lmc(k,i);
			if (upper[zi] != nil) {
				if (put_back[zi])
					x[zi] = x_rel[zi] + (x[upper[zi]] - comp[upper[zi]]);
				else
					x[zi] = x_rel[zi] + x[upper[zi]];
			}
		}
	}

	if (do_postprocessing() == true) {
		postprocessing (G, x, y);
	}

	init_bends (G, x, y, bends, lmc);

	iop.clear();
	space = array<int>();
	comp  = shift = node_map<int>();
	prev  = next = upper = node_map<node>();
	compL = node_map<list<pair_v_d> >();
	S = list<node>();
	the_bb_upper = node_map<int>();
//	hided = edge_map<edge>();
}


/********************************************************************
		  bb_upper, bb_lower, bb_ll, bb_ur, bb_ul, bb_ur

  these methods return the bounding box of a node
********************************************************************/

int MixedModelLayout::bb_upper (node v) {
	return the_bb_upper [v];
}

int MixedModelLayout::bb_lower (node v) {
	return in_r(v);
}

int MixedModelLayout::bb_ll (node v) {
	const list<InOutPoint> &L = iop.in_points(v);
	return (L.empty()) ? 0 : -L[L.first()].dx;
}

int MixedModelLayout::bb_lr (node v) {
	const list<InOutPoint> &L = iop.in_points(v);
	return (L.empty()) ? 0 : L[L.last()].dx;
}

int MixedModelLayout::bb_ul (node v) {
	const list<InOutPoint> &L = iop.out_points(v);
	return (L.empty()) ? 0 : -L[L.first()].dx;
}

int MixedModelLayout::bb_ur (node v) {
	const list<InOutPoint> &L = iop.out_points(v);
	return (L.empty()) ? 0 : L[L.last()].dx;
}


/********************************************************************
							set_out_coord

  assign coordinates to the outpoints of node v
  OS_left and OS_right allow to use outpoints with dy = 0
********************************************************************/

void MixedModelLayout::set_out_coord (node v, Outpoint_Style style)
{
	const list<InOutPoint> &L = iop.out_points(v);
	int on_left, on_right;
	int i, y_mid, y_left, y_right;
	list_item it;

	switch (style) {
	case OS_left:
		on_left = out_r(v);
		y_mid = out_l(v);
		y_left = 0;
		y_right = 1;
		break;
	case OS_middle:
		on_left = out_l(v);
		y_mid = out_r(v);
		y_left = 1;
		y_right = 1;
		break;
	case OS_right:
		on_left = out_l(v);
		y_mid = out_l(v);
		y_left = 1;
		y_right = 0;
		break;
	default:  // no valid Outpoint_Style
	  return;
	}
	the_bb_upper [v] = y_mid;
	on_right = out(v) - on_left - 1;

	if (out(v) == 0) return;
	for (i = 0, it = L.first(); i < on_left; i++, it = L.succ(it)) {
		iop.set_out_coord (v, it, -on_left + i, y_left + i);
	}
	iop.set_out_coord (v, it, 0, y_mid);
	for (i = 1, it = L.succ(it); i <= on_right; i++, it = L.succ(it)) {
		iop.set_out_coord (v, it, i, on_right - i + y_right);
	}
}


/********************************************************************
							horiz_height

********************************************************************/

int MixedModelLayout::horiz_height (PlaneGraphCopy& G, edge e,
	node_map<int>& x, node_map<int>& y)
{
	const InOutPoint &u_s = iop.point_of (e),
		&u_t = iop.point_of (G.reversal(e));
	node v = source(e), w = target(e);
	if (u_s.dx+x[v] - (u_t.dx+x[w]) != 0) {
		return Max (u_s.dy+y[v], u_t.dy+y[w]);
	} else {
		return 0;
	}
}


/********************************************************************
							perform_backshift

********************************************************************/

void MixedModelLayout::perform_backshift (node from, node to,
	LmcOrder& lmc, node_map<int>& x, node_map<int>& x_rel)
{
   int d;
   node v;

	for (v = from; v != to; v = next[v]) {
		if (shift[v] > 0) S.push(v);
		while (!S.empty() && lmc.rank(v) <= lmc.rank(S.head()) && in(v) >= 3) {
			S.pop();
		}
		while (!S.empty() && lmc.rank(v) <= lmc.rank(S.head()) &&
			space[lmc.rank(v)] > 0) {
			d = Min(shift[S.head()], space[lmc.rank(v)]);
			compL [S.head()].append (pair_v_d(v,d));
			shift[S.head()] -= d; space[lmc.rank(v)] -= d;
			x[lmc[lmc.rank(v)].right()] -= d;
			 x_rel[lmc[lmc.rank(v)].right()]-=d; // added
			if (shift[S.head()] == 0) S.pop();
		}
	}
}


/********************************************************************
							compute_comp

********************************************************************/

void MixedModelLayout::compute_comp (PlaneGraphCopy &G, LmcOrder& lmc)
{
	const int m = lmc.length();
	node_array<int> sum (G, 0);
	node_array<list<pair_v_d> > L(G);
	array<list<pair_v_d> > compLR (1, m);
	list<pair_v_d> LR;
	int i, j, s, d;
	node v;
	pair_v_d vd;

	forall_nodes (v, G)
		forall (vd, compL [v])
			sum[v] += vd.d;

	for (i = m; i >= 1; i--) {
		const OrderSet &Vi = lmc[i];
		if (lmc.right(i) != nil) {
			s = d = 0;
			for (j = Vi.len(); j >= 1; j--) {
				list<pair_v_d> &L1 = compL [Vi[j]];
				L1.conc (L[Vi[j]]);
				while (!L1.empty() && L1.head().v == Vi[j])
					d += L1.pop().d;
				LR.conc(L1);
			}
			for (j = 1; j <= Vi.len(); j++) {
				s += sum[Vi[j]];
				comp[Vi[j]] += s;
			}
			L[lmc.right(i)].conc (LR);
			sum[lmc.right(i)] += s - d;
		}
	}
}


/********************************************************************
							postprocessing

  improves the placing of nodes v with in(v) = out(v) = 2
********************************************************************/

void MixedModelLayout::postprocessing (PlaneGraphCopy &G, node_map<int>& x,
	node_map<int>& y)
{
	node v;

	forall_nodes (v, G) {
		if (in(v) == 2 && out(v) == 2) {
			const list<InOutPoint> &L_in  = iop.in_points(v);
			const list<InOutPoint> &L_out = iop.out_points(v);
			const InOutPoint
				&v_in_1  = L_in [L_in.first()],
				&v_in_2  = L_in [L_in.last()],
				&v_out_1 = L_out [L_out.first()],
				&v_out_2 = L_out [L_out.last()],
				&p_out1 = iop.point_of (G.reversal(v_in_1.e)),
				&p_out2 = iop.point_of (G.reversal(v_in_2.e));

			node vi1 = target (v_out_1.e),
				 vi2 = target (v_out_2.e),
				 vo1 = target (v_in_1.e),
				 vo2 = target (v_in_2.e);

			int bends_left = 0, bends_right = 0;

			if (y[vo1] < y[v] && x[vo1] + p_out1.dx == x[v])
				if (exists(G,v_in_1.e)) bends_right++;
			if (y[vo2] < y[v] && x[vo2] + p_out2.dx == x[v]+1)
				if (exists(G,v_in_2.e)) bends_left++;
			if (y[vi1] > y[v]+1)
				if (exists(G,v_out_1.e)) bends_right++;
			if (y[vi2] > y[v]+1)
				if (exists(G,v_out_2.e)) bends_left++;

			if (bends_right < bends_left)
			{
				x[v] += 1;
				iop.out_dx (v, L_out.first()) -= 1;
				iop.out_dx (v, L_out.last ()) -= 1;
			}
		}
	}
}


/********************************************************************
							make_edge_seg

********************************************************************/
/*
void MixedModelLayout::make_edge_seg (const IPoint& p1,
	const IPoint& p2, node v_s, node v_t, IPolyline& seg,
	node_map<int>& x, node_map<int>& y, LmcOrder& lmc)
{
	seg.append (p1);
	if (lmc.rank(v_s) < lmc.rank(v_t)) {
		seg.append (IPoint(p1.xcoord(),p2.ycoord()));
	} else {
		seg.append (IPoint(p2.xcoord(),p1.ycoord()));
	}
	seg.append (p2);
}
*/

/********************************************************************
							init_bends

  assigns polylines to edges of the original graph
********************************************************************/

void MixedModelLayout::init_bends (PlaneGraphCopy &G, node_map<int>& x,
	node_map<int>& y, edge_map<IPolyline>& bends, LmcOrder& lmc)
{
	edge e;
	forall_edges (e, G) {
		if (G.original(e) != nil) {
			node v_s = source(e);
			node v_t = target(e);
			InOutPoint &p_s = iop.point_of(e);
			InOutPoint &p_t = iop.point_of(G.reversal(e));
			IPoint p1 (x[v_s] + p_s.dx, y[v_s] + p_s.dy);
			IPoint p2 (x[v_t] + p_t.dx, y[v_t] + p_t.dy);

			bends[e].append (p1);
			if (lmc.rank(v_s) < lmc.rank(v_t)) {
				bends[e].append (IPoint(p1.xcoord(),p2.ycoord()));
			} else {
				bends[e].append (IPoint(p2.xcoord(),p1.ycoord()));
			}
			bends[e].append (p2);
		}
	}
}

bool MixedModelLayout::exists (PlaneGraphCopy& G, edge e)
{
	return (G.original(e) != nil || G.original(G.reversal(e)) != nil);
}

//
// I/O Operators as required by LEDA 3.4 (dummy only)
//

ostream& operator<< (ostream& out, InOutPoint)
{
	return out;
}

istream& operator>> (istream& in, InOutPoint)
{
	return in;
}

//
// I/O Operators as required by LEDA 3.4 (dummy only)
//

ostream& operator<< (ostream& out, const pair_v_d)
{
	return out;
}

istream& operator>> (istream& in, pair_v_d)
{
	return in;
}
