//-------------------------------------------------------------------
//
//                   PlanarStraightLayout.cpp
//
//    author: Carsten Gutwenger
//    date:   august 1997
//-------------------------------------------------------------------


#include <AGD/PlanarStraightLayout.h>
#include <AGD/PlanAug.h>
#include <AGD/CanonicalOrder.h>
#include <AGD/PlaneGraphCopy.h>


PlanarStraightLayout::PlanarStraightLayout () :
	GridLayoutModule(true)
{
	add_precondition (key::planar);
	add_precondition (key::simple);
	add_precondition (key::no_self_loops);

	add_post_rule (key::planar_layout, 0);
	add_post_rule (key::straight_line);
	add_post_rule (key::no_crossings);

	size_opt (true);
	max_outer (true);
	base_ratio (0.33);

	array<KeySet> PRE(1,1), POST(0,1);
	PRE [1] << key::planar << key::simple << key::no_self_loops
		    << key::directed;
	POST[1] << key::planar << key::biconnected << key::simple
		    << key::no_self_loops;
	augmenter.init (PRE,POST,PlanAug());

}

AGDModule *PlanarStraightLayout::clone() const
{
	PlanarStraightLayout *M = new PlanarStraightLayout;

	M->size_opt (size_opt());
	M->max_outer (max_outer());
	M->base_ratio (base_ratio());

	M->set_augmenter (augmenter.get());
	M->set_coordinate_mapper (get_coordinate_mapper());

	return M;
}

void PlanarStraightLayout::_call_grid (PlaneGraphCopy& G, face F, GridLayout &gl)
{
	// handle graphs with less than 3 nodes
	node v1, v2;

	switch (G.number_of_nodes()) {
	case 0:
		return;
	case 1:
		v1 = G.first_node();
		gl.x(v1) = gl.y(v1) = 0;
		return;
	case 2:
		v1 = G.first_node();
		v2 = G.succ_node (v1);
		gl.x(v1) = gl.y(v1) = gl.y(v2) = 0;
		gl.x(v2) = 1;
		return;
	}

	augmenter.get().call (G);

	make_map (G);
	G.make_planar_map();

	LmcOrder lmc (G, co_biconnected, G.max_face(), false, base_ratio());
	compute_coordinates (G, lmc, gl.x(), gl.y());
}

void PlanarStraightLayout::compute_coordinates (graph &G, LmcOrder &lmc,
	node_map<int>& x, node_map<int>& y)
{
	// let c_1,...,c_q be the the current contour, then
	// next[c_i] = c_i+1, prev[c_i] = c_i-1
	node_array<node>	next(G), prev(G),
	// upper[v] = w means x-coord. of v is relative to w
	// (abs. x-coord. of v = x[v] + abs. x-coord of w)
						upper(G, nil);

	node v1 = lmc (1,1),
		 v2 = lmc (1,lmc.len(1));

	node		v, z1, cl, cr;
	const int	n = lmc.length();
	int			k, i, sum;

	for (i = 1; i <= lmc.len(1); i++) {
		y [lmc(1,i)] = 0;
		x [lmc(1,i)] = (i == 1) ? 0 : 2;
		if (i < lmc.len(1))
			next[lmc(1,i)] =lmc(1,i+1);
		if (i > 1)
			prev[lmc(1,i)] = lmc(1,i-1);
	}
    prev[v1] = next[v2] = nil;

	for (k = 2; k <= n; k++) {
		const OrderSet &Vk = lmc[k];
		int l = Vk.len(), offset;	// Vk = { z_1,...,z_l }
		z1 = Vk[1];
		cl = Vk.left(); cr = Vk.right();

		// compute relative x-distance from ci to cl for i = l+1, ..., r
		int x_cr = 0;
		for (v = next[cl]; v != cr; v = next[v]) {
			x_cr += x[v]; x[v] = x_cr;
		}
		x_cr += x[cr]; // x_cr = abs. x-coord(cr) - abs. x-coord(cl)

		if (size_opt() == true) {
			// optimization: compute minimal value offset, by which cr must be
			// shift to right
			int y_max = y[cr];

			// if there is an edge from cl to right with slope +1, or from cr
			// to left with slope -1, then offset must be at least 2
			offset = (y[cl] < y[next[cl]] || y[cr] < y[prev[cr]]) ? 2 : 0;

			// y_max = max { y[c_i] | l <= i <= r }
			for (v = cl; v != cr; v = next[v]) {
				if (y[v] > y_max) y_max = y[v];
			}
			// offset must be at least so large, that y[zi] > y_max for all i
			offset = Max (offset, 2*(y_max+l)-x_cr-y[cl]-y[cr]);
		} else
			offset = 2*l;

		x_cr += offset;
		// compute insert coordinates of zi
		x[z1] = (x_cr + y[cr] - y[cl]) / 2 - l + 1;
		y[z1] = (x_cr + y[cr] + y[cl]) / 2 - l + 1;

		for (i = 2; i <= l; i++) {
			x[Vk[i]] = 2;
			y[Vk[i]] = y[z1];
		}

		// Compute shift values for cl,...,cr and relative x-coord. to a node
		// upper[v] for inner nodes
		// Let l <= c_alpha <= c_beta <= r max. with
		//		y[cl] > ... > y[c_alpha] and y[c_beta] < ... < y[cr]
		// Shift c_alpha,...,c_beta by offset/2 (c_alpha,c_beta only if != cl,
		// cr)
		// Shift c_beta+1,...,cr by offset
		// x-coord. of c_l+1,    ...,c_alpha  relative to cl
		//       -"-   c_alpha+1,...,c_beta-1 relative to z1
		//       -"-   c_beta,   ...,cr       relative to cr
		node ca, cb;
		if (y[cl] <= y[next[cl]]) ca = cl;
		else {
			for (ca = next[cl], v = next[ca]; ca != cr && y[v] < y[ca];
				ca = v, v = next[v])
			{
				x[ca] += 0; upper[ca] = cl;
			}
			if (ca != cr) {
				x[ca] += (offset/2); upper[ca] = cl;
			}
		}
		if (y[cr] <= y[prev[cr]]) {
			cb = cr;
		} else {
			for (cb = prev[cr], v = prev[cb]; cb != cl && y[v] < y[cb];
				cb = v, v = prev[v])
			{
				x[cb] += offset - x_cr; upper[cb] = cr;
			}
			if (cb != cl && cb != ca) {
				x[cb] += (offset/2) - x_cr; upper[cb] = cr;
			}
		}
		if (ca != cb)
			for (v = next[ca]; v != cb; v = next[v]) {
				x[v] += (offset/2) - x[z1]; upper[v] = z1;
			}

		x[cr] = x_cr - (x[z1] + 2*(l-1));

		// update contour after insertion of z1,...,zl
		for (i = 1; i <= l; i++) {
			if (i < l) next[Vk[i]] = Vk[i+1];
			if (i > 1) prev[Vk[i]] = Vk[i-1];
		}
		next [cl] = z1;    next [Vk[l]] = cr;
		prev [cr] = Vk[l]; prev [z1] = cl;
	}
	// compute final x-coordinates for the nodes on the (final) contour
	sum = 0;
	for (v = v1; v != nil; v = next[v]) {
		x[v] = (sum += x[v]);
	}

	// compute final x-coordinaten for the inner nodes
	for (k = n-1; k >= 1; k--)
		for (i = 1; i <= lmc.len(k); i++) {
			node zi = lmc (k,i);
			if (upper[zi] != nil)	// upper[zi] == nil <=> zi on contour
				x[zi] += x[upper[zi]];
		}
}
