//-------------------------------------------------------------------
//
//                         SpringLayout.c
//
//    author: Stefan Naeher, AGD-Anpassung David Alberts
//    date:   April 1997
//-------------------------------------------------------------------


#include <AGD/SpringLayout.h>
#include <AGD/LayoutInterface.h>
#include <math.h>

SpringLayout::SpringLayout() : LayoutModule()
{
	add_post_rule (key::animation,0);
	add_post_rule (key::straight_line);

  // default parameters
  the_iterations = 400;
  xleft = ysmall = 0.0;
  xright = ybig = 250.0;
  the_noise = true;
}


AGDModule *SpringLayout::clone() const
{
	SpringLayout *M = new SpringLayout;

	M->iterations (iterations());
	M->noise (noise());

	return M;
}

void SpringLayout::_call(const graph& G, LayoutInterface& a)
{
  do_spring(G,a);
}


#define FREPULSE(d) ((k2 > d) ? kk/d : 0)


void SpringLayout::do_spring(const graph& G, LayoutInterface& a,
			     const list<node>& fixed_nodes)
{
  if(initialize(G,a,fixed_nodes))
  {
    for(int i=1; i<=the_iterations; i++) main_step(G,a);
    cleanup();
  }
}

bool SpringLayout::initialize(const graph& G, LayoutInterface& a,
			      const list<node>& fixed_nodes)
{
  a.make_straight_line();  // remove possible bends

  node v;

  if(G.number_of_nodes() <= 1) return false;  // nothing to do
  
  A = 0;

  // compute bounding box, DA
  /*v = G.first_node();
  xleft = xright = a.get_x(v);
  ysmall = ybig = a.get_y(v);
  forall_nodes(v,G)
  {
    if(a.get_x(v) < xleft)   xleft = a.get_x(v);
    if(a.get_x(v) > xright)  xright = a.get_x(v);
    if(a.get_y(v) < ysmall)  ysmall = a.get_y(v);
    if(a.get_y(v) > ybig)    ybig = a.get_y(v);
  }

  if( xleft == xright || ysmall == ybig )
  {
    error_handler(0,"SpringLayout: sorry, I cannot handle this.");
    return false;
  }*/

	// compute a suitable area (xleft,ysmall), (xright,ybig)
	// zoom the current layout into that area

	double w_sum = 0.0, h_sum = 0.0;
	double xmin, xmax, ymin, ymax;

	v = G.first_node();
	xmin = xmax = a.get_x(v);
	ymin = ymax = a.get_y(v);

	forall_nodes(v,G) {
		if(a.get_x(v) < xmin) xmin = a.get_x(v);
		if(a.get_x(v) > xmax) xmax = a.get_x(v);
		if(a.get_y(v) < ymin) ymin = a.get_y(v);
		if(a.get_y(v) > ymax) ymax = a.get_y(v);
		w_sum += a.get_width (v);
		h_sum += a.get_height(v);
	}

	double sqrt_n = sqrt(G.number_of_nodes());
	xleft  = 0; xright = (w_sum > 0) ? 8 * w_sum / sqrt_n : 1;
	ysmall = 0; ybig   = (h_sum > 0) ? 8 * h_sum / sqrt_n : 1;

	double fx = xright / (xmax - xmin);
	double fy = ybig   / (ymax - ymin);

	forall_nodes(v,G) {
		a.set_x (v, (a.get_x(v) - xmin) * fx);
		a.set_y (v, (a.get_y(v) - ymin) * fy);
	}


  lit.init(G);

  fixed.init(G,false);
  forall(v,fixed_nodes) fixed[v] = true;

  width  = xright - xleft;
  height = ybig - ysmall;

  if( (width < 0) || (height < 0) )  // should not happen
  {
    error_handler(0,"SpringLayout: Oops, width or height less than zero??");
    return false;
  }

  tx_null = width/50;
  ty_null = height/50;
  tx = tx_null;
  ty = ty_null;

  k = sqrt(width*height / G.number_of_nodes()) / 2;
  k2 = 2*k;
  kk = k*k;

  ki = int(k);

  if (ki == 0) ki = 1;
  
  c_f = 1;

  // build  matrix of node lists
  xA = int(width / ki + 1);
  yA = int(height / ki + 1);
  A = new array2<list<node> >(-1,xA,-1,yA);

  forall_nodes(v,G)
  {
    double xv = a.get_x(v);
    double yv = a.get_y(v);

    int i = int((xv - xleft) / ki);
    int j = int((yv - ysmall) / ki);

    if( (i >= xA) || (i <= -1) )
      error_handler(1,"SpringLayout: node out of range");
    if( (j >= yA) || (j <= -1) )
      error_handler(1,"SpringLayout: node out of range");

    lit[v] = (*A)(i,j).push(v);
  }
  
  return true;
}

void SpringLayout::main_step(const graph& G, LayoutInterface& a)
{
  node u,v;
  edge e;
  
  node_array<double> xdisp(G,0); 
  node_array<double> ydisp(G,0); 

  // repulsive forces
  forall_nodes(v,G) 
  { 
    double xv = a.get_x(v);
    double yv = a.get_y(v);

    int i = int((xv - xleft) / ki);
    int j = int((yv - ysmall) / ki);

    for(int m = -1; m <= 1; m++)
      for(int n = -1; n <= 1; n++)
        forall(u,(*A)(i+m,j+n))
        { 
	  if(u == v) continue;
          double xdist = xv - a.get_x(u);
          double ydist = yv - a.get_y(u);
          double dist = sqrt(xdist*xdist + ydist*ydist);
          if(dist < 1e-3) dist = 1e-3;
          xdisp[v] += FREPULSE(dist) * xdist / dist;
          ydisp[v] += FREPULSE(dist) * ydist / dist;
        }
  }

  // attractive forces
  forall_edges(e,G)
  {
    node u = G.source(e);
    node v = G.target(e);
    double xdist = a.get_x(v) - a.get_x(u);
    double ydist = a.get_y(v) - a.get_y(u);
    double dist = sqrt(xdist*xdist + ydist*ydist);

    float f = (G.degree(u)+G.degree(v))/6.0f;

    dist /= f;

    xdisp[v] -= xdist*dist/k;
    ydisp[v] -= ydist*dist/k;
    xdisp[u] += xdist*dist/k;
    ydisp[u] += ydist*dist/k;
  }

  // noise      // moved this down from repulsive forces loop, DA
  if(the_noise)
  {
    forall_nodes(v,G) 
    {
      xdisp[v] *= (double(rand_int(750,1250))/1000.0);  // adapted to BETA
      ydisp[v] *= (double(rand_int(750,1250))/1000.0);  // version, DA
    }
  }


  // preventions

  forall_nodes(v,G)
  { 
    if (fixed[v]) continue;

    double xv = a.get_x(v);
    double yv = a.get_y(v);

    int i0 = int((xv - xleft) / ki);
    int j0 = int((yv - ysmall) / ki);

    double xd = xdisp[v];
    double yd = ydisp[v];
    double dist = sqrt(xd*xd+yd*yd);

    if (dist < 1) dist = 1;

    xd = tx*xd/dist;
    yd = ty*yd/dist;

    double xp = xv + xd;
    double yp = yv + yd;

    int i,j;
     
    if( (xp > xleft) && (xp < xright) )
    {
      a.set_x(v,xp);
      i = int((xp - xleft) / ki);
    }
    else i = i0;

    if( (yp > ysmall) && (yp < ybig) )
    {
      a.set_y(v,yp);
      j = int((yp - ysmall) / ki);
    }
    else j = j0;
 
    if( (i != i0) || (j != j0) )
    { 
      if (lit[v] == nil) error_handler(1,"delete nil item");
      (*A)(i0,j0).del_item(lit[v]);
      lit[v] = (*A)(i,j).push(v);
    }
  }

  tx = tx_null / log2(c_f); 
  ty = ty_null / log2(c_f);

  c_f++;
}
