
#include <AGD/TopologyModule.h>
#include <AGD/impl/_sort_edges_double.h>

const int TopologyModule::in_graph = 1;
const int TopologyModule::in_layout = 2;
const int TopologyModule::out_plan_rep = 1;

TopologyModule::TopologyModule() : AGDModule(2, 1) {
  add_precondition(key::planar, in_graph);
  add_precondition(key::connected, in_graph);
}


double TopologyModule::angle(DPoint p, DPoint q, DPoint r) {

  if (q == r) return 2 * LEDA_PI;

  double dx1 = q.xcoord() - p.xcoord(), dy1 = q.ycoord() - p.ycoord();
  double dx2 = r.xcoord() - p.xcoord(), dy2 = r.ycoord() - p.ycoord();

  if ((dx1 == 0 && dy1 == 0) || (dx2 == 0 && dy2 == 0)) 
     error_handler(1,"DPoint, angle:  zero vector input.");
  
  double norm  = (dx1*dx1+dy1*dy1)*(dx2*dx2+dy2*dy2);

  double cosphi = (dx1*dx2+dy1*dy2) / sqrt(norm);

  if (cosphi >=  1.0 ) return 0;
  if (cosphi <= -1.0 ) return LEDA_PI;
  
  double phi = acos(cosphi);

  if (dx1*dy2 < dy1*dx2) phi = -phi;

  if (phi < 0) phi += 2*LEDA_PI;

  return phi;
}

bool TopologyModule::check(const graph& G, string& message) {
  return check_graph(in_graph, G, message);
}

leda_list<leda_edge> TopologyModule::call(graph& G, 
  face& f_0, LayoutInterface& L) {
  list<edge> M;
  f_0 = (get_planar_representation(G, L, M)) ? get_outer_face(G, L) : nil;
  return M;
}

leda_list<leda_edge> TopologyModule::call(graph& G, LayoutInterface& L) {
  list<edge> M;
  get_planar_representation(G, L, M);
  return M;
}

AGDModule *TopologyModule::clone() const {
  TopologyModule *T = new TopologyModule();
  return T;
}


bool TopologyModule::get_planar_representation(graph& G, LayoutInterface& L,
  list<edge>& M) {

  G.make_map(M);

  L.refresh();

  edge e; forall(e, M) {
    DPolyline l = L.get_bend_points(G.reversal(e));
    l.reverse();
    L.set_bend_points(e, l);
  }

  edge_array<double> dx(G), dy(G);

  forall_edges(e, G) {
    DPoint ps = L.get_position(source(e));
    DPoint pt = L.get_position(target(e));
    if (!L.get_bend_points(e).empty()) pt = L.get_bend_points(e).head();
    dx[e] = pt.xcoord() - ps.xcoord();
    dy[e] = pt.ycoord() - ps.ycoord();
  }

  return SORT_EDGES(G, dx, dy); 
}


face TopologyModule::get_outer_face(graph& G, const LayoutInterface& L) {
  
  G.compute_faces();

  face f; forall_faces(f, G) {
    
    DPolyline l; double rho = 0;
    edge e; forall_face_edges(e, f) {
      l += L.get_position(source(e));
      DPolyline b = L.get_bend_points(e);
      l.conc(b);
    }
    
    list_item it; forall_items(it, l) {
      DPoint p = l[it], r = l[l.cyclic_pred(it)], q = l[l.cyclic_succ(it)];
      rho += angle(p, q, r) - LEDA_PI;
    }

    if (rho > 0) return f;
  
  }

  return nil;
}

