//-------------------------------------------------------------------
//
//                          graph.cpp
//
//    author: Carsten Gutwenger
//    date:   july 1998
//-------------------------------------------------------------------


#include <AGD/PlaneGraphCopy.h>
#include <AGD/LeveledGraph.h>
#include <LEDA/array.h>
#include <LEDA/stack.h>


//-------------------------------------------------------------------
//                            GraphCopy
//-------------------------------------------------------------------

GraphCopy::GraphCopy(const GraphCopy& GC) : graph(GC)
{
	_init_GC (GC);
}

GraphCopy& GraphCopy::operator=(const GraphCopy& GC)
{
	graph::operator=(GC);
	_init_GC(GC);
	return *this;
}

void GraphCopy::_init_GC (const GraphCopy& GC)
{
	node v, w;
	edge e, e1;

	g_orig = GC.g_orig;
	e_orig = edge_map<edge>(*this,nil);
	v_orig = node_map<node>(*this,nil);
	_in_chain.init(*this, nil);

	if (g_orig != nil) {
		_chain   .init(*g_orig);
		_copy    .init(*g_orig);
		node_array<node> v_copy(GC);
		edge_array<edge> e_copy(GC);

		map_to_copy (v_copy, e_copy, GC);

		forall_nodes (v, GC)
			v_orig[v_copy[v]] = GC.original(v);

		forall_edges (e, GC)
			e_orig[e_copy[e]] = GC.original(e);

		forall_nodes (v, *this) {
			if ((w = v_orig[v]) != nil)
				_copy[w] = v;
		}

		forall_edges (e, *g_orig)
			forall (e1, GC._chain[e]) {
				_in_chain[e_copy[e1]] = _chain[e].append(e_copy[e1]);
			}
	}
}

GraphCopy::GraphCopy(const graph& G) : graph(G)
{
	node v;
	edge e;

	g_orig = &G;
	e_orig = edge_map<edge>(*this,nil);
	v_orig = node_map<node>(*this,nil);
	_copy    .init(G);
	_chain   .init(G);
	_in_chain.init(*this, nil);


	edge_array<edge> e_copy(G);

	map_to_copy (_copy, e_copy, G);

	forall_nodes (v, G)
		v_orig[_copy[v]] = v;

	forall_edges (e, G)
		e_orig[e_copy[e]] = e;

	forall_edges (e, *this) {
		_in_chain[e] = _chain[original(e)].append(e);
	}

}

void GraphCopy::map_to_copy (node_array<node>& v_copy,
	edge_array<edge>& e_copy, const graph& G)
{
	node v, vc;
	edge e, ec;

	vc = first_node();
	forall_nodes (v, G) {
		v_copy[v] = vc;
		ec = _first_adjacent(vc);
		for (e = _first_adjacent(v); e != nil; e = _succ_adjacent(e,v)) {
			e_copy[e] = ec;
			ec = _succ_adjacent(ec,vc);
		}
		vc = succ_node(vc);
	}
}

edge GraphCopy::_first_adjacent (node v)
{
	edge e = First_Adj_Edge(v,0);

	if (e != nil && source(e) != v)
		e = _succ_adjacent(e,v);

	return e;
}

edge GraphCopy::_succ_adjacent (edge e, node v)
{
	do {
		e = Succ_Adj_Edge(e,v);
	} while (e != nil && source(e) != v);

	return e;
}

edge GraphCopy::split (edge e)
{
	edge e2;

	split_edge (e, e, e2);

	if (original(e) != nil) {
		_in_chain[e2] = _chain[original(e)].insert(e2,_in_chain[e],after);
		e_orig[e2] = original(e);
	}

	return e2;
}

//-------------------------------------------------------------------
//                          PlaneGraphCopy
//-------------------------------------------------------------------

struct MMDelNode
{
	MMDelNode() { }
	MMDelNode(edge _eo, edge _eor, node _vo, edge _eref) :
		eo(_eo), eo_rev(_eor), vo(_vo), e_ref(_eref) { }

	edge eo, eo_rev;
	node vo;
	edge e_ref;

	friend ostream& operator<< (ostream& os, const MMDelNode&)
	{ return os; }
	friend istream& operator>> (istream& is, MMDelNode&)
	{ return is; }
};

void PlaneGraphCopy::_init (const graph& G, list<edge>& E)
{
	make_directed();

	edge e;
	forall (e, E) {
		del_edge(chain(e).head());
		_chain[e].clear();
	}

	if (!Is_Planar_Map(*this)) {
		list<edge> R;
		make_map(R);
		make_planar_map();
	} else
		compute_faces();
}

PlaneGraphCopy::PlaneGraphCopy (const PlaneGraphCopy& PG) :
	GraphCopy(PG), S(0)
{
	if (g_orig != nil) compute_faces();
}

PlaneGraphCopy::PlaneGraphCopy(const graph& G) : GraphCopy(G), S(0)
{
	list<edge> empty;
	_init (G, empty);
}

PlaneGraphCopy::PlaneGraphCopy(const graph& G, list<leda_edge>& E) : GraphCopy (G), S(0)
{
	_init (G, E);
}

PlaneGraphCopy::~PlaneGraphCopy()
{
	delete S;
}


PlaneGraphCopy& PlaneGraphCopy::operator= (const PlaneGraphCopy& PG)
{
	GraphCopy::operator=(PG);
	if (g_orig != nil) compute_faces();
	return *this;
}

edge PlaneGraphCopy::split (edge e)
{
	edge r = reversal(e), e1 = split_map_edge(e);

	if ((e_orig[e1] = original(e)) != nil) {
		_in_chain[e1] = _chain[original(e)].insert(e1,_in_chain[e],after);
		//e_orig[e1] = original(e);
	}

	if ((e_orig[reversal(e)] = original(r)) != nil) {
		_in_chain[reversal(e)] =
			_chain[original(r)].insert(reversal(e),_in_chain[r],after);
		//e_orig[reversal(e)] = original(r);
	}

	return e1;
}

void PlaneGraphCopy::new_edge_chain (edge e, edge from, edge to,
	const list<edge>& crossings)
{
	edge e1, ed, ec;

	forall (ec, crossings) {
		e1 = split (ec);
		ed = split_face (from, e1);
		_in_chain[ed] = _chain[e_orig[ed] = e].append(ed);
		from = reversal(ec);
	}
	ed = split_face (from, to);
	_in_chain[ed] = _chain[e_orig[ed] = e].append(ed);
}


face PlaneGraphCopy::expand (node v)
{
	edge e;
	face f;

	if (degree(v) < 6)
		error_handler(1,"PlaneGraphCopy::expand: deg(v) < 3");

	forall_out_edges(e,v)
		split(e);

	forall_out_edges(e,v)
		split_face (reversal(e), 
                            face_cycle_succ(cyclic_adj_pred(e)));


	forall_out_edges(e,v) {
		if (original(e) != nil) _chain[original(e)].pop_front();
		if (original(reversal(e)) != nil)
			_chain[original(reversal(e))].pop_back();

		f = join_faces(e);
	}

	del_node(v);

	return f;
}

node PlaneGraphCopy::collapse (face f, node orig)
{
	edge e, e1, e2;
	list<edge> edges = adj_edges(f);
	list_item it = edges.first();

	e1 = edges[it]; it = edges.succ(it);
	e2 = edges[it]; it = edges.succ(it);
	e1 = split_map_edge(split_face(e1,e2));

	node v = source(e1);
	v_orig[v] = orig;
	if (orig != nil) _copy[orig] = v;

	for (; it != nil; it = edges.succ(it))
		e1 = split_face(e1,edges[it]);

	while (!edges.empty()) {
		e = edges.pop();

		extend_chain(cyclic_adj_pred(e),reversal(cyclic_adj_succ(e)));
		join_faces(e);
	}

	return v;
}

// adds the dummy edge e2 to the chain to which e1 belongs
void PlaneGraphCopy::extend_chain(edge e1, edge e2)
{
	edge e1r = reversal(e1), e2r = reversal(e2);
	edge e1o = original(e1), e1ro = original(e1r);

	if (e1o != nil) {
		if (target(e1) == source(e2))
			_in_chain[e2] = _chain[e1o].push_back(e2);
		else
			_in_chain[e2] = _chain[e1o].push_front(e2);
		e_orig[e2] = e1o;
	}

	if (e1ro != nil) {
		if (target(e1r) == source(e2r))
			_in_chain[e2r] = _chain[e1ro].push_back(e2r);
		else
			_in_chain[e2r] = _chain[e1ro].push_front(e2r);
		e_orig[e2r] = e1ro;
	}
}

face PlaneGraphCopy::max_face() const
{
	int max = 0;
	face f, f_max = nil;

	forall_faces (f, *this) {
		if (size(f) >= max) {
			max = size(f);
			f_max = f;
		}
	}
	return f_max;
}

edge PlaneGraphCopy::add_edge(edge orig, edge rev_orig, edge e, node w,
	int dir)
{
	edge e1, e2;

	e_orig[e1 = new_edge(e,w,          dir)] = orig;
	e_orig[e2 = new_edge(w,reversal(e),dir)] = rev_orig;

	if (orig)     _chain[orig]     = list<edge>(e1);
	if (rev_orig) _chain[rev_orig] = list<edge>(e2);
	set_reversal(e1,e2);

	return e1;
}

edge PlaneGraphCopy::add_edge(edge orig, edge rev_orig, node v, node w)
{
	edge e1, e2;

	e_orig[e1 = new_edge(v,w)] = orig;
	e_orig[e2 = new_edge(w,v)] = rev_orig;

	if (orig)     _chain[orig]     = list<edge>(e1);
	if (rev_orig) _chain[rev_orig] = list<edge>(e2);
	set_reversal(e1,e2);

	return e1;
}

void PlaneGraphCopy::remove_deg1_nodes(const node_array<bool> &mark)
{
	node v;

	delete S;
	S = new stack<MMDelNode>;

	// delete marked nodes; store originals and reference edge in S
	for(v = first_node(); v; v = succ_node(v)) {
		if (mark[v] || outdeg(v) == 0) continue;

		edge e, e_ref, e_ref_1;
		for (e_ref = first_adj_edge(v); e_ref && mark[target(e_ref)];
			e_ref = adj_succ(e_ref)) ;
		if (!e_ref) {
			forall_adj_edges(e,v) {
				S->push(MMDelNode(original(e),original(reversal(e)),
					original(target(e)),nil));
				del_node(target(e));
			}
			continue;
		}
		for (e_ref_1 = e_ref, e = cyclic_adj_succ(e_ref); e != e_ref_1;
			e = cyclic_adj_succ(e)) {
			if (mark[target(e)]) {
				S->push(MMDelNode(original(e),original(reversal(e)),
					original(target(e)),e_ref));
				del_node(target(e));
			} else
				e_ref = e;
		}
	}
	compute_faces();
}


void PlaneGraphCopy::restore_deg1_nodes(list<node> &deg1s)
{
	if (!S) return;

	while (!S->empty()) {
		node v;
		MMDelNode mmdn = S->pop();
		if (mmdn.e_ref)
			add_edge(mmdn.eo, mmdn.eo_rev, mmdn.e_ref, v = add_node(mmdn.vo));
		else
			add_edge(mmdn.eo, mmdn.eo_rev, copy(source(mmdn.eo)), v = add_node(mmdn.vo));
		deg1s.append(v);
	}

	delete S; S = 0;
}


//-------------------------------------------------------------------
//                           LeveledGraph
//-------------------------------------------------------------------

LeveledGraph::LeveledGraph() : graph(), _set_level(true), _max_level(-1),
	_nodes(list<node>())
{
	_level       .init(*this,0);
	_pos_in_nodes.init(*this,nil);
}


LeveledGraph::LeveledGraph(const LeveledGraph &LG) : graph(), _set_level(true),
	_nodes(list<node>())
{
	_init(LG);
}


void LeveledGraph::_init(const LeveledGraph &LG)
{
	_max_level = LG._max_level;
	_level       .init(*this,0);
	_pos_in_nodes.init(*this,nil);

	node v, v_LG;
	for (v = first_node(), v_LG = LG.first_node(); v != nil;
		v = succ_node(v), v_LG = LG.succ_node(v_LG))
	{
		_pos_in_nodes[v] = _nodes[_level[v] = LG._level[v_LG]].append(v);
	}
}


LeveledGraph& LeveledGraph::operator=(const LeveledGraph &LG)
{
	graph::operator=(LG);
	_nodes = map<int,list<node> >(list<node>());
	_init(LG);

	return *this;
}


node LeveledGraph::new_node (int l)
{
	_set_level = false;
	node v = new_node();
	_set_level = true;
	_pos_in_nodes[v] = _nodes[_level[v]=l].append(v);

	if (l > _max_level)
		_max_level = l;

	return v;
}


void LeveledGraph::post_new_node_handler(node v)
{
	if (_set_level) {
		_pos_in_nodes[v] = _nodes[_level[v]=0].append(v);
		if (_max_level < 0)
			_max_level = 0;
	}
}


void LeveledGraph::pre_del_node_handler(node v)
{
	_nodes[_level[v]].del_item(_pos_in_nodes[v]);
}


void LeveledGraph::level(node v, int l_new)
{
	int l_old = _level[v];

	if (l_new != l_old) {
		_nodes[l_old].del_item(_pos_in_nodes[v]);
		_pos_in_nodes[v] = _nodes[_level[v]=l_new].append(v);

		if (l_new > _max_level)
			_max_level = l_new;
	}
}
