//-------------------------------------------------------------------
//
//                        heuristics.c
//
//    author: Carsten Gutwenger
//            Michael Krger
//    date:   march 1998
//-------------------------------------------------------------------


#include <AGD/BarycenterHeuristic.h>
#include <AGD/MedianHeuristic.h>
#include <AGD/GreedyInsertHeuristic.h>
#include <AGD/GreedySwitchHeuristic.h>
#include <AGD/SplitHeuristic.h>
#include <AGD/impl/CrossingsMatrix.h>
#include <AGD/Hierarchy.h>


//-------------------------------------------------------------------
//                        BarycenterHeuristic
//-------------------------------------------------------------------

void BarycenterHeuristic::call (const graph& G, Layer& L)
{
	const Hierarchy& H = L.hierarchy();
	const double f = (double)L.neighbored_layer().length() / (double)L.length();
	node v, w;
	int i;

	for (i = 0; i < L.length(); i++) {
		v = L[i];
		long sumpos = 0L;
    
		forall (w, L.adj_nodes(v))
			sumpos += H.pos(w);

		weight[v] = L.adj_nodes(v).empty() ? f * (double)i :
			(double) sumpos /
			(double) L.adj_nodes(v).size();
	}
	
	L.quicksort(weight);
}


//-------------------------------------------------------------------
//                        MedianHeuristic
//-------------------------------------------------------------------

void MedianHeuristic::call (const graph& G, Layer& L)
{
	const Hierarchy& H = L.hierarchy();
	const Layer &LN = L.neighbored_layer();
	const double f = (double)LN.length() / (double)L.length();

	array<int> count(L.length()), odds(L.length());

	int i, vpos;
	node v;

	for (i = 0; i < L.length(); i++) {
		v = L[i];
		count [i] = (L.adj_nodes(v).size()+1) / 2;
		odds  [i] = 1;
		weight[v] = (count[i] == 0) ? f * (double)i : 0.;
	}

	for (i = 0; i < LN.length(); i++) {
		const list<node>& adj = LN.adj_nodes(LN[i]);
		forall (v,adj) {
			vpos = H.pos(v);
			if (--count[vpos] == 0)
				if (L.adj_nodes(v).length() & 1)
					weight[v] = i;
				else {
					weight[v] += (double)i / 2.;
					count[vpos] += odds[vpos]--;
				}
		}
	}

	L.quicksort(weight);
}


void GreedyInsertHeuristic::call( const graph &G, Layer &L ) {
  
  CrossingsMatrix crossings( L );
  register int index, i;
  register long prio;
 
  /* Initialisierung und Prioritaetszuweisung */
  for ( i = 0; i < L.length(); i++ ) {
    for ( prio = index = 0; index < L.length(); index++ )
      prio += crossings( i, index );
    /* stable quicksort: no need for unique prio */
    weight[ L[ i ] ] = (double) prio;
  }
  L.quicksort( weight );
}


void GreedySwitchHeuristic::call( const graph &G, Layer &L ) {

  CrossingsMatrix crossings( L );
  register int index;
  bool nolocalmin;
  
  do {
    nolocalmin = false;

    for ( index = 0; index < L.length() - 1; index++ )
      if ( crossings( index, index + 1 ) > crossings( index + 1, index ) ) {

	nolocalmin = true;
	
        L.swap( index, index + 1 );
        crossings.swap( index, index + 1 );
      }
  } while ( nolocalmin );
}

#if defined(_MSC_VER)
int compare (const list_item&, const list_item&);
#endif

void SplitHeuristic::call( const graph &G, Layer &L ) {

  CrossingsMatrix crossings( L );
  array<list_item> pivotpos( L.length() );
  
  _rek_call( L, crossings, pivotpos, 0, L.length() - 1 );
}


void SplitHeuristic::_rek_call( Layer &L, CrossingsMatrix &crossings,
				array<list_item> &pivotpos,
				int low, int high ) {

  const Hierarchy &H = L.hierarchy();
  static int pivot, oben, index;
  static list<node> L_nodes;
  static list_item litem;
  
  pivot = low;
  oben  = high;

  for ( index = pivot + 1; index <= oben; index++ )
    L_nodes.append( L[ index ] );
  
  index = pivot + 1;
  forall_items ( litem, L_nodes ) {
    if ( crossings( index, pivot ) < crossings( pivot, index ) )
      pivotpos[ low++ ] = litem;
    index++;
  }

  index = oben;
  forall_rev_items ( litem, L_nodes ) {
    if ( crossings( index, pivot ) >= crossings( pivot, index ) )
      pivotpos[ high-- ] = litem;
    index--;
  }

  pivotpos[ low ] = L_nodes.push( L[ pivot ] );

  for ( index = pivot; index <= oben; index++ ) {
    for ( litem = L_nodes.pred( pivotpos[ index ] );
	  litem != nil && H.pos( L_nodes.inf( litem ) ) >= index;
	  litem = L_nodes.pred( litem ) ) {
      const int
        pos_i = H.pos( L_nodes.inf( pivotpos[ index ] ) ),
        pos_j = H.pos( L_nodes.inf( litem ) );
	
      L.swap( pos_i, pos_j );
      crossings.swap( pos_i, pos_j );
    }
    L_nodes.del_item( pivotpos[ index ] );
  }

  if ( --low > pivot )
    _rek_call( L, crossings, pivotpos, pivot, low );
  if ( oben > ++high )
    _rek_call( L, crossings, pivotpos, high, oben );
}

//---------------------------------------------------------------------

CrossingsMatrix::CrossingsMatrix( const Layer &L ) {

  const Hierarchy &H = L.hierarchy();
  register int index1, index2;
  int pos_op_i, pos_op_j;
  node i, op_i, j, op_j;
  
  _crossings = new array2<int> ( L.length(), L.length() );

  for ( index1 = 0; index1 < L.length(); index1++ )
    for ( index2 = 0; index2 < L.length(); index2++ )
      (*_crossings)( index1, index2 ) = 0;

  for ( index1 = 0; index1 < L.length(); index1++ ) {
    i = L[ index1 ];
    
    forall ( op_i, L.adj_nodes( i ) ) {
      pos_op_i = H.pos( op_i );
      
      for ( index2 = index1 + 1; index2 < L.length(); index2++ ) {
	j = L[ index2 ];
	forall ( op_j, L.adj_nodes( j ) ) {
	  pos_op_j = H.pos( op_j );
	  (*_crossings)( index1, index2 ) += pos_op_i > pos_op_j;
	  (*_crossings)( index2, index1 ) += pos_op_j > pos_op_i;
	}
      }
    }
  }
}


void CrossingsMatrix::swap( int i, int j ) {
  
  register int k;

  for ( k = _crossings->low1(); k <= _crossings->high1(); k++ )
    leda_swap( (*_crossings)( k, i ), (*_crossings)( k, j ) );
  for ( k = _crossings->low2(); k <= _crossings->high2(); k++ )
    leda_swap( (*_crossings)( i, k ), (*_crossings)( j, k ) );
}
