//-------------------------------------------------------------------
//
//                        planarization.cpp
//
//
//    author: Carsten Gutwenger
//    date:   august 1997
//-------------------------------------------------------------------


#include <AGD/SubgraphPlanarizer.h>
#include <AGD/PlanarizationLayout.h>
#include <AGD/PlanarizationGridLayout.h>
#include <AGD/ShortestPathInserter.h>
#include <AGD/Quod.h>
#include <AGD/PlanarSubgraph.h>
#include <AGD/PlaneGraphCopy.h>
#include <AGD/GraphCopyLI.h>

#include <LEDA/node_set.h>

//-------------------------------------------------------------------
//                        PlanarizerModule
//-------------------------------------------------------------------

const int PlanarizerModule::in_graph = 1;
const int PlanarizerModule::out_graph = 1;

bool PlanarizerModule::check (const GraphCopy& G, string& message)
{
	return check_graph (in_graph, G, message);
}

void PlanarizerModule::call (const graph& G, PlaneGraphCopy& PG)
{
	_call (G, PG);
	PG.compute_faces();
}


//-------------------------------------------------------------------
//                           SubgraphModule
//-------------------------------------------------------------------

const int SubgraphModule::in_graph = 1;
const int SubgraphModule::out_graph = 1;

bool SubgraphModule::check (const graph& G, string& message)
{
	return check_graph (in_graph, G, message);
}

void SubgraphModule::call (const graph& G, list<edge>& L)
{
	L.clear();

	_call (G, L);

	_num_del_edges = L.length();
}

void SubgraphModule::call (graph& G)
{
	list<edge> L;
	edge e;

	_call (G, L);

	_num_del_edges = L.length();
	forall (e, L) G.del_edge(e);
}

void SubgraphModule::call (GraphCopy& G, list<edge>& L)
{
	L.clear();

	_call (G, L);

	_num_del_edges = L.length();

	list_item it;
	forall_items (it, L) {
		edge e = L[it];
		L[it] = G.original(e);
		G.del_edge(e);
	}
}


//-------------------------------------------------------------------
//                          SubgraphPlanarizer
//-------------------------------------------------------------------

SubgraphPlanarizer::SubgraphPlanarizer()
{
	add_precondition (key::simple);
	add_precondition (key::no_self_loops);

	add_post_rule (key::planar);
	add_post_rule (key::simple);
	add_post_rule (key::no_self_loops);

	// initialisiere subgraph
	array<KeySet> PRE(1,1), POST(0,1);

	PRE [1] << key::simple << key::no_self_loops
		<< key::no_reversals;
	POST[1] << key::planar << key::simple << key::no_self_loops
		<< key::no_reversals;
	subgraph.init (PRE,POST,PlanarSubgraph());

	// initialisiere inserter
	array<KeySet> PRE_I(1,2), POST_I(0,1);
	inserter.init (PRE_I,POST_I,ShortestPathInserter());
}

AGDModule *SubgraphPlanarizer::clone () const
{
	SubgraphPlanarizer *M = new SubgraphPlanarizer;

	M->set_subgraph (subgraph.get());
	// M->set_inserter ...

	return M;
}

void SubgraphPlanarizer::_call (const graph& G, PlaneGraphCopy& PG)
{
	list<edge> L;

	subgraph.get().call (G, L);
	PG = PlaneGraphCopy(G, L);
	inserter.get().call (PG, L);
}


//-------------------------------------------------------------------
//                         PlanarizationLayout
//-------------------------------------------------------------------

PlanarizationLayout::PlanarizationLayout() : LayoutModule()
{
	add_precondition (key::simple);
	add_precondition (key::no_self_loops);

	//
	// initialize used modules
	//

	array<KeySet> PRE(1,1), POST(0,1);

	PRE [1] << key::simple << key::no_self_loops << key::directed;
	POST[1] << key::planar << key::simple        << key::no_self_loops;
	planarizer.init (PRE,POST,SubgraphPlanarizer());

	PRE[1].clear(); POST[1].clear();

	PRE [1] << key::simple << key::no_self_loops << key::planar << key::four_graph << key::connected;
	POST[0] << key::planar_layout;
	POST[1] << key::no_crossings;
	planar_layouter.init (PRE,POST,Quod());
}

AGDModule *PlanarizationLayout::clone () const
{
	PlanarizationLayout *M = new PlanarizationLayout;

	M->set_planarizer (planarizer.get());
	M->set_planar_layouter (planar_layouter.get());

	return M;
}

void PlanarizationLayout::set_planar_layouter (const LayoutModule& M)
{
	if (planar_layouter.set(M)) {
		if (M.precondition(1).member(key::connected))
			add_precondition(key::connected);
		else
			rem_precondition(key::connected);
		if (M.precondition(1).member(key::four_graph))
			add_precondition(key::four_graph);
		else
			rem_precondition(key::four_graph);
	}
}

void PlanarizationLayout::_call (const graph& G, LayoutInterface& A)
{
	PlaneGraphCopy PG;

	planarizer.get().call (G, PG);

	GraphCopyLI B_PG(PG,A);

	planar_layouter.get().call (PG, nil, B_PG);
}


//-------------------------------------------------------------------
//                      PlanarizationGridLayout
//-------------------------------------------------------------------

PlanarizationGridLayout::PlanarizationGridLayout() : GridLayoutModule()
{
	add_precondition (key::simple);
	add_precondition (key::no_self_loops);

	//
	// initialize used modules
	//

	array<KeySet> PRE(1,1), POST(0,1);

	PRE [1] << key::simple << key::no_self_loops << key::directed;
	POST[1] << key::planar << key::simple        << key::no_self_loops;
	planarizer.init (PRE,POST,SubgraphPlanarizer());

	PRE[1].clear(); POST[1].clear();

	PRE [1] << key::simple << key::no_self_loops << key::planar << key::four_graph << key::connected;
	POST[0] << key::planar_layout;
	POST[1] << key::no_crossings;
	planar_layouter.init (PRE,POST,Quod());
}

AGDModule *PlanarizationGridLayout::clone () const
{
	PlanarizationGridLayout *M = new PlanarizationGridLayout;

	M->set_planarizer (planarizer.get());
	M->set_planar_layouter (planar_layouter.get());

	return M;
}

void PlanarizationGridLayout::set_planar_layouter (const GridLayoutModule& M)
{
	if (planar_layouter.set(M)) {
		if (M.precondition(1).member(key::connected))
			add_precondition(key::connected);
		else
			rem_precondition(key::connected);
		if (M.precondition(1).member(key::four_graph))
			add_precondition(key::four_graph);
		else
			rem_precondition(key::four_graph);
	}
}

void PlanarizationGridLayout::_call_grid (const graph& G, GridLayout &gl)
{
	PlaneGraphCopy PG;

	planarizer.get().call (G, PG);

	GridLayout gl_pg;

	planar_layouter.get().call_grid (PG, nil, gl_pg);
	transform_grid_layout (PG, gl, gl_pg);
	set_grid_bbox (planar_layouter.get().get_grid_area());
}


//-------------------------------------------------------------------
//                         ShortestPathInserter
//-------------------------------------------------------------------

void ShortestPathInserter::_call (PlaneGraphCopy& PG, list<edge>& e_insert)
{
	edge e;

	cout << "edges to insert: ";
	forall (e, e_insert)
		cout << "(" << index(source(e)) << "," << index(target(e)) << ") ";
	cout << endl;

	//construct dual graph
	//--------------------

	GRAPH<int,edge> LG_dual; //edge-parameter is the corresponding edge in the primal
	edge_map<node> left_face(PG); //every edge in the primal graph knows its
                                      //left face
	face f;
	//construct nodes
	forall_faces(f,PG) {
		node dual_node=LG_dual.new_node(0); //for any face there is a node in the dual
    
		forall_face_edges(e,f) {
			left_face[e]=dual_node;  //edge in primal has pointer to that node
		}
	}

	//insert edges in the dual graph
	forall_edges(e,PG) 
		LG_dual.new_edge(left_face[e],left_face[PG.reversal(e)],e); 
  

	while (!e_insert.empty()) {

		edge current_edge = e_insert.pop();
		node src = PG.copy(source(current_edge)),
			 tgt = PG.copy(target(current_edge));


		//find possible source nodes for shortest path in LG_dual
		//-------------------------------------------------------
		//node source=source_nodes.pop();
		list<node> shortest_sources;
		forall_out_edges(e,src) {
			if (left_face[e]!=left_face[PG.reversal(e)]) 
				shortest_sources.append(left_face[e]);

			else if (shortest_sources.empty()) shortest_sources.append(left_face[e]);
		}
     
		//find possible target nodes for shortest path in LG_dual
		//-------------------------------------------------------
		//node target=target_nodes.pop();
		list<node> shortest_targets;
		forall_out_edges(e,tgt) {
			if (left_face[e]!=left_face[PG.reversal(e)])
				shortest_targets.append(left_face[e]);

			else if (shortest_targets.empty()) shortest_targets.append(left_face[e]);
		}
  
		//compute shortest path between shortest_sources and shortest_targets
		//-------------------------------------------------------------------
		list<node> unscanned;
		node_array<bool> visited(LG_dual,false);
    
		node_array<edge> pred(LG_dual,nil);
		face common_face=nil;
		node target_found;
		bool path_found=false;

		node i;
		node_set T(LG_dual);
		forall(i,shortest_targets) {
			T.insert(i);
		}

		forall(i,shortest_sources) {
			visited[i]=true;
			unscanned.append(i);
			if (T.member(i)) {
				edge e_help=LG_dual.first_adj_edge(i);
				e_help=LG_dual[e_help];
				common_face=PG.face_of(e_help);
				path_found=true;
			}
		}

		while (!unscanned.empty()&&!path_found) {
			i=unscanned.pop();

			edge e;
			forall_out_edges(e,i) {
				node j=LG_dual.opposite(i,e);
				if (!visited[j]) {
					pred[j]=e;
					visited[j]=true;
					unscanned.append(j);
				}

				if (T.member(j)) {
					target_found=j;
					path_found=true;
				}
			}
		}
if (!path_found) cout << "no path found!";

		list<edge> shortest_path;
		while (common_face==nil && pred[target_found]!=nil) {
			shortest_path.push(pred[target_found]);
			target_found=LG_dual.opposite(target_found,pred[target_found]);
		}   
    
		cout << "found shortest path of length: " << shortest_path.size() << endl;

		//for any crossing put a dummy node
		//---------------------------------
		edge e_first,e_last;
		list<edge> crossed;

		if (shortest_path.empty()) {
			edge e_help;
			forall_face_edges(e_help,common_face) {
				if (PG.source(e_help)==src) e_first=e_help;
				if (PG.source(e_help)==tgt) e_last=e_help;
			}
		}

		else {
			edge e_help=LG_dual[shortest_path.head()];
			face f=PG.face_of(e_help);

			forall_face_edges(e_help,f) {
				if (PG.source(e_help)==src) e_first=e_help;
			}
      
			e_help=PG.reversal(LG_dual[shortest_path.tail()]);
			f=PG.face_of(e_help);

			forall_face_edges(e_help,f) {
				if (PG.source(e_help)==tgt) e_last=e_help;
			}
		}

		edge e;
		forall(e,shortest_path) {
			crossed.append(LG_dual[e]); 
		}

		PG.new_edge_chain (current_edge, e_first, e_last, crossed);
/*
    list<edge> inserted;
    while (!crossed.empty()) {
      e=crossed.pop();

      int the_edge_id;
      if (PG.inf(e)==-1) {
        the_edge_id=PG.inf(PG.reversal(e));
      }

      node n=left_face[PG.reversal(e)];
      edge new_edge=PG.split_map_edge(e);
      e_first=PG.reversal(e);
      
      left_face[PG.reversal(e)]=n;
      left_face[PG.reversal(new_edge)]=n;

      if (PG.inf(e)!=-1) {
	PG[e]=PG.inf(e);
	PG[PG.reversal(new_edge)]=-1;
        PG[PG.reversal(e)]=-1;
      }

      else {
	PG[new_edge]=-1;
	PG[PG.reversal(new_edge)]=the_edge_id;
	PG[PG.reversal(e)]=the_edge_id;
      }

      node dummy=PG.source(new_edge);
      PG[dummy]=-1;
      dummy_nodes.append(dummy);

      new_edge=PG.split_face(e_first,e);
      PG[new_edge]=edge_id;
      PG[PG.reversal(new_edge)]=-1;
      inserted.append(new_edge);
    }

    e=PG.split_face(e_first,e_last);
    PG[e]=edge_id;
    PG[PG.reversal(e)]=-1;
    inserted.append(e);
*/

		while (!shortest_path.empty())
			LG_dual.del_node(source(shortest_path.pop()));
		LG_dual.del_node(left_face[e_last]);

		forall (e, PG.chain(current_edge)) {
			edge e1;
			face f_l = PG.face_of(e),
				 f_r = PG.face_of(PG.reversal(e));
			node v_l = LG_dual.new_node(0),
				 v_r = LG_dual.new_node(0);

			forall_face_edges (e1, f_l)
				left_face[e1] = v_l;
			forall_face_edges (e1, f_r)
				left_face[e1] = v_r;
		}

		forall (e, PG.chain(current_edge)) {
			edge e1;
			face f_l = PG.face_of(e),
				 f_r = PG.face_of(PG.reversal(e));
			node v_l = left_face[PG.first_face_edge(f_l)],
				 v_r = left_face[PG.first_face_edge(f_r)];

			forall_face_edges (e1, f_l) {
				node v1 = left_face[PG.reversal(e1)];
				LG_dual.new_edge(v_l,v1,e1);
				if (v1 != v_r) LG_dual.new_edge(v1,v_l,PG.reversal(e1));
			}
			forall_face_edges (e1, f_r) {
				node v1 = left_face[PG.reversal(e1)];
				LG_dual.new_edge(v_r,v1,e1);
				if (v1 != v_l) LG_dual.new_edge(v1,v_r,PG.reversal(e1));
			}
		}
		//Make_Bidirected(LG_dual);
		if (!Is_Connected(LG_dual)) cout << "Dual is not connected!\n";
		if (!Is_Planar(LG_dual)) cout << "Dual is not planar!\n";

/*
		list<edge> inserted = PG.chain(current_edge);
		//update dual graph
		while (!inserted.empty()) {
			LG_dual.del_node(LG_dual.source(shortest_path.pop()));
			node n1=LG_dual.new_node(0);
			node n2=LG_dual.new_node(0);

			edge e=inserted.pop();
 
			face f=PG.face_of(e);

			edge e_help;
			forall_face_edges(e_help,f) {
				left_face[e_help]=n1;
			}

			forall_face_edges(e_help,f) {
				LG_dual.new_edge(n1,left_face[PG.reversal(e_help)],e_help);

				if (left_face[PG.reversal(e_help)]!=n1) {
					LG_dual.new_edge(left_face[PG.reversal(e_help)],n1,PG.reversal(e_help));
				}
			}

			f=PG.face_of(PG.reversal(e));

			forall_face_edges(e_help,f) {
				left_face[e_help]=n2;
			}

			forall_face_edges(e_help,f) {
				if (left_face[PG.reversal(e_help)]!=n1) {
					LG_dual.new_edge(n2,left_face[PG.reversal(e_help)],e_help);

					if (left_face[PG.reversal(e_help)]!=n2) {
						LG_dual.new_edge(left_face[PG.reversal(e_help)],n2,PG.reversal(e_help));
					}
				}
			}
		}*/

	}

}
