/************************************************************************

         Filename   :   planarize.cc

         Version    :   09.1994

         Author     :   Sebastian Leipert

         Language   :   C++


 File containing procedures for controlling the planarization
 algorithm and for executing the vertex addition step.

************************************************************************/ 


#include <AGD/impl/MaxPlanSub/proto.h>

     
extern  int      verticecount;
extern  int      edgecount;
extern  int      directed;
extern  int      total_num_del_edges;
extern  int      node_count;
extern  pq_node  *root;

extern  int      _printDuringReduction;
extern  int      _printPQTreeIterative;
extern  int      _iterationNumber;
extern  char     _debugInformation[INFOSIZE];


/************************************************************************
                          planar
************************************************************************/ 
              
void planar(
            adjlists *block,
            int sink,
            adjlists *deleted_edges
           )

  // Planarizes a biconnected component of a block.
  // This procedure controlls the vertex-addition algorithm.
  // In each step it calls up the necessary procedures for computing
  // the [w,h,a]-numbering, reducing the maximal pertinent sequence
  // and applying the vertex addition step. All deleted edges
  // are gathered in the object deleted_edges of the class adjlists.

{
  stack<int>           *new_leaves;        // leaves that have to be added to
                                           // the PQ-tree during the vertex
                                           // addition step
  stack<int>           *pertinent_leaves;  // pertinent leaves
  stack<pq_node*>      *pert_nodes = new stack<pq_node*>;
                                           // pertinent nodes
  stack<pq_node*>      *killed_nodes = new stack<pq_node*>;
                                           // saves killed nodes for a
                                           // valid cleanup until the end
  leaves               *leave_ptr = new leaves(sink);
                                           // class containing all leaves
                                           // of the actual PQ-tree
  pq_node              *pert_root;         // root of the pertinent subtree
  

  node_count = 0;
  root = new pq_node(1,0,node_count++,P_NODE,EMPTY);
  pert_root = root;
  new_leaves = get_higher_neighbors(block,1);
                                // Copy new leaves into the PQ-tree.
  copy_leaves_into_tree(root,new_leaves,leave_ptr,1);
  delete new_leaves;            // Clean up.
  for (int i = 2; i <= sink; i++)
                                // Get the pertinent leaves of the actual step.
  {

     if (i == _iterationNumber)
        _printDuringReduction = TRUE;
     else
        _printDuringReduction = FALSE;

     pertinent_leaves = get_lower_neighbors(block,i);

     if (_printPQTreeIterative)
     {
        FILE      *output;
        output = fopen(_debugInformation,"a");
        fprintf(output,"\n\n Leafnummer: %d Anzahl: %d\n",i,pertinent_leaves->count());
        fclose(output);
     }
                                // Bubble up the tree to find pertinent nodes,
                                // update parent pointers and count for ever 
                                // pertinent node the number of pertinent
                                // children.
     bubble(i,sink,pertinent_leaves,leave_ptr,pert_nodes);

     char _number[INFOSIZE];
     sprintf(_number,"%d",i-1);
     if (_printPQTreeIterative)
        print_out_pq_tree("PQTREE",_number,i-1);  
     fflush(stdout);

                                // Comput the [w,h,a]-numbers.
                                // Delete edges to gain reducabillity.
     compute1(&pert_root,i,sink,pertinent_leaves,leave_ptr,deleted_edges);

                                // Apply the pattern template algorithm
                                // of Booth and Lueker in order to reduce
                                // the maximal pertinent sequence.
     reduce(&pert_root,i,sink, pertinent_leaves,leave_ptr,pert_nodes);

     delete pertinent_leaves;   // Empty stack with pertinent leaves.

                                // Get new leaves.
     new_leaves = get_higher_neighbors(block,i);

                                // Do the vertex addition step.
     if (pert_root->label == FULL)
                                // Replace the root of the pertinent
                                // subtree by a new P-node.
        replace_P_node(i,pert_root,leave_ptr,new_leaves);

     else
                                // Replace the sequence of full children of
                                // the root of the pertinent subtree
                                // by a new P-node.
        replace_Q_node(i,pert_root,leave_ptr,new_leaves);

     check_pert_nodes(pert_nodes,killed_nodes);

     delete new_leaves;
  }// for

  pq_node   *helpptr = NULL;
  while (!killed_nodes->stack_empty())
  {
     helpptr = killed_nodes->pop();
     delete helpptr;
  }// while
  if (root != NULL)
     delete root;
  delete killed_nodes;
  delete pert_nodes;
  delete leave_ptr;
}// planar



/************************************************************************
                          get_higher_neighbors
************************************************************************/ 

stack<int>* get_higher_neighbors(
                                 adjlists *block,
                                 int node
                                )

  // Computes all neighbors of node with a larger st-number
  // and saves them in a stack.

{
  int          helpnode;
  stack<int>   *helplist = new stack<int>;

  (*block).start_at_first_node(node);
  while (!(*block).end_of_list(node))
  {
     helpnode = (*block).read_next_node(node);
     if (helpnode > node)
        helplist->push(helpnode);
  }// while
  return helplist;
}// get_higher_neighbors



/************************************************************************
                          get_lower_neighbors
************************************************************************/ 

stack<int>* get_lower_neighbors(
                                adjlists *block,
                                int node
                               )

  // Computes all neighbors of node with a lower st-number
  // and saves them in a stack.
{
  int          helpnode;
  stack<int>   *helplist = new stack<int>;

  (*block).start_at_first_node(node);
  while (!(*block).end_of_list(node))
  {
     helpnode = (*block).read_next_node(node);
     if (helpnode < node)
        helplist->push(helpnode);
  }// while
  return helplist;
}// get_lower_neighbors
        


/************************************************************************
                          replace_Q_node
************************************************************************/ 

void replace_Q_node(
                    int node,
                    pq_node *pert_root,
                    leaves *leave_ptr,
                    stack<int> *new_leaves
                   )

   // Replaces the full children of the root of the pertinent subtree
   // if the root is partial. Observe that the root then is a Q-node.
   // Furthermore it contains more than one full child, otherwise this
   // full child would be the root of the pertinent subtree.

{
   int      left_found = FALSE;
   int      right_found = FALSE;
   pq_node  *helpptr;
   pq_node  *left_full = NULL;     // leftmost child of the full children
   pq_node  *right_full = NULL;    // rightmost child of the full children
   pq_node  *left_empty = NULL;    // empty sibling of left_full, NULL if
                                   // pert_root is of type H
   pq_node  *right_empty = NULL;   // empty sibling of right_full
   pq_node  *new_P_node = NULL;
   pq_node  *check_sib = NULL;   

   pert_root->child_count = pert_root->child_count + 1 -
                            pert_root->full_children->count();
   pert_root->full_children->start_at_bottom();
   while ((!left_found || !right_found) &&
           !pert_root->full_children->read_last())
                                   // Search for start and end of the
                                   // pertinent sequence.
   {
      helpptr = pert_root->full_children->read_next();
      helpptr->imm_siblings->start_at_bottom();
      
      if (!left_found)
                                   // Search for the left end first.
      {
         if (helpptr->imm_siblings->count() == 1)
                                   // We have a type H Q-node, the pertinent
                                   // sequence is on one side of the Q-node.
         {
            left_full = helpptr;
            left_empty = NULL;
            left_found = TRUE;
         }// if
         else
                                   // Helpptr must have 2 siblings,
                                   // at least one of them must be full.
         {
            helpptr->imm_siblings->start_at_bottom();
            while (!helpptr->imm_siblings->read_last())
            {
               check_sib = helpptr->imm_siblings->read_next();
               if (check_sib->label == EMPTY)
                                   // Found the end of the sequence on the
                                   // left side.
               {
                  left_full = helpptr;
                  left_empty = check_sib;
                  left_found = TRUE;
               }// if
            }// while
         }// else
      }// if

      else
                                   // Left end was already found.
                                   // Search for right end.
      {
         if (helpptr->imm_siblings->count() == 1)
                                   // We have a type H Q-node, the pertinent
                                   // sequence is on one side of the Q-node.
         {
            right_full = helpptr;
            right_empty = NULL;
            right_found = TRUE;
         }// if
         else
                                   // Helpptr must have 2 siblings,
                                   // at least one of them must be full.
         {
            helpptr->imm_siblings->start_at_bottom();
            while (!helpptr->imm_siblings->read_last())
            {
               check_sib = helpptr->imm_siblings->read_next();
               if (check_sib->label == EMPTY)
                                   // Found the end of the sequence on the
                                   // right side.
               {
                  right_full = helpptr;
                  right_empty = check_sib;
                  right_found = TRUE;
               }// if
            }// while
         }// else
      }// else
   }// while


                                    // Now replace the full children by
                                    // new_P_node.
   new_P_node = new pq_node(node,0,node_count++,P_NODE,EMPTY);
   replace_P_node(node,new_P_node,leave_ptr,new_leaves);
   new_P_node->parent = pert_root;
   if (left_empty == NULL)
   {
      pert_root->endmost_children->search_and_change(left_full,new_P_node);
      right_empty->imm_siblings->search_and_change(right_full,new_P_node);
      new_P_node->imm_siblings->push(right_empty);
   }// if
   else if (right_empty == NULL)
   {
      pert_root->endmost_children->search_and_change(right_full,new_P_node);
      left_empty->imm_siblings->search_and_change(left_full,new_P_node);
      new_P_node->imm_siblings->push(left_empty);
   }// else if
   else
   {
      right_empty->imm_siblings->search_and_change(right_full,new_P_node);
      left_empty->imm_siblings->search_and_change(left_full,new_P_node);
      new_P_node->imm_siblings->push(left_empty);
      new_P_node->imm_siblings->push(right_empty);
   }// else

}// replace_Q_node



/************************************************************************
                          replace_P_node
************************************************************************/ 

void replace_P_node(
                    int node,
                    pq_node *pert_root,
                    leaves *leave_ptr,
                    stack<int> *new_leaves
                   )
                   
  // Replaces the root of the pertinent subtree by a P-node
  // with the incoming edges of the added vertex, if the root
  // is a P-node.
{
   int      helpnode;
   
   

   if (new_leaves->count() == 1)
                                     // The root becomes a leaf.
   {
      helpnode = new_leaves->pop();
      pert_root->typ = LEAF;
      pert_root->label = EMPTY;
      pert_root->key = node;
      pert_root->leafnum = helpnode;
      pert_root->child_count  = 0;
      leave_ptr->insert_ptr(node,helpnode,pert_root);
   }// if
   else if (new_leaves->count() > 1)
                                     // The root of the subtree becomes 
                                     // a P-node with the newly introduced
                                     // leaves as children.
   {
      pert_root->typ = P_NODE;
      pert_root->label = EMPTY;
      pert_root->child_count = 0;
      copy_leaves_into_tree(pert_root,new_leaves,leave_ptr,node);
   }// else if


}// replace_P_node



/************************************************************************
                          copy_leaves_into_tree
************************************************************************/

void copy_leaves_into_tree(
                           pq_node *tree_node,
                           stack<int> *new_leaves,
                           leaves *leave_ptr,
                           int node
                          )

   // Copies all new_leaves as children to the tree-node.
{
   int      helpnode;
   pq_node  *oldson = NULL;
   pq_node  *firstson = NULL;
   pq_node  *aktual_son = NULL;


   if (!new_leaves->stack_empty())
   {
      helpnode = new_leaves->pop();
      firstson = new pq_node(node,helpnode,node_count++,LEAF,EMPTY);
      firstson->parent = tree_node;
      tree_node->child_count++;
      leave_ptr->insert_ptr(node,helpnode,firstson);
      oldson = firstson;

      while (!new_leaves->stack_empty())
      {
         helpnode = new_leaves->pop();
         pq_node *helpptr = new pq_node(node,helpnode,node_count++,LEAF,EMPTY);
         aktual_son = helpptr;
         aktual_son->parent = tree_node;
         tree_node->child_count++;
         leave_ptr->insert_ptr(node,helpnode,aktual_son);
         oldson->sib_next = aktual_son;
         aktual_son->sib_pre = oldson;
         oldson = aktual_son;
      }// while

      firstson->sib_pre = oldson;
      oldson->sib_next = firstson;
      tree_node->child_sib_list = firstson;
      firstson->sib_list_father = tree_node;
   }// if
}// copy_leaves_into_tree
      


/************************************************************************
                          get_father
************************************************************************/ 

pq_node* get_father(
                    pq_node *node_ptr
                   )

  // Gets the father of a node.
{
  pq_node  *son = node_ptr;
 
   if (son->parent == NULL)
      return NULL;
   else if (son->parent->label != KILLED)
      return son->parent;
   else
   {
      while (son->parent->label == KILLED)
         son = son->parent;
      return son->parent;
   }// else
}// get_father



/************************************************************************
                          check_pert_nodes
************************************************************************/ 

void check_pert_nodes(
                      stack<pq_node*> *pert_nodes,
                      stack<pq_node*> *killed_nodes
                     )

   // Checks all nodes that have been pertinent during the last
   // reduction step.
{
   pq_node  *helpptr;

   while (!pert_nodes->stack_empty())
   {
      helpptr = pert_nodes->pop();
      switch (helpptr->label)

      {
         case TO_BE_DELETED:
                      delete helpptr;
                      break;

         case FULL:
                      if (helpptr == root)
                         root = NULL;
                      delete helpptr;
                      break;

         case KILLED:
                      helpptr->pert_child_count = 0;
                      helpptr->visited_child_count = 0;
                      helpptr->not_visited_child_count = 0;
                      helpptr->pert_leaf_count = 0;
                      helpptr->full_children->empty_stack();
                      helpptr->partial_children->empty_stack();
                      killed_nodes->push(helpptr);
                      break;

         default:
                      helpptr->label = EMPTY;
                      helpptr->pert_child_count = 0;
                      helpptr->visited_child_count = 0;
                      helpptr->not_visited_child_count = 0;
                      helpptr->pert_leaf_count = 0;
                      helpptr->full_children->empty_stack();
                      helpptr->partial_children->empty_stack();
                      break;
      }// switch
   }// while
}// check_pert_nodes


  
/************************************************************************
                          bubble
************************************************************************/ 

void bubble(
            int i,
            int,
            stack<int> *pertinent_leaves,
            leaves *leave_ptr,
            stack<pq_node*> *pert_nodes
           )

   // Bubbels up from the pertinent leaves to find all pertinent nodes.
   // Computes at the same time the number of pertinent children of
   // every pertinent node. Every pertinent node is stored in a stack
   // pert_nodes in order to do a valid cleanup of the pertinent nodes
   // after applying the reduction step.

{
   pq_node   *node_ptr;
   pq_node   *father;
   int       helpnode;

   queue<pq_node*> *Q = new queue<pq_node*>;
   
   pertinent_leaves->start_at_bottom();
   while (!pertinent_leaves->read_last())
                                   // Put the pertinent leaves into a Queue Q.
   {
      helpnode = pertinent_leaves->read_next();
      node_ptr = leave_ptr->get_ptr(helpnode,i);
                                   // Observe that we reduce vertex number i
                                   // so all pertinent leaves are numbered i.
      node_ptr->not_visited_child_count = 1;
      Q->enqueue(node_ptr);
      pert_nodes->push(node_ptr);   
   }// while
   while (!Q->queue_empty())
                                  // while Q not empty
   {
                                 // Pop out first node out of Q.
      node_ptr = Q->dequeue();
      father = get_father(node_ptr);
      node_ptr->parent = father;
      if (node_ptr != root)
                                  // If not root of the PQ-tree,
                                  // it has a father.
      {
         if (node_ptr->parent->pert_child_count == 0)
                                  // Father was not visited yet, so put
                                  // father on to Q.
         {
            Q->enqueue(node_ptr->parent);
            pert_nodes->push(node_ptr->parent);
         }// if
         node_ptr->parent->pert_child_count++;
                                  // Update number of pertinent children of
                                  // father.
         node_ptr->parent->not_visited_child_count++;
      }// if
   }// while

   delete Q;
}// bubble
   
      



